/*
  Copyright 2004 Michalis Kamburelis.

  This file is part of "Jamy & Nory".

  "Jamy & Nory" is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  "Jamy & Nory" is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with "Jamy & Nory"; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package jamy_i_nory.common;

import java.net.*;
import java.io.*;
import java.nio.*;
import java.nio.channels.*;

/** Ta klasa opakowuje SocketChannel (razem z akompaniujcym mu Socket).
   Obie kocwki poczenia powinny uywac tej klasy do komunikacji
   i wtedy mog si wymienia caymi Stringami, tzn. ta klasa pozwala
   na zapis i nieblokujcy odczyt stringw. Przy czym to jak te stringi
   s kodowane we waciwym kanale (jako bajty) jest spraw wewntrzn
   implementacji tej klasy - obie komunikujce si strony powinny tylko
   uywa tej samej klasy eby wszystko dziaao.  
   
   <p>W razie bdw sieci metody tego obiektu mog wypisywa rne
   informacje na System.err. 
   
   <p>Ta klasa nie jest zwizana z mechanik naszej gry (rzeczami
   w pakiecie {@link jamy_i_nory.game}). Jest to po prostu narzdzie
   do komunikacji przez sie uywane zarwno przez serwer jak i przez
   klienta. 
   
   <p>Stringi s wysyane i odbierane w taki sposb aby wszystko dziaao
   nawet pomidzy komputerami ktre rnie koduj pl ogonki i maj rn
   kolejno bajtw w sowie (endianess). */
public class StringSocket {

  final private SocketChannel socketChannel;
  final private Socket socket;
  private boolean fIsConnected = true;  
   
  /** Tworzy obiekt StringSocket uywajcy podanego aSocketChannel.
     Oznacza to e wszystkie odczyty / zapisy do StringSocket
     bd odpowiednio realizowane na danym aSocketChannel. 
     Po przekazaniu aSocketChannel do konstruktora teg obiektu nie moesz juz 
     w programie uywa aSocketChannel (ani jego socketa, tzn. 
     aSocketChannel.socket()) (innymi sowy, moesz uywa aSocketChannel 
     tylko za porednictwem utworzonego obiektu StringSocket). 
     
     Requires already connected socket, such as returned by 
     ServerSocketChannel.accept(). 
     
     Throws IOException if this socketChannel cannot be set 
     to non-blocking mode. */
  public StringSocket(SocketChannel aSocketChannel)
  throws IOException
  {
    socketChannel = aSocketChannel;
    socket = socketChannel.socket();
    
    socketChannel.configureBlocking(false);
    fIsConnected = true;        
  }
  
  /** Zamyka opakowywany socketChannel tym samym koczc poczenie. 
     Po wywoaniu tej metody nie moesz ju uywa tego obiektu. 
     
     Ta metoda nie jest wywoywana przez adn inn metod tego obiektu,
     np. jeeli zajdzie bd przy odczycie z gniazda podczas {@link #read}
     obiekt uzna e poczenie jest przerwane i {@link #isConnected}
     zacznie zwraca false. Ale cigle bdziesz musia wywoa {@link #close}
     samemu. */
  public void close()
  {
    try {
      socket.close();
    } catch (IOException e) {
      /* Bledy przy zamykaniu gniazda ignorujemy; w tym momencie oznaczaja
         one ze jest jakis problem z siecia z ktorym i tak nie umiemy sobie
         poradzic z wnetrza naszego programu. */
      System.err.println("Error while closing socket from " +this +". Ignoring.");
    } 
  }
  
  /** Proste testowe toString, zwraca port i adres skd przyszed klient. */
  public String toString()      
  {
    return socket.getInetAddress() +"[port:" +socket.getPort() +"]";
  }  
  
  /** Data read from sokect that does not constitute a whole message yet. 
     Used only in read(). */
  private byte[] pendingData = new byte[0];
  
  /** Tablica a musi mie dugo >= 2. Odczyta pierwsze dwa bajty tej tablicy
     tak jakby new DataInputStream(new ByteArrayInputStream(newPendingData)).
     readUnsignedShort() (ale efektywniej, nie tworzc po drodze tych
     obiektw). */
  private int byteArrayReadUnsignedShort(byte[] a)
  {
    return (((a[0] & 0xff) << 8) | (a[1] & 0xff));
  }
  
  /** Returns next String. Does not ever block - if whole message is not 
     yet available, returns null. If not isConnected, returns null too. */
  public String read()
  {
    /* Cz 1: czytaj z socketChannel, by moe zmie (dopisz co do)
       pendingData, by moe ustaw fIsConnected na false. NIE wywouj std
       return. */
    try {
      /* Podana niej wielkoc bufora nie musi byc nie wiadomo jak dua.
         Najwyej wiadomoc nie bdzie odczytana za jednym zamachem w jednym
         read() - to i tak byoby moliwe bo nic nie gwarantuje e 
         socketChannel.read() jest w stanie zwrci od razu co co reprezentuje
         cae message, by moe odebrano dopiero cz wiadomoci. */
      ByteBuffer bb = ByteBuffer.allocate(100);
      int readCount = socketChannel.read(bb);
      if (readCount == -1)
        fIsConnected = false; else
      if (readCount > 0) 
      {
        ByteArrayOutputStream newPendingData = new ByteArrayOutputStream
          (pendingData.length + readCount);
        newPendingData.write(pendingData, 0, pendingData.length);
        newPendingData.write(bb.array(), 0, readCount);
        pendingData = newPendingData.toByteArray();
      }
    } catch (IOException e) {
      System.err.println("Error while reading from " +this +": " +e);
      fIsConnected = false;
    }  
    
    /* Cz 2: sprawd czy mamy ju pene message w pendingData. 
       Robimy to nawet gdy ostatnie readCount <= 0, bo by moe zostaa nam
       w pendingData jaka caa message (co jest moliwe jeli np. w poprzednim
       read odczytalimy tak due pendingData e zawierao od razu dwie cae
       wiadomoci).
       
       Najpierw sprawdz czy mamy zapisane pierwsze 2 bajty - dugo. 
       Jeli tak to odczytaj dugo (koniecznie przy pomocy dokadnie 
       takiej metody jak DataInputStream.readUnsignedShort, bo wanie 
       w takim formacie zapisuje dugo DataOutputStream.writeUTF). */
    if (pendingData.length >= 2)
    {
      int dataNeededLength = byteArrayReadUnsignedShort(pendingData) + 2;
      if (pendingData.length >= dataNeededLength)
      {
        String result;
        try {
          result = new DataInputStream(
            new ByteArrayInputStream(pendingData)).readUTF();
        } catch (IOException e) {
          throw new Utils.InternalError("StringSocket.read", e);
        }
        /* wytnij pocztkowe dataNeededLength bajtw z pendingData */
        byte[] newPendingData = new byte[pendingData.length - dataNeededLength];
        System.arraycopy(pendingData, dataNeededLength, newPendingData, 0, 
          newPendingData.length);
        pendingData = newPendingData;
        return result;
      }          
    }
    return null;
  }

  /** If not isConnected, does not do anything (works like NOP). */
  public void write(String value)
  {
    if (!isConnected()) return;
    
    try {
      /* Zamie String na UTFa uywajc DataOutput.writeUTF.
         Aby taki zapis dopisa do socketChannel uywamy ByteArrayOutputStream
         i byte[], pniej byte[] wkadamy do ByteBuffer i ju moemy zapisa
         do socketChannel. Ale sednem sprawy jest eby uy
         DataOutput.writeUTF, to nam zapewnia kodowanie stringw bez straty
         UTFa i od razu dwubajtowy marker na pocztku stringa oznaczajcy
         wielkoc danych. */
      ByteArrayOutputStream byteArrayStream = new ByteArrayOutputStream(); 
      DataOutputStream dataOutput = new DataOutputStream(byteArrayStream);
      dataOutput.writeUTF(value);      
      ByteBuffer byteBuf = ByteBuffer.wrap(byteArrayStream.toByteArray());
      while (byteBuf.hasRemaining())
        socketChannel.write(byteBuf);
    } catch (IOException e) {
      System.err.println("Error while writing to " +this +": " +e);
      fIsConnected = false;
    }
  }

  /** Initially true. Will be false when the client will disconnect (cleanly,
     i.e. closing his socket, or not cleanly). */ 
  public boolean isConnected() 
  { 
    return fIsConnected; 
  }  
}

/* 
 * $Log: StringSocket.java,v $
 * Revision 1.9  2004/02/19 07:32:50  michalis
 * * small docs corrections
 * * sekcja "Przed uruchomieniem" skoczona
 * * some improvements to jar generating (one step closer to making client Applet)
 *
 * Revision 1.8  2004/02/09 23:08:50  michalis
 * + GPL license clause i copyrighty (moje i gdzieniegdzie Michaa, sorry Rafa
 *   ale naprawd nic nie zostao) do wszystkich *.java
 *
 * Revision 1.7  2004/02/05 20:05:26  michalis
 * * Zrobiem eby w logach CVSa zapisanych w plikach byo wszdzie TO-DO
 *   z mylnikiem w rodku. Wprawdzie nieznacznie rozsynchronizowuje to
 *   zapisane w plikach logi z wynikiem 'cvs log' ale za to grep TO-DO
 *   (gdzie w wyrazie TO-DO naley usun mylnik) dziaa jak trzeba.
 *
 * Revision 1.6  2004/02/01 00:56:58  michalis
 * * Being.dead very small changes
 * * StringSocket small write improvement
 * + Server.Server processes messages from clients using UserCommand.
 *   Dziaa ! Merge JamySingle + Client zakoczony - mamy zrobione
 *   co co mona nazwa dziaajcym MUDem !
 *   (i udao mi si zachowa te JamySingle w wersji dziaajcej !)
 *
 * Revision 1.5  2004/01/31 06:02:09  michalis
 * + Server.world
 * + SimpleClient.newPlayer
 * * Idea of Connection changed : now it does not inherit from StringSocket.
 *   It just holds stringSocket inside, and can create PlayerConnection.
 * + PlayerConnection : inherits from Player, contains StringSocket.
 *   (This is what was eariler meant to be in Connection)
 * * Server.users split to connections and world.players()
 * + Server adds ready connections to world.players() using Connection.createPlayer
 * - Server.findById (it was just a TO-DO) removed,
 *   now it can be implemented using IdedList.byId
 * + important comments at world.action
 * * StringSocket.read bug (when receiving two messages *fast*) fixed
 * + Utils.InternalError(String, Throwable)
 * * some Server docs improved
 *
 * Revision 1.4  2004/01/31 01:39:20  michalis
 * * StringSocket.read i write zaimplementwane porzdnie; wszystko teraz powinno
 *   dziaa nawet gdy serwer i klient stoj na maszynach pomidzy ktrymi jest
 *   wolne poczenie, z rnymi OSami (rne kodowania pl ogonkw),
 *   procesorami (rne endianess).
 *
 * Revision 1.3  2004/01/30 00:59:43  michalis
 * * StringSocket.read small fix
 * + Server.broadcastMessage
 * - Removed some TO-DO in Server by Rafa; jeli dobrze je rozumiem to wykonaem je
 *   robic klas StringSocket
 * + Server wysya wiadomo do wszystkich klientw gdy podcza si nowy klient
 *   (to tylko testowe, w kocowej grze to nie ma by widoczne dla wszystkich)
 * + SimpleClient odbiera wiadomoci od serwera i wywietla; w ten sposb Server
 *   i SimpleClient robi pen komunikacj - obaj mog co wysya,
 *   obaj mog co odbiera
 *
 * Revision 1.2  2004/01/30 00:38:53  michalis
 * * docs improved
 *
 * Revision 1.1  2004/01/29 19:55:49  michalis
 * + code from server/Connection moved to common/StringSocket
 *   (with some minor improvements and clarification of interface)
 * - Connection class is empty for now - it only inherits everything from
 *   StringSocket, nothing more
 * * SimpleServer.sendMessage(Connection,String) implemented (using new
 *   Connection/StringSocket interface), so all sendMessage methods uncommented again;
 *   and they don't throw IOException anymore - this is already handled
 *   in StringSocket.write()
 * * SimpleServer.closeServerSession: some minor improvemnets
 * - SimpleServer comment : @author Rafal Wawrzkowicz removed - myl e
 *   zrobiem wystarczajco duo w tej klasie eby mie co do powiedzenia.
 *   Nie twrzmy szczegowych list autorw do kadej klasy - w logach kady
 *   widzi co kto zrobi; a oryginalny autor pliku z klas nie zawsze jest osob
 *   ktra doprowadzia klas do stanu wzgldnej uywalnoci...
 *
 */
