/*
  Copyright 2004 Michalis Kamburelis.

  This file is part of "Jamy & Nory".

  "Jamy & Nory" is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  "Jamy & Nory" is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with "Jamy & Nory"; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package jamy_i_nory.common;

import java.util.*;

/** General-purpose static utilities. This class cannot be instantiated
   (the only constructor is private). Note: since this is something
   not related to our game, I (Michalis) suggest that we keep strings
   in this file in English. */
public class Utils {

  /** Throw this to indicate that a bug occured. Message should be something
     understandable to developers - it does not need to be understandable by
     user. */
  static public class InternalError extends Error {
    public InternalError(String message) { super(message); }
    public InternalError(Throwable cause) { super(cause); }
    public InternalError(String message, Throwable cause) { super(message, cause); }
  }
  
  /** If not trueValue then throws InternalError("Check failed").
     Something like assert but true value is always calculated and checked. */
  final static public void check(boolean trueValue)
  {
    check(trueValue, "Check failed");
  }

  /** If not trueValue then throws InternalError(message).
     Something like assert but true value is always calculated and checked. */
  final static public void check(boolean trueValue, String message)
  {
    if (!trueValue) throw new InternalError(message);
  }    
  
  /** Generator liczb losowych do globalnego uzytku przez cokolwiek
     w tym programie. Nigdy nie rb mu setSeed - on jest tylko do
     generowania liczb ktre naprawd chcemy eby byy losowe.  */
  final static public Random rand = new Random();

  /** Zamienia kolekcj na wielolinijkowy String z tytuem.
     @return Jeeli c.isEmpty to "", wpp. title + {@link #nl nl} +
       collectionToMultiLineString(c, linePrefix). */
  final static public String collectionToMultiLineString(Collection c,
    String title, String linePrefix)
  {
    if (c.isEmpty())
      return ""; else
    {
      StringBuffer result = new StringBuffer(title + nl);
      Iterator i = c.iterator();
      while (i.hasNext())
        result.append(linePrefix + i.next() + nl);
      return result.toString();
    }
  }
  
  /** Zamienia kolekcj na wielolinijkowy String.
     Dla kadego elementu i kolekcji dopisuje do wyniku
     inePrefix + i + {@link #nl nl}. Zwracam wic uwag e zawsze 
     koczy si znakiem nowej linii, chyba e jest "".
     Elementy kolekcji bed wypisywane w takiej kolejnoci w jakiej
     bd zwracane przez iterator kolekcji. */
  final static public String collectionToMultiLineString(Collection c,
    String linePrefix)
  {
    StringBuffer result = new StringBuffer();
    Iterator i = c.iterator();
    while (i.hasNext())
      result.append(linePrefix + i.next() + nl);
    return result.toString();
  }  

  /** Znajd w tablicy strs string s. Case-sensitive (poprzez String.equals).
     Zwraca indeks do tablicy lub -1 jeli nie znalaz. */
  final static public int stringArrayIndexOf(String[] strs, String s)
  {
    for (int i = 0; i < strs.length; i++)
      if (strs[i].equals(s)) return i;
    return -1;
  }
  
  /** Returns strs[offset] + sep + strs[offset + 1] + sep + ... + sep 
     + strs[leng-1]. If len == 0 returns "". */  
  final static public String stringArrayConcat(String[] strs, 
    int offset, int len, String sep)
  {
    StringBuffer result = new StringBuffer();
    if (len != 0)
    {
      for (int i = offset; i < offset +len -1; i++) 
        result.append(strs[i] + sep);
      result.append(strs[offset +len -1]);
    }
    return result.toString();
  }  
  
  /** Zwraca true jeli podane dwie tablice stringw maj rwne dugoci
     i na kadej pozycji maj rwne stringi (w sensie metody String.equals)
     lub jedna (lub obie) z podanych tablic maj na tej pozycji null. 
     Innymi sowy null zachowuje si tu jak doker - pasuje do wszystkiego. */
  final static public boolean stringArraysNullEqual(String[] s1, String[] s2)
  {
    if (s1.length != s2.length) return false;

    for (int i = 0; i < s1.length; i++)
      if (s1[i] != null && s2[i] != null && !s1[i].equals(s2[i]))
        return false;

    return true;    
  }

  /** Returns strs[0] + sep + strs[1] + sep + ... + sep + strs[strs.length-1].
     If strs.length == 0 returns "". */
  final static public String stringArrayConcat(String[] strs, String sep)
  {
    return stringArrayConcat(strs, 0, strs.length, sep);
  }
  
  /** Zwr s z pierwsz liter zamienion na du. Gdy s = null lub ""
     to zwraca s. */
  final static public String capitalize(String s)
  {
    if (s != null && s.length() != 0)
      return Character.toUpperCase(s.charAt(0)) + s.substring(1); else
      return s;
  }
  
  /** Zwraca listName +" (" +list.size() +"):" +Utils.nl+
     Utils.collectionToMultiLineString(list, "  ") */
  final static public String listToString(List list, String listName)
  {
    return listName +" (" +list.size() +"):" +Utils.nl+
      Utils.collectionToMultiLineString(list, "  ");
  }
  
  /** If (s != null) and (s != "") then make s[0] uppercase. */
  final static public void capitalize(StringBuffer s)
  {
    if (s != null && s.length() != 0)
      s.setCharAt(0, Character.toUpperCase(s.charAt(0)));
  }

  /** Klasa opisujca opcj w linii polece, na potrzeby 
     {@link #parseCommandLine}. Obiekty tej klasy s immutable. */
  static public class CommandLineOption {
  
    /** Duga nazwa opcji. Uytkownik bdzie musia j poprzedzi "--". */
    final public String longName;
    
    /** Liczba argumentw opcji - tzn. parametrw ktre musz by podane
       za dan opcj i bd potraktowane jako "towarzyszce danej opcji"
       przy parsowaniu linii polece programu. */
    final public int argsCount;
    
    public CommandLineOption(String aLongName, int aArgsCount)
    { 
      longName = aLongName;
      argsCount = aArgsCount;      
    }
  }
    
  static public interface CommandLineOptionProcesser {
    void processCommandLineOption(int optionNum, String[] optionArgs);
  }
  
  static public class CommandLineInvalidError extends Error {
    public CommandLineInvalidError(String message) { super(message); }
  }

  /** Parsuje argv zgodnie ze specyfikacj opcji options i kad
     sparsowan opcj podaje do processer (podajc optionNum = numer opcji
     na licie options i optionArgs = argumenty opcji). Nierozpoznane opcje
     powoduj CommandLineInvalidError. Jeli podano za mao argumentw 
     dla danej opcji to te CommandLineInvalidError.  
     
     <p>Parametry nie bdce opcjami (ani ich argumentami) podaje do processer
     z optionNum = -1 i 1-elementowym optionArgs. 
     
     <p>To jest bardzo proste parsowanie parametrw - by moe
     bdzie si rozwijao, w zalenoci od potrzeb klienta i serwera
     naszego MUDa. Np. naleaoby doda krtkie opcje i rozpoznawanie "--". */
  static public void parseCommandLine(String[] argv, CommandLineOption[] options, 
    CommandLineOptionProcesser processer)
  {
    /* Below we must create new LinkedList because Arrays.asList returns
       fixed-size list. Besides, we don't know what optional List functionality
       (e.g. iterator.remove) is implemented in Arrays.asList. */
    LinkedList argvlist = new LinkedList(Arrays.asList(argv));            

    Iterator i = argvlist.iterator();
    while (i.hasNext())
    { 
      String s = (String) i.next();
      
      /* evaluate optionNum */
      int optionNum = -1;
      if (s.startsWith("-"))
      {        
        for (int j = 0; j < options.length; j++)
          if (s.equals("--" +options[j].longName))
            { optionNum = j; break; }
        if (optionNum == -1)
          throw new CommandLineInvalidError("Invalid option \"" +s +"\"");
      }
        
      /* evaluate optionArgs */
      String[] optionArgs;
      if (optionNum == -1)
        optionArgs = new String[] { s }; else
      {
        optionArgs = new String[options[optionNum].argsCount];
        for (int j = 0; j < optionArgs.length; j++)
        {
          if (i.hasNext())
            optionArgs[j] = (String) i.next(); else
            throw new CommandLineInvalidError("Not enough arguments for option --" 
              +options[optionNum].longName);
        }
      }      
      
      /* pass optionNum and optionArgs to processer */
      processer.processCommandLineOption(optionNum, optionArgs);
    }
  }
  
  final static public int clamped(int value, int min, int max)
  {
    if (value < min) return min; else
    if (value > max) return max; else
                     return value;
  }

  /** Returns c repated count times. */
  final static public String makeString(char c, int count)
  {
    StringBuffer result = new StringBuffer(count);
    for (int i = 0; i < count; i++) result.append(c);
    return result.toString();
  }
  
  /** Returns s padded from the right with padChar characters.
     If length of s is >= minLength returns s. 
     Else returns s with padChar appended so many times that result.length() 
     is minLength. */  
  final static public String rightPad(String s, int minLength, char padChar)
  {
    if (s.length() < minLength)
      return s + makeString(padChar, minLength - s.length()); else
      return s;
  }

  /** Run some simple tests for this class. Run as
     java -ea jamy_i_nory.common.Utils */
  final static public void main(String[] args)
  {
    String[] strs = { "raz", "dwa", "trzy" };
    
    assert stringArrayIndexOf(strs, "Raz") == -1;
    assert stringArrayIndexOf(strs, "raz") == 0;
    assert stringArrayIndexOf(strs, "trzy") == 2;
    assert stringArrayIndexOf(strs, "trzynacie") == -1;
    
    assert stringArrayConcat(strs, "/").equals("raz/dwa/trzy");
    assert stringArrayConcat(strs, 1, 2, "/").equals("dwa/trzy");
    assert stringArrayConcat(new String[] {}, "/").equals("");
    
    assert capitalize("bla bla").equals("Bla bla");
    assert capitalize("ma").equals("ma");
    StringBuffer sbuf = new StringBuffer("foo");
    capitalize(sbuf);
    assert sbuf.toString().equals("Foo");
    
    assert makeString('x', 3).equals("xxx");
    assert makeString('x', 0).equals("");
    
    assert rightPad("blabla", 3, '|').equals("blabla");
    assert rightPad("blabla", 9, '|').equals("blabla|||");
    
    assert clamped(-90, -1, 10) == -1;
    assert clamped(90, -1, 10) == 10;
    assert clamped(4, -1, 10) == 4;
    
    System.out.println("All tests passed.");
  }

  /** Domylny znak newline na danej platformie. "\n" jeeli z jakiego
     powodu na danej platformie wywoanie System.getProperty("line.separator");
     rzuca wyjtek. */
  final static public String nl;

  static {
    /* initialize nl. Potrzebujemy helper variable fnl aby kompilator
       widzial ze nl jest rzeczywiscie inicjowane tylko raz.  */
    String fnl;
    try {
      fnl = System.getProperty("line.separator");
    } catch (Exception e) {
      fnl = "\n"; // set NL to default newline literal
    }
    nl = fnl;
  }
}

/*
 * $Log: Utils.java,v $
 * Revision 1.26  2004/02/19 21:54:18  michalis
 * + klasa ClientApplet ostatecznie zrobiona - dziaa !
 * * useStyles wycignite jako parametr konstruktorw IOSwing
 * + JamyUtils z pomocniczymi rzeczami gromadzcymi parametry do interfejsw,
 *   teraz i serwer i klient i single maj peen zestaw 3 parametrw --console, --simple-swing, --stylish-swing
 * * odpowiednie uaktualnienia w dokumentacji
 *
 * Revision 1.25  2004/02/18 14:37:23  michalis
 * * pl stringi
 * * --help output changed (to reflect that those classes will be run by bin/scripts)
 * preparations for ClientApplet:
 * * Client rozdzielony na Client i ClientStandalone
 * * znaczna cz pakietu common przeniesiona do nowego pakietu game
 * X* make jamy_i_nory_client_applet.jar
 *
 * Revision 1.24  2004/02/15 15:24:35  michalis
 * + pdf generating
 * + COPYING file
 * + Utils.check
 * + Location.add/removeLink
 * + LocationWithDynLink
 * * NoraOrBiblioteka.useItem simplified
 * * teraz i w bibliotece i w norze s drzwi
 * - Makefiles rozsiane po katalogach.
 * * Globalny Makefile rzdzi.
 *
 * Revision 1.23  2004/02/09 23:08:53  michalis
 * + GPL license clause i copyrighty (moje i gdzieniegdzie Michaa, sorry Rafa
 *   ale naprawd nic nie zostao) do wszystkich *.java
 *
 * Revision 1.22  2004/02/07 21:11:32  michalis
 * * StatModifier.makeFinalStat changed (bugfix: must be maxLife > 0,
 *   bez tego GrojecKiller czsto zaczyna ju jako trup)
 * * StatModifier.Modifs are named (as it was before) addModifs
 * * small improvement to Creature.action (to m.in. eliminuje problem ktry mi
 *   opisalicie e po zabiciu DungeonExplorera on jeszcze zdy (martwy)
 *   wyj z pomieszczenia)
 * + Utils.clamped
 * * Being.getDelay lepiej dostrojone
 *
 * Revision 1.21  2004/02/06 04:44:40  michalis
 * + InactivePlayer
 * + dodatkowy konstruktor do Being
 * + dodatkowy konstruktor Player(World, InactivePlayer)
 * * beforeDeath rozszerzone o parametr goingInactive
 * + Being.goInactive
 * * listToString moved from World to Utils
 * + ServerCommand : polecenie "mrozonki"
 *
 * Revision 1.20  2004/02/06 01:24:33  michalis
 * * implementacja UserCommandBlackTokens.matches przeniesiona do Utils.stringArraysNullEqual
 * + ServerCommand
 * * Server.ioCheckInput uproszczone dziki temu
 * * komendy serwera na polski, konsekwentnie
 * + ServerCommand "zabij ID"
 *
 * Revision 1.19  2004/02/05 23:27:39  michalis
 * * jeszcze mae poprawki w kwestii kiedy pisa TO-DO a kiedy to samo bez mylnika
 * * IOSwing konstruktor: parametr windowTitle
 * + Server obsuguje parametr --swing, ma pole IOSystem
 * + Utils.collectionToMultiLineString(Collection,String)
 * * Server mona ju kontrolowa interfejsem IOSystem, polecenia "x", "h", "p", "c"
 * + World.allCreatures/PlayersToString,
 * * UserCommand. "debug players/creatures" uywaj World.allCreatures/PlayersToString
 * * Poprawka w Server.ServerListener.run przy sleep
 *
 * Revision 1.18  2004/02/02 00:28:21  michalis
 * + Utils.stringArrayConcat(String, int, int, String)
 * * UserCommand some TO-DO in komenda "powiedz"  removed
 *
 * Revision 1.17  2004/01/31 06:02:09  michalis
 * + Server.world
 * + SimpleClient.newPlayer
 * * Idea of Connection changed : now it does not inherit from StringSocket.
 *   It just holds stringSocket inside, and can create PlayerConnection.
 * + PlayerConnection : inherits from Player, contains StringSocket.
 *   (This is what was eariler meant to be in Connection)
 * * Server.users split to connections and world.players()
 * + Server adds ready connections to world.players() using Connection.createPlayer
 * - Server.findById (it was just a TO-DO) removed,
 *   now it can be implemented using IdedList.byId
 * + important comments at world.action
 * * StringSocket.read bug (when receiving two messages *fast*) fixed
 * + Utils.InternalError(String, Throwable)
 * * some Server docs improved
 *
 * Revision 1.16  2004/01/30 00:38:53  michalis
 * * docs improved
 *
 * Revision 1.15  2004/01/30 00:11:59  michalis
 * + SimpleServer uses Utils.parseCommandLine, params --help and --port implemented
 * + Utils.DEFAULT_JAMY_PORT
 * * SimpleClient: small improvement to message printed by --help
 * * small improvement to local_server.sh and server.sh
 *
 * Revision 1.14  2004/01/29 23:42:47  michalis
 * * SimpleClient uses non-blocking input from IOSystem (lineBegin/End, as in JamySingle)
 * * SimpleClient uses Swing interface by default
 * + Utils.parseCommandLine
 * + SimpleClient uses command-line parsing, params --help, --port, --console
 *
 * Revision 1.13  2004/01/18 20:56:27  michalis
 * * rand moved from DiceRandom to Utils (as final static)
 *
 * Revision 1.12  2004/01/17 16:49:32  michalis
 * + InternalError.InternalError(Throwable cause)
 *
 * Revision 1.11  2004/01/15 19:59:41  michalis
 * - readInt/BoolPrompt removed (to IOSystem), in removed (to IOStd)
 *
 * Revision 1.10  2004/01/15 18:27:31  michalis
 * + capitalize(String), capitalize(StringBuffer)
 *
 * Revision 1.9  2004/01/14 22:36:49  michalis
 * * EInternalError renamed to InternalError to be consistent with Java
 *   naming conventions; InternalError still unused anywhere, but I have plans...
 *
 * Revision 1.8  2004/01/14 21:44:55  michalis
 * + readBoolPrompt
 *
 * Revision 1.7  2004/01/13 20:25:23  michalis
 * * Some docs corrected
 *
 * Revision 1.6  2004/01/13 20:18:01  michalis
 * + stringArrayIndexOf, stringArrayConcat
 *
 * Revision 1.5  2004/01/13 15:11:42  michalis
 * + EInternalError
 *
 * Revision 1.4  2004/01/11 04:12:17  michalis
 * + Utils.collectionToMultiLineString
 *
 * Revision 1.3  2004/01/10 08:27:30  michalis
 * * Nice doc comments
 *
 * Revision 1.2  2004/01/10 07:38:23  michalis
 * + I forgot to add CVS Logs when I created those files; well, now they are
 *
 */
