/*
  Copyright 2004 Michalis Kamburelis.

  This file is part of "Jamy & Nory".

  "Jamy & Nory" is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  "Jamy & Nory" is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with "Jamy & Nory"; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package jamy_i_nory.common.iosystem;

import jamy_i_nory.common.Utils;
import java.io.*;

/** Prosta implementacja IOSystem za pomoc standardowego wejcia/wyjcia
   programu, czyli System.in/out. Absolutnie przenona. 
   
   <p>eby wszystko byo jak naley ju nigdzie w programie nie powiniene
   uywa System.in/out, pozostaw to tej klasie. Z tego samego
   powodu nie twrz dwch instancji tej klasy.  
   
   <p>Pewne uwagi dla uytkownikw Windowsa uywajcych konsoli wbudowanej
   pod Windowsa (nie ma znaczenia shell, czy command.com czy 
   np. Cygwinowy bash) s w pliku README, w skrcie: uywajcie innej konsoli,
   np. rxvt Cygwina, i wszystko bdzie dobrze. */
public class IOStd extends IOSystem {

  /** In our game catching IOException is pretty pointless,
     so we will convert in this class IOException to IORuntimeException.
     So you do not have to declare them in "throws" clause (or catch them). */
  public static class IORuntimeException extends RuntimeException
  { 
    IORuntimeException(IOException cause) { super(cause); }
  }

  /** Thrown when stdin ended while we were trying to do {@link #readString}
     (or other read* implemented using {@link #readString}, 
     i.e. {@link #readInt} and {@link #readBoolean}) */
  public static class StdinEndInReadString extends RuntimeException {}
  
  /** Przy kadym doLineBegin wypisywany jest ten linePrompt. */
  public final String linePrompt;
  
  public IOStd(String aLinePrompt) 
  {
    linePrompt = aLinePrompt;
  }
  
  /** System.in wrapped in BufferedReader. */
  final private BufferedReader in
    = new BufferedReader(new InputStreamReader(System.in));  
  
  private boolean fLineReady;
  
  /** Moe by null jeli natrafilimy na EOF w in. Przy czym jeli 
     fLine = null to musi by fLineReady = true. 
     Przechowujemy fLine w StringBuffer zamiast w String co jest duzo
     wydajniejsze. */
  private StringBuffer fLine;  
  
  protected void doLineBegin()
  {
    System.out.print(linePrompt);
    fLineReady = false;
    fLine = new StringBuffer();
  }
  
  /* Used in lineReady to skip \n after \r */
  private boolean omitLF = false;

  public boolean lineReady()
  {
    try {  
      while ((!fLineReady) && in.ready()) {
        char c[] = new char[1];
        int readResult = in.read(c);
        if (readResult == -1)
        {
          fLine = null;
          fLineReady = true;
        } else
        {
          assert readResult == 1;
          char cc = c[0];
          
          /* OK, odczytalismy znak cc ze strumienia */
          boolean omitChar = (omitLF && cc == '\n');
          omitLF = false;          
          if (!omitChar)
          {
            /* Czy stoimy na kocu linii ? 
               Rozpoznawane s koce linii \n, \r i \r\n */
            if (cc == '\r' || cc == '\n')   
            {
              fLineReady = true; 
              if (cc == '\r') omitLF = true;
            } else
              fLine.append(cc);
          }
        }
      }
    } catch (IOException e) {
      throw new IORuntimeException(e);
    }      
    
    return fLineReady;
  }  
  
  protected String doLineEnd()
  {
    return (fLine == null ? null :  "" +fLine);
  }
  
  protected void doLineEndCancel()
  {
    if (! lineReady()) System.out.print("\n"); 
  }
    
  public void print(String s)
  {
    /* if lineReading() then we encapsulate s in newlines so that it does not
       interfere with user input. We do here a check so that when s is ""
       we do not print any unnecessary newlines.  */
  
    if (s.length() == 0) return;   
    
    if (lineReading() && (!s.startsWith(Utils.nl))) System.out.println();
    System.out.print(s); 
    if (lineReading() && (!s.endsWith(Utils.nl))) System.out.println();
  }  
  
  public String readString(String prompt, boolean emptyResultAllowed)
  {
    /* dla celow testowych mozna zamiast result = in.readLine();
       uzyc 
        lineBegin(); 
        while (!lineReady()) ; 
        result = lineEnd();     
      (robi busy-waiting ale dziala poprawnie, modulo wypisywanie linePrompt) */

    try {
      String result;
      do {
        print(prompt +" ");
        result = in.readLine();
        if (result == null) throw new StdinEndInReadString();
      } while ((!emptyResultAllowed) && result.length() == 0);
      return result;
    } catch (IOException e) {
      throw new IORuntimeException(e);
    } 
  }
}

/*
 * $Log: IOStd.java,v $
 * Revision 1.2  2004/02/09 23:08:56  michalis
 * + GPL license clause i copyrighty (moje i gdzieniegdzie Michaa, sorry Rafa
 *   ale naprawd nic nie zostao) do wszystkich *.java
 *
 * Revision 1.1  2004/02/05 20:28:21  michalis
 * * Klasy IOSystem/Std/Swing przeniesione do pakietu common.iosystem.
 *   Byy uywane take przez single/ wic ich dotychczasowa lokalizacja
 *   w client/ bya za. Ponadto zaraz bd uywane take przez server/.
 *
 * Logs as client/IOStd.java:
 * Revision 1.12  2004/02/05 20:05:23  michalis
 * * Zrobiem eby w logach CVSa zapisanych w plikach byo wszdzie TO-DO
 *   z mylnikiem w rodku. Wprawdzie nieznacznie rozsynchronizowuje to
 *   zapisane w plikach logi z wynikiem 'cvs log' ale za to grep TO-DO
 *   (gdzie w wyrazie TO-DO naley usun mylnik) dziaa jak trzeba.
 *
 * Revision 1.11  2004/01/31 21:40:42  michalis
 * + IOSystem and subclasses: method lineEndCancel
 * * JamySingle and Client use lineEndCancel to avoid some bug
 *
 * Revision 1.10  2004/01/18 04:31:36  michalis
 * + minor updates, docs updated (notes about Windows console)
 *
 * Revision 1.9  2004/01/18 02:31:24  michalis
 * * IOStd: handles Windows line ends, Error changed to RuntimeException,
 *   some TO DO for Windows added
 *
 * Revision 1.8  2004/01/17 16:53:02  michalis
 * * Minor docs improvement
 *
 * Revision 1.7  2004/01/16 20:42:31  michalis
 * * IOStd.print now works nicely when lineReading
 *
 * Revision 1.6  2004/01/16 20:01:59  michalis
 * * IOSystem dostao duo lepszy interfejs, bardziej przejrzysty i teraz
 *   implementacja podklasy IOSystem realizujcej interfejs np. przez
 *   jakiego swinga jest rzeczywicie moliwa.
 * * IOStd zostao przerobione aby dostosowa si do nowego interfejsu IOSystem,
 *   zyskao te pole linePrompt
 *
 * Revision 1.5  2004/01/16 01:23:50  michalis
 * * JamySingle uses non-blocking io from IOStd, some TO-DOs added
 *
 * Revision 1.4  2004/01/16 00:27:59  michalis
 * * IOSystem.readIntPrompt fixed (println -> print), minor adjustments to IOStd
 *
 * Revision 1.3  2004/01/15 21:46:28  michalis
 * * IOStd implemented
 *
 * Revision 1.2  2004/01/15 21:12:03  michalis
 * * IOSystem and IOStd fully specified and IOStd almost implemented
 *
 * Revision 1.1  2004/01/15 20:04:36  michalis
 * + jamy_i_nory.client.IOSystem
 * + jamy_i_nory.client.IOStd
 * + JamySingle.io, JamySingle.newPlayerInteractive (from Player.newInteractive)
 * * JamySingle and UserCommand use JamySingle.io instead of System.in/out
 *
 */
