/*
  Copyright 2004 Michalis Kamburelis.

  This file is part of "Jamy & Nory".

  "Jamy & Nory" is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  "Jamy & Nory" is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with "Jamy & Nory"; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package jamy_i_nory.common.iosystem;

import javax.swing.*;
import javax.swing.text.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import jamy_i_nory.common.Utils;

/** Implementacja IOSystem za pomoc maego okienka z komponentami Swinga.
    
   <p>Krtkie omwienie interfejsu (moe ulec zmianie razem z implementacj,
   to omwienie jest raczej pod ktem utkownika):
   na grze due jest due okienko z tekstem wypisywanym przez gr.
   Na dole jest linijka w ktrej wpisujemy polecenia dla gry (prawdopodobnie 
   cay czas na niej bdzie focus). Na dole po prawej may przycisk
   OK - przyciskajc go akceptujemy wpisane polecenia, ale mona
   te po prostu wcisn enter po wpisaniu polecenia.
   Rozmiar caego okienka mona swobodnie zmienia wic jeli masz
   duy ekran to moesz naraz widzie naprawd duo tekstu.
   
   <p>Moliwe s dwie wersje tego okienka - uywajca styli w gwnym
   okienku tekstowym (bdzie robia echo komend pogrubionym pismem)
   i nie uywajca styli. Ze stylami jest duo adniej ale i sporo wolniej.

   <p>TODO: okienko w Swingu pod Linuxem zbyt atwo traci focus,
   tzn. gdy robimy readString gwne okienko (appletviewera czy te
   JFrame) ma tendencj do znikania - chowa si za innymi oknami.
   No w kulki se leci. Ten problem naleaoby rozwiza w implementacji Swinga, 
   ale to nie jest odpowied. Trzeba go jako obej.
   
   TODO: readInt may be implemented nicer for this class.
*/
public class IOSwing extends IOSystem {

  /* Some general implementation notes: 
     - Remember about swing threading issues. When you are not in swing 
       event thread you can synchronize with e.g. swingThreadInvoke,
       the exception is IOSwing constructor - before components are "realized"
       (i.e. before pack) we can manipulate them freely. */

  /* swing components ---------------------------------------- */
  
  /** frame is JFrame or JApplet (but it could be any Container
     that implements RootPaneContainer). This is our enclosing component. */
  private Container frame;
  /** To samo co (RootPaneContainer) frameAsRootPaneContainer */
  private RootPaneContainer frameAsRootPaneContainer;
  /** Pewne elementy specyficzne dla sytuacji gdy frame jest instancj Window
     (czyli gdy jest typu JFrame a nie JApplet, ale generalizujemy t sytuacj)
     uywaj tej zmiennej. Jest null gdy frame nie jest typu Window, 
     wpp. jest tym samym co (Window) frame */
  private Window frameAsWindow;
  
  private JTextComponent outputArea;
  private JTextField commandText;
  private JButton buttonOK;
  
  /* ---------------------------------------- */
  
  /* dostpny tylko jeli (outputArea instanceof JTextPane) */
  private Style styleBold;
  
  /* Shared by swing event thread and main thread. Use only get/set below
     to read/write them to be safe (note: opreations on boolean are atomic
     so probably some things could be optimized; but this is not worth the
     risk of introducing bugs). 
     
     fLine can be null, so lineEnd can return null to indicate that
     user wants to close the window. We do not let user to close the
     window here but we're signalling that by returning lineEnd = null,
     that is consistent with IOSystem.lineEnd interface. */
  private String fLine;
  private boolean fLineReady;
  synchronized private String getFLine() { return fLine; }
  synchronized private void setFLine(String value) { fLine = value; }
  synchronized private boolean getFLineReady() { return fLineReady; }
  synchronized private void setFLineReady(boolean value) { fLineReady = value; }
  
  /** Call this from main thread. It will cause r.run to be executed 
     in swing event thread. (it will wait for this execution - i.e.
     if uses invokeAndWait, not invokeLater) */
  private void swingThreadInvoke(Runnable r)
  {
    try {      
      SwingUtilities.invokeAndWait(r);
    } catch (InterruptedException e)
    { throw new Utils.InternalError(e); }
      catch (java.lang.reflect.InvocationTargetException e)
    { throw new Utils.InternalError(e); }
  }
  
  /** Beware: this operates on swing components, and so should be executed
     as part of swing's event thread. */
  private void setLineInputEnabled(boolean enabled)
  {
    buttonOK.setEnabled(enabled);
    commandText.setEnabled(enabled);  
  }
  
  /** If lineReading and not already fLineReady then it sets 
     fLine to newFLine, fLineReady to true (using synchronized methods
     of course) and then calls setLineInputEnabled(false).
     Call this only from swing event thread. */
  private void makeLineReady(String newFLine)
  {
    if (lineReading() && (!getFLineReady())) {
      setFLine(newFLine);
      setFLineReady(true);
      setLineInputEnabled(false);
    }  
  }

  protected void doLineBegin()
  {    
    setFLineReady(false);
    swingThreadInvoke(new Runnable() {
        public void run() 
        {
          setLineInputEnabled(true);
          commandText.setText("");    
          commandText.requestFocusInWindow();
        }
      });
  }
  
  public boolean lineReady()
  {
    return getFLineReady();
  }
  
  /** Dopisuje string z danym stylem na kocu dokumentu. 
     Wywouj tylko w watku Swinga i tylko podczas gdy outputArea jest 
     JTextPane. str ma mie newlines ju w konwencji "\n" (czyli 
     w konwencji Swinga, NIE w konwencji Utils.nl). */
  private void appendString(String str, AttributeSet a)
  {
    StyledDocument doc = ((JTextPane) outputArea).getStyledDocument();
    try {    
      doc.insertString(doc.getLength(), str, a);
      
      /* tests:
      try {
        FileWriter fw = new FileWriter("/tmp/output");
        fw.write(outputArea.getText());
        fw.close();
      } catch (IOException e) {
        throw new Utils.InternalError(e);
      } */
      
    } catch (BadLocationException e) {
      throw new Utils.InternalError(e);
    }
  }
  
  protected String doLineEnd()
  {
    final String result = getFLine();
    /* TODO: robi to te dla TextArea ? Jak, eby byo czytelnie ? */
    if (result != null && outputArea instanceof JTextPane)
    {
      swingThreadInvoke(new Runnable() {
        public void run() 
        {  
          appendString(result +"\n", styleBold);
        }
      });
    }
    return result;
  }
  
  protected void doLineEndCancel()
  {
    /* Nothing needs to be done here. */
  }
  
  public void print(final String s)
  {
    /* We do not want to continue when s is empty because our MUD client
       often does print(""); i w rezultacie uytkownik nie moe niczego
       zaznaczyc w outputArea bo co chwila przestawiamy kursor na koniec
       tekstu. Ot, taka bzdurka ale wana eby UI dziaa rzeczywicie 
       wygodnie. */
    if (s.length() == 0) return;
    
    final String goodS = s.replaceAll(Utils.nl, "\n");
    
    swingThreadInvoke(new Runnable() {
        public void run() 
        {  
          if (outputArea instanceof JTextArea)
          {
            ((JTextArea) outputArea).append(goodS);
          } else
          {
            appendString(goodS, null);
          }
          
          /* scroll down to the end of text */
          outputArea.setCaretPosition(outputArea.getDocument().getLength());
        }
      });
  }
  
  public String readString(final String prompt, final boolean emptyResultAllowed)
  {
    /*TODO: estetyka:
      niech nie wyswietla w ogole przycisku Cancel,
      przekazac jakos emptyResultAllowed zeby po wpisaniu pustego
      stringa nie robil od nowa okienka tylko nie pozwalal wcisnac Ok ? */     
    class RunReadString implements Runnable {
      String result;
      public void run() 
      {
        do {
          result = JOptionPane.showInputDialog(frame, prompt);
        } while (result == null || 
                  ((!emptyResultAllowed) && result.length() == 0));
      }
    };    
    RunReadString runReadString = new RunReadString();
    
    swingThreadInvoke(runReadString);
    return runReadString.result;    
  }
  
  public boolean readBoolean(final String prompt, final boolean def)
  {
    class RunReadBoolean implements Runnable {
      boolean result;
      public void run() 
      {
        /*TODO: jako uy def ?*/
        result = JOptionPane.showConfirmDialog(frame, prompt, "Confirm", 
          JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION;
      }
    };
    RunReadBoolean runReadBoolean = new RunReadBoolean();
            
    swingThreadInvoke(runReadBoolean);
    return runReadBoolean.result;
  }
  
  public void dispose()
  {
    if (frameAsWindow != null)
    {
      swingThreadInvoke(new Runnable() {
          public void run() { frameAsWindow.dispose(); } 
        });
    } else
    {
      /* Jeeli nie moemy zamkn okienka to przynajmniej zablokujmy kontrolki. */
      swingThreadInvoke(new Runnable() {
          public void run() { setLineInputEnabled(false); } 
        });
    }
  }
  
  /* initializing ---------------------------------------- */
  
  /** Skonstruuj interfejs dziaajcy w okienku (JFrame) o tytule windowTitle. */
  public IOSwing(String windowTitle, boolean useStyles)
  {
    JFrame f = new JFrame(windowTitle);
    f.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
    f.addWindowListener(new WindowAdapter() {
        public void windowClosing(WindowEvent e) 
        { makeLineReady(null); }
      });  
    postConstructor(f, useStyles);
  }
  
  /** Skonstruuj interfejs dziaajcy w aplecie. */
  public IOSwing(final JApplet applet, final boolean useStyles)
  {
    swingThreadInvoke(new Runnable() {
        public void run() { postConstructor(applet, useStyles); } 
      });    
  }

  /** To jest faktyczny konstruktor ktry konstruuje interfejs dziaajcy
     w dowolnym Container ktry implementuje RootPaneContainer. 
     Jego tre mona wykona jako kawaek wtku Swinga, chocia
     nie trzeba jeli nic wczeniej w programie nie zainicjowao wtku
     Swinga (czyli w przypadku gdy robimy wersj JFrame wywoywan
     z ClientStandalone). */
  private void postConstructor(Container aFrame, boolean useStyles)
  {
    frame = aFrame;
    frameAsRootPaneContainer = (RootPaneContainer) frame;
    if (frame instanceof Window)
      frameAsWindow = (Window) frame;
    
    /* set up outputArea ---------------------------------------- */ 

    if (useStyles)
    {
      JTextPane tp = new JTextPane();
      tp.setPreferredSize(new Dimension(500, 400));
         
      /* Initialize styles */
      Style def = StyleContext.getDefaultStyleContext().
                      getStyle(StyleContext.DEFAULT_STYLE);

      StyledDocument doc = tp.getStyledDocument(); 
      
      styleBold = doc.addStyle(null, def);
      StyleConstants.setBold(styleBold, true);
         
      outputArea = tp;
    } else
    {
      JTextArea ta = new JTextArea(20, 50);
      ta.setLineWrap(true);
      ta.setWrapStyleWord(true);
      outputArea = ta;
    }
    outputArea.setEditable(false);
    
    JScrollPane outputAreaScrollPane = new JScrollPane(outputArea);
    outputAreaScrollPane.setHorizontalScrollBarPolicy(
      JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
    outputAreaScrollPane.setVerticalScrollBarPolicy(
      JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);

    /* set up user input controls ---------------------------------------- */    
    
    commandText = new JTextField(20);
    
    buttonOK = new JButton("OK");      
    buttonOK.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) 
        { makeLineReady(commandText.getText()); }
      });
    frameAsRootPaneContainer.getRootPane().setDefaultButton(buttonOK);
    
    setLineInputEnabled(false);    
    
    /* lay out controls ---------------------------------------- */
    
    final int BORDER = 5;
    
    JPanel lowerPanel = new JPanel();
    lowerPanel.setLayout(new BorderLayout());
    lowerPanel.add(commandText, BorderLayout.CENTER);
    lowerPanel.add(buttonOK, BorderLayout.EAST);
    lowerPanel.setBorder(BorderFactory.createEmptyBorder(BORDER, BORDER, 
      BORDER, BORDER));
      
    JPanel upperPanel = new JPanel();
    upperPanel.setLayout(new BorderLayout());
    upperPanel.add(outputAreaScrollPane, BorderLayout.CENTER);
    upperPanel.setBorder(BorderFactory.createEmptyBorder(BORDER, BORDER, 
      0, BORDER));
    
    Container cp = frameAsRootPaneContainer.getContentPane();
    cp.add(upperPanel, BorderLayout.CENTER);
    cp.add(lowerPanel, BorderLayout.SOUTH); 
    
    /* realize controls ---------------------------------------- */
  
    if (frameAsWindow != null) {
      frameAsWindow.pack();  
      frameAsWindow.setLocationRelativeTo(null);
    }
    frame.setVisible(true);  
  }
}
  
/*
 * $Log: IOSwing.java,v $
 * Revision 1.7  2004/02/19 21:54:20  michalis
 * + klasa ClientApplet ostatecznie zrobiona - dziaa !
 * * useStyles wycignite jako parametr konstruktorw IOSwing
 * + JamyUtils z pomocniczymi rzeczami gromadzcymi parametry do interfejsw,
 *   teraz i serwer i klient i single maj peen zestaw 3 parametrw --console, --simple-swing, --stylish-swing
 * * odpowiednie uaktualnienia w dokumentacji
 *
 * Revision 1.6  2004/02/19 07:54:48  michalis
 * * small bug fixik
 *
 * Revision 1.5  2004/02/18 14:37:25  michalis
 * * pl stringi
 * * --help output changed (to reflect that those classes will be run by bin/scripts)
 * preparations for ClientApplet:
 * * Client rozdzielony na Client i ClientStandalone
 * * znaczna cz pakietu common przeniesiona do nowego pakietu game
 * X* make jamy_i_nory_client_applet.jar
 *
 * Revision 1.4  2004/02/15 20:11:46  michalis
 * * lepszy interfejs w Swingu (JTextPane ze stylami, echo komendy bold'em,
 *   pole do dalszych upiksze)
 * * bugfix do "powiedz" w UserCommand (dla przypadku command.length == 0)
 * * linki do nory i biblioteki poczone przez wayBack
 *
 * Revision 1.3  2004/02/09 23:08:56  michalis
 * + GPL license clause i copyrighty (moje i gdzieniegdzie Michaa, sorry Rafa
 *   ale naprawd nic nie zostao) do wszystkich *.java
 *
 * Revision 1.2  2004/02/05 23:27:41  michalis
 * * jeszcze mae poprawki w kwestii kiedy pisa TO-DO a kiedy to samo bez mylnika
 * * IOSwing konstruktor: parametr windowTitle
 * + Server obsuguje parametr --swing, ma pole IOSystem
 * + Utils.collectionToMultiLineString(Collection,String)
 * * Server mona ju kontrolowa interfejsem IOSystem, polecenia "x", "h", "p", "c"
 * + World.allCreatures/PlayersToString,
 * * UserCommand. "debug players/creatures" uywaj World.allCreatures/PlayersToString
 * * Poprawka w Server.ServerListener.run przy sleep
 *
 * Revision 1.1  2004/02/05 20:28:21  michalis
 * * Klasy IOSystem/Std/Swing przeniesione do pakietu common.iosystem.
 *   Byy uywane take przez single/ wic ich dotychczasowa lokalizacja
 *   w client/ bya za. Ponadto zaraz bd uywane take przez server/.
 * 
 * Logs as client/IOSwing.java:
 * Revision 1.3  2004/01/31 21:40:42  michalis
 * + IOSystem and subclasses: method lineEndCancel
 * * JamySingle and Client use lineEndCancel to avoid some bug
 *
 * Revision 1.2  2004/01/17 22:14:15  michalis
 * * some docs improved in JamySingle, IOSystem and IOSwing
 * * JOptionPane in IOSwing used only by swingThreadInvoke
 *
 * Revision 1.1  2004/01/17 16:55:01  michalis
 * + Penowartociowa implementacja IOSystem za pomoc komponentw Swinga !
 *
 */
