/*
  Copyright 2004 Michalis Kamburelis.

  This file is part of "Jamy & Nory".

  "Jamy & Nory" is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  "Jamy & Nory" is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with "Jamy & Nory"; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package jamy_i_nory.common.iosystem;

import jamy_i_nory.common.Utils;

/** Ta abstrakcyjna klasa specyfikuje co musi zapewnia system 
   wejcia/wyjcia klienta naszego MUDa. W ten sposb moemy napisa 
   jednego klienta ktry ca komunikacj z wejciem/wyjciem
   robi przy pomocy klasy typu IOSystem i wewntrz tego klienta
   mamy swobod jakiej nieabstrakcyjnej klasy uylimy aby
   zrealizowa obiekt klasy IOSystem.
   W tym momencie dostpne s dwie podklasy IOSystem:
   <ul>
     <li>{@link IOStd} uywajca standardowego wejcia / wyjcia
       programu (System.in/out, nawet adnych szczeglnych funkcji do 
       obsugi terminala), 
     <li>{@link IOSwing} realizujca niniejszy interfejs przez komponenty swinga
   </ul>
   
   <p>Najwaniejsz rzecz jakiej wymagam od interfejsu tej klasy jest eby
   dostarcza wygodne funkcje <b>nieblokujce</b> do odczytu linii.
   Osigamy to metodami {@link #lineBegin}, 
   {@link #lineReady}, {@link #lineEnd}. Ponadto w dowolnym
   momencie moemy wywoa {@link #print}, nawet pomidzy 
   {@link #lineBegin} i {@link #lineEnd},
   i nie powinno to w specjalny sposb kolidowa z tym co user wpisuje. 
   
   <p>Te rzeczy s potrzebne bo w naszym MUDzie w czasie gdy 
   gracz bdzie mozolnie wpisywa z klawiatury swoje polecenie rne istoty 
   (sterowane przez serwer gry lub przez innych graczy) mog robi 
   rne rzeczy o ktrych trzeba gracza powiadomi, np. kto moe 
   zaatakowa gracza itp. 
   
   <p>Notka: nie naley zmienia metod tej klasy na static.
   Mimo e w przypadku klasy {@link IOStd} nie mona tworzy
   wicej ni jednej jej instancji w programie i uczynienie
   wszystkich jej metod static tylko bu tu pomogo. 
   Jednak dla innych podklas IOSystem to niekoniecznie bdzie prawdziwe.
   
   <p>Po uyciu obiektu tej klasy pamitaj wywoa {@link #dispose}.
   
   <p>TODO: by moe pniej dodamy tu bardziej wyspecjalizowane metody,
   np. "pozwl uytkownikowi wybra przedmiot z listy". Tego typu rzeczy
   bdziemy mogli zaimplementowa w IOSwing bardzo adnie.
   Zobaczymy czy bdzie czas eby si w to bawi.
   
   <p>TODO: everything should be read in more convenient way from user. 
   Something like Readline (but lib-readline-java sucks, it's only a wrapper
   around native GNU readline library). Maybe UserCommand should check for
   some prefixes when recognizing command ? But this would be very non-elegant,
   I would prefer implementing such functionality mostly in this class API,
   UserCommand powinien co najwyej mc tworzy list uzupenie lub
   (lepiej) dostarcza interfejs z metod w rodzaju completeCommand. */
public abstract class IOSystem {
  
  private boolean fLineReading = false;
  
  /** Czy jestesmy w trybie odczytywania linii tekstu w sposb
     nieblokujcy, czyli pomidzy lineBegin i lineEnd. */
  public boolean lineReading() { return fLineReading; }
  
  /** Pokrywaj w podklasach (nie pokrywaj lineBeing). 
     Wywoywane zawsze po zmianie lineReading z false na true. */
  protected abstract void doLineBegin();

  /** Wywoaj t metod aby zaznaczy e chcemy odczyta od usera
     jedn lini tekstu w trybie nieblokujcym. Czyli ustaw
     {@link #lineReading} na true. Jeeli {@link #lineReading}
     ju jest true to ta metoda dziaa jak NOP. */
  final public void lineBegin() 
  { 
    if (!fLineReading) { fLineReading = true; doLineBegin(); }
  }
    
  /** Wywouj tylko podczas gdy {@link #lineReading} = true.
     Sprawdza czy mona ju uy lineEnd, czyli czy linia jest gotowa. */
  public abstract boolean lineReady();

  /** Pokrywaj w podklasach - po prostu zwr to co trzeba i nie martw
     si o zmian lineReading().  */
  protected abstract String doLineEnd();
  
  /** Wywouj tylko podczas gdy {@link #lineReading} = true
     i tylko jeli {@link #lineReady} (tzn. jakie {@link #lineReady}
     wywoane po ostatnim {@link #lineBegin}) zwrcio true.
     Zwraca odczytan lini i ustawia {@link #lineReading} na false.
     Jako specjalny przypadek moe te zwrci null co oznacza
     e uytkownik w jakis sposb przerwa nasz kana komunikacji
     (np. w przypadku {@link IOStd} oznacza to e standardowe wejcie 
     si nagle skoczyo a w przypadku {@link IOSwing} e dostalimy
     polecenie zamknicia okienka od Window Managera). */
  final public String lineEnd()
  {
    assert lineReading();
    assert lineReady();
    String result = doLineEnd();
    fLineReading = false;
    return result;
  } 
  
  /** Pokrywaj w podklasach; bdzie wywoywane z {@link #lineEndCancel}
     tu przed przestawieniem lineReading z true na false. */
  protected abstract void doLineEndCancel();
  
  /** Intencj tej metody jest zmieni {@link #lineReading} na false, 
     bez wzgldu na to czy {@link #lineReady} i czy w ogle
     {@link #lineReading}.
     Moesz j wywoa gdy {@link #lineReading} = false, zadziaa jak NOP.
     Wpp. zmieni {@link #lineReading} na false.
     W przeciwiestwie do {@link #lineEnd} : po pierwsze, nie musi
     zachodzi {@link #lineReady}; po drugie, ta metoda zwraca void - 
     jeeli skadao si tak e {@link #lineReady} = true to trudno, 
     input usera przepada. */
  final public void lineEndCancel()
  {
    if (lineReading()) {
      doLineEndCancel();
      fLineReading = false;
    }
  }
  
  /** Wypisz s. Ta metoda zadziaa nawet w czasie gdy {@link #lineReading} 
     = true. 
     
     <p>Ze wzgldu na ograniczenia prostych implementacji podklas
     (jak {@link IOStd}) powiniene zawsze stara si przekazywa tutaj 
     moliwie cay string jaki ma zosta wypisany,
     np. zamiast robi <code>print(a); print(b);</code> lepiej zrobi
     <code>print(a + b);</code>. 
     
     <p>Newlines podawaj jako {@link jamy_i_nory.common.Utils#nl},
     jak zreszt wszdzie w tym programie.  */
  public abstract void print(String s);
  
  /** Skrt dla print(s +Utils.nl); */
  final public void println(String s) { print(s +Utils.nl); }
  
  /** Ta metoda pobiera od uytkownika dowolny string (rny 
     od "" jeli emptyResultAllowed), nigdy nie zwraca null
     (w przypadku readString nie ma sensu sygnalizowa koca stdin
     przez zwracanie nulla (tak jak to robi {@link #lineEnd})
     bo w przypadku readString i tak program bdzie zawsze wymaga
     czego konkretnego i "koniec stdin" i tak musiaby by
     potraktowany jako jaka sytuacja wyjtkowa).
     Uywaj tej metody kiedy na chwil wystarczy ci blokujcy 
     odczyt inputu uytkownika. Uywaj tylko gdy {@link #lineReading} = false.
     Wywietla przy tym prompt uytkownikowi, prompt to powinno
     by co jak <tt>"Podaj imi:"</tt> (zauwa e adna spacja
     po dwukropku nie jest potrzebna (ani zalecana)). */
  public abstract String readString(String prompt, boolean emptyResultAllowed);
  
  /** Input integer. Idea jest taka sama jak readString tyle
     e tutaj zwracamy inta z przedziau [minValue; maxValue].
     
     <p>W klasie IOSystem znajduje si prosta implementacja ktra dziaa
     na zasadzie robienia w pli readString i konwersji
     otrzymanego stringa na integer (jeeli prba si nie udaa
     to wypisuje o tym informacj przez {@link #print} i powtarza
     readString). To z pewnoci nie jest najadniejsza implementacja,
     np. pod swingiem zdecydowanie naleaoby pokry t metod
     dostarczajc co przyjemniejszego. Ale zawsze bdzie dziaa.
     
     @param prompt prompt for {@link #readString}
     @param minValue minimum value for user input
     @param maxValue maximum value for user input
     @return obtained integer */
  public int readInt(String prompt, int minValue, int maxValue)
  {
    do {
      try {
        /* Co podamy jako parametr emptyResultAllowed dla readString 
           poniej nie ma znaczenia dla poprawnoci readInt. Ale uytkownik 
           dostanie bardziej przejrzysty komunikat gdy podamy true. */
        int maybeResult = Integer.parseInt(readString(prompt, true));
        if (minValue <= maybeResult && maybeResult <= maxValue)
          return maybeResult; else
          println("  Invalid value : You must enter value between "
            +minValue +" and " +maxValue +".");
      } catch (NumberFormatException e)
      { println("  Invalid integer : " +e.getMessage()); }
    } while (true);
  }
  
  /** Input boolean ("yes" / "no" query).
     
     <p>W klasie IOSystem znajduje si prosta implementacja tej metody ktra
     dziaa w oparciu o readString. W ptli robi readString (podajc prompt 
     z dopisanym stringiem oznaczajacym co mozna wpisac i jaki jest def, np. dla
     prompt = "Are you hungry ?", default = true bdzie wypisywa 
     "Are you hungry ? [Y/n] ", dla default = false bdzie
     "Are you hungry ? [y/N] ") tak dugo a w kocu dostanie "y", "Y" 
     (daj true), "n", "N" (daj false), "" (daje def). */
  public boolean readBoolean(String prompt, boolean def)
  {
    String finalPrompt;
    finalPrompt = prompt + (def ? " [Y/n] " : " [y/N] ");
    do {
      String input = readString(finalPrompt, true);
      if (input.equals("y") || input.equals("Y") || (input.equals("") && def)) 
        return true; else
      if (input.equals("n") || input.equals("N") || (input.equals("") && (!def)))
        return false;
    } while (true);
  }  
  
  /** Call this to make sure that the user interface closed it's windows 
     and freed it's threads.
     In other words, before you exit your main() you should call this
     if you want the program to terminate. If you won't implementations
     that initiate some threads (like {@link IOSwing} that runs AWT
     threads) will prevent the program from terminating. 
     
     <p>In IOSystem class this method is implemented as NOP 
     (because in some subclasses, like {@link IOStd}, this is sufficient). */
  public void dispose() { }
}

/*
 * $Log: IOSystem.java,v $
 * Revision 1.3  2004/02/19 21:54:20  michalis
 * + klasa ClientApplet ostatecznie zrobiona - dziaa !
 * * useStyles wycignite jako parametr konstruktorw IOSwing
 * + JamyUtils z pomocniczymi rzeczami gromadzcymi parametry do interfejsw,
 *   teraz i serwer i klient i single maj peen zestaw 3 parametrw --console, --simple-swing, --stylish-swing
 * * odpowiednie uaktualnienia w dokumentacji
 *
 * Revision 1.2  2004/02/09 23:08:56  michalis
 * + GPL license clause i copyrighty (moje i gdzieniegdzie Michaa, sorry Rafa
 *   ale naprawd nic nie zostao) do wszystkich *.java
 *
 * Revision 1.1  2004/02/05 20:28:22  michalis
 * * Klasy IOSystem/Std/Swing przeniesione do pakietu common.iosystem.
 *   Byy uywane take przez single/ wic ich dotychczasowa lokalizacja
 *   w client/ bya za. Ponadto zaraz bd uywane take przez server/.
 *
 * Logs as client/IOSystem.java:
 * Revision 1.12  2004/02/05 20:05:24  michalis
 * * Zrobiem eby w logach CVSa zapisanych w plikach byo wszdzie TO-DO
 *   z mylnikiem w rodku. Wprawdzie nieznacznie rozsynchronizowuje to
 *   zapisane w plikach logi z wynikiem 'cvs log' ale za to grep TO-DO
 *   (gdzie w wyrazie TO-DO naley usun mylnik) dziaa jak trzeba.
 *
 * Revision 1.11  2004/01/31 21:43:52  michalis
 * * shit; one more fix to lineEndCancel; OK, now it really works as expected
 *
 * Revision 1.10  2004/01/31 21:40:42  michalis
 * + IOSystem and subclasses: method lineEndCancel
 * * JamySingle and Client use lineEndCancel to avoid some bug
 *
 * Revision 1.9  2004/01/31 20:10:28  michalis
 * + IOSystem: some TO-DO with comments moved from UserCommand to IOSystem
 * * UserCommand docs improved
 * * UserCommand.s renamed to commandPattern
 * + PlayersMessages
 * * World.messageToPlayersInLocation moved to PlayersMessages.tellEveryoneInLocation
 * * World.action changed to use PlayersMessages (much cleaner code now!)
 * * Creature.action and doAction use PlayersMessages
 * * UserCommand.execute interface changed:
 *   - world (excessive, one can always use player.world)
 *   + playersMessages (to tell other players about effects of this command)
 *   + returns String (tell command invoker about effects of this command)
 * * every command's execute changed to use new interface:
 *   nothing uses JamySingle.io, nothing may call Player.dead()
 *
 * Revision 1.8  2004/01/17 22:14:15  michalis
 * * some docs improved in JamySingle, IOSystem and IOSwing
 * * JOptionPane in IOSwing used only by swingThreadInvoke
 *
 * Revision 1.7  2004/01/17 16:51:38  michalis
 * * Minor doc updates, + IOSystem.dispose
 *
 * Revision 1.6  2004/01/16 20:01:59  michalis
 * * IOSystem dostao duo lepszy interfejs, bardziej przejrzysty i teraz
 *   implementacja podklasy IOSystem realizujcej interfejs np. przez
 *   jakiego swinga jest rzeczywicie moliwa.
 * * IOStd zostao przerobione aby dostosowa si do nowego interfejsu IOSystem,
 *   zyskao te pole linePrompt
 *
 * Revision 1.5  2004/01/16 00:27:59  michalis
 * * IOSystem.readIntPrompt fixed (println -> print), minor adjustments to IOStd
 *
 * Revision 1.4  2004/01/15 21:46:28  michalis
 * * IOStd implemented
 *
 * Revision 1.3  2004/01/15 21:12:03  michalis
 * * IOSystem and IOStd fully specified and IOStd almost implemented
 *
 * Revision 1.2  2004/01/15 20:35:12  michalis
 * * docs improved, doc/Makefile changed to generate docs for jamy_i_nory.client
 *
 * Revision 1.1  2004/01/15 20:04:36  michalis
 * + jamy_i_nory.client.IOSystem
 * + jamy_i_nory.client.IOStd
 * + JamySingle.io, JamySingle.newPlayerInteractive (from Player.newInteractive)
 * * JamySingle and UserCommand use JamySingle.io instead of System.in/out
 *
 */
