/*
  Copyright 2004 Michalis Kamburelis.

  This file is part of "Jamy & Nory".

  "Jamy & Nory" is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  "Jamy & Nory" is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with "Jamy & Nory"; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package jamy_i_nory.game;

import jamy_i_nory.common.*;

/** Istota; co co moe by sterowane przez gracza lub przez
   komputer. Nie owijajc w bawen, po prostu wsplna nadklasa
   dla Creature i Player gromadzca wsplne waciwoci i metody. */
public abstract class Being extends Ided implements Comparable {

  /** Race of this being. Przede wszystkim race supplies base statistics
     for a character, ale moe te by uywana w wielu innych mniej oczywistych
     miejscach, np. {@link World#playerStartLocation World.playerStartLocation}
     moe generowa rne miejsce startowe w zalenoci od rasy gracza. */
  final public Race race;
  final public StatModifier raceRandomStat;
  
  /** Being's name. Set only once at object creation. null if being has no name. */
  final public String name;
  
  /* used items ---------------------------------------- 
     eby doda nowy used item: 
     - dodaj stala USED_xxx
     - USED_COUNT++
     - dodaj funkcje 
       public XxxType xxx() { return (XxxType) fUsedItems[USED_xxx]; }
     - rozszerz usedItemName, usedItemInputName
     - rozszerz classifyUsedItem
     
     Ponisza implementacja wyglda jak co strasznie "brudnego" -
     kiepskie bezpieczestwo typw, sporo upcasting. Osiga ona jednak
     dwa cele: 
     1. dodanie nowego used item jest bardzo proste.
     2. operacje jednorodne na wszystkich used items s moliwe.
     W rezultacie dodanie nowego used item to tylko kilka
     lokalnych zmian w poniszym kodzie i _wszystko_
     powinno "samo z siebie" dziaa jak trzeba, cznie z takimi rzeczami jak
     polecenia "upu" w JamySingle, metody wear, drop, fullStats.
  */
  
  final static public int USED_WEAPON = 0;
  final static public int USED_ARMOR = 1;
  final static public int USED_LEFT_RING = 2;
  final static public int USED_RIGHT_RING = 3;
  final static public int USED_COUNT = 4;

  /** Weapon this character wields. Can be null. */
  public Weapon weapon() { return (Weapon) fUsedItems[USED_WEAPON]; }
/*  public void wearWeapon(Weapon weapon) { wear(USED_WEAPON, weapon); }*/
  
  /** Armor this character wears. Can be null. */
  public Armor armor() { return (Armor) fUsedItems[USED_ARMOR]; }
  
  /** Left ring used by character. Can be null. */
  public Ring leftRing() { return (Ring) fUsedItems[USED_LEFT_RING]; }
  
  /** Right ring used by character. Can be null. */
  public Ring rightRing() { return (Ring) fUsedItems[USED_RIGHT_RING]; }
  
  /** Zwraca jako co mona nosi dany przedmiot na podstawie klasy
     przedmiotu. Np. jeeli (it instance Weapon) zwraca USED_WEAPON.
     Jeeli przedmiotu nie mona nosi zwraca -1.
     
     <p>W przypadku jeeli przedmiot mona nosi na wicej ni 1 sposb
     (np. Ring mona nosi jako USED_LEFT_RING lub USED_RIGHT_RING)
     stara si zwrci to miejsce ktre jest wolne (np. jeeli it jest klasy 
     Ring i usedItem(USED_LEFT_RING) == null a usedItem(USED_RIGHT_RING) != null
     to jest gwarantowane e zwrci USED_LEFT_RING. Wpp. (jeeli adno
     miejsce nie jest wolne) zwrci arbitralnie jedno z tych zajtych miejsc. */
  public int classifyUsedItem(Item it)
  {
    if (it instanceof Weapon) return USED_WEAPON;
    if (it instanceof Armor) return USED_ARMOR;
    if (it instanceof Ring) {
      /* wkadamy piercie na woln rk. jeli obie zajte, wkadamy
         na praw. */
      if (usedItem (USED_LEFT_RING) == null) 
        return USED_LEFT_RING; else
        return USED_RIGHT_RING;
    }
    return -1;
  }
  
  /** Plecak postaci. Kada posta przez cay czas trwania gry
     bdzie miaa jeden plecak (innymi sowy, pole poniej zostanie
     utworzone dla kadej postaci, ponadto jest final, wic m.in.
     nigdy nie bdzie null). Plecak postaci o specialName!=null jest 
     w pewnym stopniu wyjtkowy: jest podpisany jego specialName.

     <p>Kiedy posta ginie upuszcza swj plecak w miejscu w ktrym zgina
     (ale kiedy jaki gracz na chwil odczy si od serwera gry
     to oczywicie nie traci zawartoci swojego plecaka...). */
  final public ItemsBag backpack;
  
  /* Will be initialized to nulls. */
  private StatModifyingItem[] fUsedItems = new StatModifyingItem[USED_COUNT]; 
  
  /** Name of used item number. This a short desciption
     (preferably one word, at most a few words), lowercase. */
  final static public String[] usedItemName = 
    { "bro", "zbroja", "piercie na lewej rce", "piercie na prawej rce" };
    
  /** Input-name of used item number. This is one word (no white chars),
     no polish ogonki chars. */
  final static public String[] usedItemInputName = 
    { "bron", "zbroja", "lpierscien", "ppierscien" };
  
  /** Zapewnia jednorodny dostp do wszystkich pl okreslajcych
     rzeczy ktre posta nosi/uywa. 
     Dla USED_WEAPON zwraca weapon. Dla USED_ARMOR zwraca armor. Itd.
     Argument = jedna ze staych USED_xxx czyli int z przedziau 
     0 .. USED_COUNT-1. */ 
  public StatModifyingItem usedItem(int num)
  {
    return fUsedItems[num];
  }
  
  /** Funkcja bliniacza do {@link #usedItem}. Ustawia odpowiedni used item.
     Uywajac jej pamitaj e value musi byc odpowiedni podklas Item. */
  public void setUsedItem(int num, StatModifyingItem value)
  {
    fUsedItems[num] = value;
  }

  /** Replaces item with given one and returns reference to it */
  public StatModifyingItem replace (int usedItemNum, StatModifyingItem item)
  {
      StatModifyingItem      it = usedItem (usedItemNum);

      setUsedItem (usedItemNum, item);
      return it;
  }
  
  /** Wear given item. Always give item of appropriate class
     ({@link Armor} for USED_ARMOR etc.). */
  public void wear(int usedItemNum, StatModifyingItem item)
  {
      dropOnFloor (replace (usedItemNum, item));
  }
  
  /** Wear given item. This method searches inventory and current location
   * for item with given ID and replaces currently used one with it.
   * If operation succeeded returns something in 0..USED_COUNT-1
   * indicating "as what" we use given item. Else it returns -1. 
     
     <p>Dodaje odpowiednie putMessage dla wszystkich zainteresowanych,
     w szczeglnoci gwarantuje dodanie przynajmniej jednego putMessage 
     dla this. */
  public int wearById (long id)
  {
      Item item;
      boolean onfloor;      /* odkladamy na podloge czy do plecaka? */

      /* szukamy w plecaku... */
      onfloor = false;
      item = (Item) backpack.items().byId (id);
      /* jak nie ma to szukamy na pododze */
      if (item == null) {
          onfloor = true;
          item = (Item) nowLocation().items().byId (id);
      }
      
      if (item != null) {                                      
          int result = classifyUsedItem(item);
          if (result == -1) 
          {
            putMessage("Przedmiotu " +item +" nie moesz zaoy ani "
              +"uywa jako broni.");
            return result;
          }
          
          /* dopiero teraz wiemy e moemy zrzutowa item na StatModifyingItem */
          StatModifyingItem it = (StatModifyingItem) item;
          
          /* niektre metody mog dodawa odpowiednie putMessage wic jest
             wane eby wywoywa tu metody w takiej kolejnoci w jakiej 
             zachodz odpowiednie zdarzenia: najpierw podnie przedmiot
             (ew. wyjmij z plecaka), potem za, potem od to co zdje. */
          if (onfloor) pickFromFloor (it);
          else backpack.items().remove (it);
          
          StatModifyingItem rep = replace (result, it);
          putMessage("Odtd uywasz przedmiotu " +it + " jako " 
            +usedItemName[result] +".");
                    
          if (rep != null) 
          {          
            /* jeli co zamienilimy, odkladamy gdzie trzeba */
            if (onfloor) dropOnFloor (rep);
            else backpack.items().add (rep);
          }
          return result;
      } else
      {
        putMessage("Ani w swoim plecaku ani na ziemi nie znajdujesz "
          +"przedmiotu o id = " +id +".");
        return -1;
      }
  }

  /** Zdejmij i drop given item on the floor. If usedItem(usedItemNum) != null
     this method zdejmuje i odklada na ziemi odpowiedni przedmiot wywoujc
     odpowiedni putMesage wszystkim zainteresowanym (w szczeglnoci jest
     wtedy gwarantowane e this dostanie odpowiedni putMessage), 
     wpp. this is valid NOP. */
  public void drop(int usedItemNum)
  {
      dropOnFloor (replace (usedItemNum, null));
  }

  /** Helper - put item on floor. If i != null to kadzie przedmiot
     na ziemi i powiadamia wszystkich zainteresowanych przez putMessage, 
     w szczeglnoci this dostaje odpowiedni putMessage. 
     If i == null, works like NOP. */
  private void dropOnFloor (Item i)
  {
    if (i != null) 
    {
      nowLocation().items().add(i);
      tellEveryoneInNowLocation(new MessageFormatter(
        "%1 upuszcza przedmiot " + i + ".", this));
      putMessage("Upuszczasz przedmiot " + i + ".");
    }
  }
  
  /** Szuka wrd przedmiotu na licie usedItem() o podanym id. Zwraca 
     numer przedmiotu 0..USED_COUNT-1 jeeli go znalaz, wpp. zwraca -1. */     
  public int findUsedItem(long id)
  {
    for (int i = 0; i < USED_COUNT; i++) {
      Item it;
      it = usedItem (i);
      if (it != null && it.id == id) return i; 
    }
    return -1;
  }
  
  /** Zwraca przedmiot o danym id posiadany przez posta. Szuka wrd
     przedmiotw w plecaku i uywanych przedmiotw. Zwraca null jeli
     nie ma takiego przedmiotu. */     
  public Item findItem(long id)
  {
    /* najpierw wyposaenie */
    int i = findUsedItem(id);
    if (i != -1) return usedItem(i);

    /* potem plecak */
    return (Item) backpack.items().byId (id);
  }

  /** Drop given item on the floor.
   *  This method searches our inventory and equipment for object with
   *  specified id, drops it and returns it. 
   
     Uaktualnia odpowiednie putMessage wszystkim zainteresowanym,
     w szczeglnoci gwarantuje wykonanie co najmniej raz this.putMessage. */
  public Item dropById (long id)
  {
      /* najpierw wyposaenie */
      int i = findUsedItem(id);
      if (i != -1)
      {
        Item it = usedItem(i);
        drop (i);
        return it;
      }

      /* potem plecak */
      Item it = (Item) backpack.items().byId (id);
      if (it != null) {
          backpack.items().remove (it);
          dropOnFloor (it);
          return it;
      }
      
      putMessage("Nie posiadasz przedmiotu o id = " +id +".");
      return it;
  }

  /** Pick up specified object and put into inventory. item must not be null.
     Wszyscy zainteresowani (a w szczeglnoci na pewno this) dostaj 
     odpowiednie putMessage. */
  private void pickFromFloor (Item item)
  {
     nowLocation().items().remove(item);
     tellEveryoneInNowLocation(new MessageFormatter(
       "%1 podnosi przedmiot " +item +".", this));
     putMessage("Podnosisz " + item + ".");
  }

  /** Pick up object specified by id and put into inventory.
   *  Returns reference to picked Item or null if there is no
   *  item with specified id.
   
     <p>Wszyscy zainteresowani (a w szczeglnoci na pewno this, nawet
     jeli nie znaleziono przedmiotu o takim id) dostaj odpowiednie 
     putMessage. */
  public Item pickById (long id)
  {
      Item it = (Item) nowLocation().items().byId(id);

      if (it != null) {
          pickFromFloor (it);
          backpack.items().add (it);
      } else
        putMessage("Nie znalaze na ziemi przedmiotu o id = " +id +".");
      return it;
  }
  
  /** Szuka przedmiotu typu ItemsBag o podanym id wrd przedmiotw lecych
     na ziemi i w backpack. Jeli go znajdzie to rozpakowuje go,
     tzn. wyciga z niego wszystkie przedmiotu i wyrzuca je na ziemi
     (jesli przedmiot znalaz na ziemi) lub do backpack (jeli przedmiot
     znalaz w backpack).
     
     <p>Wszyscy zainteresowani (a w szczeglnoci na pewno this, nawet
     jeli nie znaleziono przedmiotu o takim id lub taki przedmiot nie by
     typu ItemsBag lub by pusty w rodku itd. itp.) dostaj odpowiednie 
     putMessage. */
  public void unpackItem(long id)
  {
    /* szukaj wrd lecych na ziemi */    
    Item maybeBag = (Item) nowLocation().items().byId(id);
    boolean onFloor = true;    
    
    if (maybeBag == null) {
      /* szukaj we wasnym plecaku */
      maybeBag = (Item) backpack.items().byId(id);
      onFloor = false;
      
      if (maybeBag == null) {
        putMessage("Nie znalaze przedmiotu o id = " +id +" na ziemi ani "+
          "w swoim wasnym plecaku.");
        return;
      }
    }
    
    if (! (maybeBag instanceof ItemsBag)) 
    { 
      putMessage("Przedmiotu " +maybeBag +" nie mona w aden sposb rozpakowa.");
      return;
    }
    
    ItemsBag bag = (ItemsBag) maybeBag;    
    
    if (bag.items().size() == 0) 
    {
      putMessage("Przedmiot " +bag +" jest ju pusty.");
      return;
    }
    
    int bagStartSize = bag.items().size();
    
    if (onFloor) 
      tellEveryoneInNowLocation(new MessageFormatter(
        "%1 wypakowuje przedmiot " +bag +" na ziemi.", this));
    putMessage("Rozpakowujesz przedmiot " +bag + 
      (onFloor ? " na ziemi" : "") +".");
    
    while (bag.items().size() > 0)
    {
      Item item = (Item) bag.items().removeFirst();
      if (onFloor) 
        dropOnFloor(item); else
      {
        backpack.items().add(item);
        putMessage("Wypakowujesz przedmiot " +item 
          +" i wkadasz go do swojego plecaka.");
      }
    }
    
    putMessage("Wypakowae " +bagStartSize +" przedmiotw.");
  }
  
  /* ---------------------------------------- */

  /** How healthy is this being now ?
     life == {@link #fullStats fullStats()}.maxLife() means that this Being
     is "fully healthy". life > {@link #fullStats fullStats()}.maxLife()
     (that <b>is</b> an allowed value for life) means that this being
     is somehow (e.g. magically) more healthy than "fully healthy".
     life <= 0 means that this being is quite dead...

     <p>Something about scale: we assume here that an average human
     has maxLife() = 20.

     <p>Note: do not dare to move this field to
     {@link StatModifier StatModifier}. Wearing magical items etc. should
     modify {@link StatModifier StatModifier}.maxLife(), <b>not</b>
     life. */
  public int life;

  /** wiat w ktrym porusza si dana posta. Zadany at object creation.
     Jeeli Being jest klasy Player to musi by elementem
     world.players(). Jeeli Being jest klasy Creature to
     musi by elementem world().creatures(). */
  final public World world;
  
  /** Zwraca bazowe informacje o graczu - takie ktre mog by 
     wycigniete z gracza nawet przed wywoaniem pierwszego setNowLocation()
     w konstruktorze i po ostatnim setNowLocation(null) w dead().
     Tylko klasie Being wiemy jakie to mog by informacje, ponadto 
     z zewntrz powiniene zawsze uywa normalnych toString - dlatego
     ta metoda jest prywatna. 
     
     <p>TODO: uycie toStringBase nie jest zbyt dobre bo niestety informacja
     jest doc okrojona. W szczeglnoci "ycie" i "nastawienie" nie mog
     by wypisywane (bo te wymagaj do dziaania nowLocation() != null
     bo fullStats wymaga nowLocation() != null). Tym niemniej taki zapis
     jest absolutnie bezpieczny - nie da si spowodowa bdu robic w podklasie
     co co "nie przyszo mi do gowy w momencie implementowania Being".
     Poszukaj wic lepszego rozwizania ale *nie* kosztem bezpieczestwa kodu. */
  private String toStringBase()  
  {
    return "[" +id +"] " +(name != null ? "\"" +name +"\" " : "") +race;
  }

  private Location fNowLocation;
  
  /** Get Location where this being currently is.
     Cannot be null (except when you call this from Being's constructor
     or when you use this after dead() returned true).
     Every being in our game is always in exactly one location.
     Must be a member of {@link #world world}.locations(). */
  public Location nowLocation() { return fNowLocation; }
  
  /** Set current being location. value must not be null.
     link moe by null, jeeli nie jest to musi on by na licie
     nowLocation().links() i link.target == value.

     <p>For creatures and players, this method will take care of
     {@link #nowLocation nowLocation()}.creatures/players()
     (it can delete self from nowLocation, 
     change nowLocation to value and then add self to new nowLocation).
     
     <p>Note for implementors of subclasses: this method
     is called from constructor. It means that if you override
     it in subclass then before calling super.setNowLocation
     nowLocation() can return null. This method is also called from
     {@link #dead} and it means that value CAN be null in this 
     <b>one, special, internal</b> situation. W tych dwch przypadkach
     w ogle musisz by gotowy na to e obiekt moe by w do kiepskim
     stanie co znaczy tyle e powiniene wykorzysta tylko value (albo
     nowLocation, w zalenoci od tego ktre jest not-null) i NIE uywa
     adnych innych wasnoci obiektu this w tych dwch wewntrznych przypadkach.
     W tych dwch przypadkach link na pewno bdzie null.
     
     <p>This method calls appropriate players putMessage with
     appropriate values (gracze widz kto wchodzi do pomieszczenia 
     i kto wychodzi z pomieszczenia), do gracza this te zostanie wysane
     odpowiednie putMessage. 
     
     <p>Tym niemniej jeeli setNowLocation
     zostanie wywoane z konstruktora (a wic w czasie gdy nowLocation
     == null) to wypisze komunikat ktry uyje tylko toStringBase()
     wypisujcego niepene informacje a jeeli zostanie uyte z dead() 
     (a wic w czasie gdy value == null) to nie wypisze NIC (wypisywaniem
     komunikatu o mierci zajmujemy si w dead() aby mc tam uy
     toString() zamiast toStringBase()). */
  public void setNowLocation(Location value, LinkToLocation link) 
  { 
    Location oldLocation = nowLocation(),
             newLocation = value;

    if (nowLocation() != null)
    {
      /* sprawdz czy newLocation != null aby sprawdzi e nie jest to 
         ostatnie wywoanie setNowLocation z dead(). */
      if (newLocation != null) 
        tellEveryoneInNowLocation(new MessageFormatter(
          "%1 wychodzi z pomieszczenia" +
          (link != null ? link.toParenthString("przejciem") : "") + ".", 
          this));
    }

    fNowLocation = value; 
    
    if (oldLocation != null && newLocation != null && link != null)
    {
      putMessage("Podasz przejciem [" + link.shortName +"]" +
        link.wayBackToParenthString("przejcie z powrotem to") + ".");
    }
    
    if (nowLocation() != null)
    {
      if (oldLocation == null)     
        tellEveryoneInNowLocation(toStringBase() + " pojawia si w pomieszczeniu."); 
      else
      {
        tellEveryoneInNowLocation(
          new MessageFormatter("%1 wchodzi do pomieszczenia" +
            (link != null ? link.wayBackToParenthString("z przejcia") : "") + ".", 
            this));
      }
    }
  }
  
  /** Just a shortcut for {@link Location#toString(Being,boolean)
     nowLocation().toString(this, false)} */
  public String nowLocationToString()
  {
    return nowLocation().toString(this, false);
  }  

  /** Obvious constructor. Tworzy now posta w grze.
  
     <p>Ten konstruktor nie tylko ustawia
     pole {@link #world world} na podan warto ale sprawia te e
     warunki podane w specyfikacji pola {@link #world world}
     s spenione, tzn. {@link Player Player} bdzie automatycznie dodany
     do listy {@link World#players world.players()} a
     {@link Creature Creature} bdzie automatycznie dodane do
     {@link World#creatures world.creatures()} (oraz do odpowiednich
     list w startLocation). 
     
     <p>Poniewa ustawia nowLocation na startLocation (ktre oczywicie
     musi by not null) wic za porednictwem setNowLocation wszystkie
     istoty w startLocation s powiadamiane e w lokacji pojawia si
     nowa posta. */
  public Being(World aWorld, Location startLocation, Race aRace, String aName)
  {
    this(aWorld, startLocation, aRace, aName, 
      new ItemsBag (2.0f, "plecak", null,   
        (aName != null ?  "Plecak postaci " +aName : null)),
      0, aRace.randomStat());
      
    /* Poniewa eby obliczy life potrzebujemy wywoa fullStats() wic
       uywamy maej sztuczki - podajemy powyej life = 0 (czyli tworzymy
       chwilowo martwego gracza, ale nic tego nie zauway) i ustawiamy
       je dobrze dopiero tutaj. */
    life = fullStats().maxLife();
  }
  
  /** Ten konstruktor take tworzy now posta w grze ale umoliwia
     podanie niektrych parametrw pocztkowych postaci w swobodny sposb.
     W ten sposb mona stworzy posta niezgodnie z zasadami gry -
     aby stworzy now posta zawsze uywaj prostszego konstruktora,
     {@link Being#Being(World, Location, Race, String)}.
     Ten konstruktor jest przydatny gdy chcesz doda do gry posta
     zachowan w InactivePlayer - jest uywany z {@link Player#Player(World, 
     InactivePlayer)}. */
  public Being(World aWorld, Location startLocation, Race aRace, String aName,
    ItemsBag aBackpack, int aLife, StatModifier aRaceRandomStat)
  {
    world = aWorld;    
    name = aName;
    race = aRace;
    raceRandomStat = aRaceRandomStat;
    
    /* setNowLocation MUSI by uyte przed uyciem jakiejkolwiek innej metody
       obiektu Being (wszystkie pozostae metody Being dziaaj przy zaoeniu 
       e nowLocation != null). */
    setNowLocation(startLocation, null);
    
    life = aLife;
    backpack = aBackpack;
    calcNextAction ();
  }

  /** Oblicza wartoci cech postaci. Jest final bo jest uywane z konstruktora
     Being (wic zanim jakakolwiek podklasa ma szans si zainicjowa). */
  final public StatModifier fullStats ()
  {
    StatModifier result = new StatModifier ();

    result.apply (race.stat);
    result.apply (raceRandomStat);
    
    for (int i = 0; i < fUsedItems.length; i++)
      if (fUsedItems[i] != null) result.apply (fUsedItems[i].stat);
      
    result.apply (nowLocation().statModifier());
      
    result.makeFinalStat();
      
    return result;
  }
  
  /** Nastawienie tego Being do target.
     Zwraca inta zdefiniowanego w klasie {@link Attitude Attitude}. */
  public abstract int realAttitude(Being target);

  /** Nastawienie tego Being do target widziane z punktu widzenia target.
     Zwraca inta zdefiniowanego w klasie {@link Attitude Attitude}.
     Zwraca UNKNOWN jeeli this jest co najmniej tak mdre jak 
     target, wpp. zwraca realAttitude(target). */
  public int observedAttitude (Being target)
  {
    /* najpierw wywoaj realAttitude, dopiero potem (tylko jeli result
       nie jest UNKNOWN) porwnaj inteligencje. To dlatego e wywoania
       fullStats() s stosunkowo kosztowne czasowo a realAttitude nie. */
    int result = realAttitude(target);
    if (result != Attitude.UNKNOWN &&
        fullStats().intelligence() >= target.fullStats().intelligence())
      result = Attitude.UNKNOWN;
    return result; 
  }

  /** Zwraca jak to Being jest postrzegane przez inny obiekt klasy
     Being ktry znajduje si w tej samej lokacji. Zwraca adny string
     ktry moemy pokaza userowi; bez adnych nowych linii. 
     Moe uwzglednia szczeglny przypadek gdy observer == this
     i wtedy wypisywa peniejsz informacj. Ale zasadniczo pokazywana
     tu informacja powinna by do krtka, tylko kilka podstawowych rzeczy. 
     Peniejsz informacj moesz dostac z {@link #toDetailString} */
  public String toString(Being observer)
  {
    StatModifier f = fullStats();

    StringBuffer result = new StringBuffer(toStringBase());    
    result.append(" {");
    result.append("ycie: " +life +"/" +f.maxLife());
    if (this != observer)
    {
      int att = observedAttitude(observer);
      result.append("; nastawienie: " +Attitude.toStr[att]);
    }
    result.append("}");

    return result.toString();
  }

  /** Zwraca {@link #toString(Being) toString(this)} czyli wypisuje stosunkowo 
     pelna informacje o danej istocie. Ta metoda jest final - nie pokrywaj jej, 
     zamiast tego pokryj {@link #toString(Being)}. */
  final public String toString()
  {
    return toString(this);
  }
  
  /** Podobnie jak {@link #toString} ale wyswietla wielolinijkowa informacje
     ktora zawiera absolutnie wszystko co observer wie o this.
     Zakonczony zawsze newlinem. */
  public String toDetailString(Being observer)
  {
    StatModifier f = fullStats();
    
    StringBuffer result = new StringBuffer();
    
    result.append(
      (name != null ? "\"" +name +"\" " : "") +
      race +Utils.nl);
    result.append("  ycie: " +life +"/" +f.maxLife() +Utils.nl);
    result.append("  Cechy: " + fullStats() + Utils.nl);
    result.append("  Opnienie (w milisek): " +getDelay() +Utils.nl);
    
    for (int i = 0; i < USED_COUNT; i++)
      if (usedItem(i) != null) 
        result.append("  " +Utils.capitalize(usedItemName[i]) +": " 
          +usedItem(i) +Utils.nl);
          
    if (weapon() == null) 
      result.append("  Bez broni (" +race.unarmedDamage +")" +Utils.nl);
    if (race.skinDefense != 0)
      result.append("  Ochrona skry: " +race.skinDefense +Utils.nl);
          
    if (this != observer) {
      result.append("  Nastawienie: " +Attitude.toStr[observedAttitude(observer)] 
        +Utils.nl);
    } else {
        /* Sami o sobie wiemy wszystko, m.in. co mamy na plecach */
        result.append (
                Utils.collectionToMultiLineString (
                    backpack.items(),
                    "  Zawarto plecaka:",
                    "    "
                    )
                );
    }

    /* TODO: powinnimy uywa jakiej bardziej wyspecjalizowanej metody
       ktra bierze pod uwag stan wiedzy observer, jak dobrze observer
       zna this. */

    return result.toString();
  }
  
  /** Wywoluje toDetailString(this). Nie pokrywaj w podklasach - pokrywaj
      {@link #toDetailString(Being)} */
  final public String toDetailString()
  {
    return toDetailString(this);
  }
  
  /** Mwi do kadej istoty w komnacie nowLocation() message poprzez 
     zrobienie putMessage do odpowiednich graczy.
     
     <p>W tym momencie potwory (podklasy Creature) w aden sposb nie 
     mog "usysze" tych wiadomoci
     (a to byoby fajne gdyby mogy - moglibymy wtedy wmontowa Eliz
     do naszego programu i w ogle swobodnie rozmawianie z NPCami,
     jak w "Wizardry"). Ale (dziki tej metodzie) potwory mog spokojnie
     same co mwi (i w ten sposb zachowujemy nasz zasad ze
     postacie sterowane sztuczn inteligencj s moliwie nieodrnialne
     od graczy). 
     
     <p>Nazywa si "say" a nie "tell" eby nie mylio si z 
     tellEveryoneInLocation ktre jest oglniejsze bo nie dokleja do 
     message czego w rodzaju "Stworzenie <this> mwi ". */
  public void sayEveryoneInNowLocation(final String message)
  {
    tellEveryoneInNowLocation(
      new MessageFormatter("Stworzenie %1 mwi \"" +message +"\".", this));
  }
  
/*  TODO jak tylko bdzie potrzebne:
  public void sayEveryoneInNowLocation(final MessageGenerator mesgGenerator, 
    PlayersMessages playersMessages)
*/      
  
  /** Skrt do nowLocation().tellEveryone(mesgGenerator, this).
     Czyli: dla kadego gracza w tej samej lokacji (ale nie do samego
     siebie) zrb player.putMessage(mesgGenerator.genString(player))  */
  public void tellEveryoneInNowLocation(MessageGenerator mesgGenerator)
  {
    nowLocation().tellEveryone(mesgGenerator, this);
  }
  
  /** Skrt do nowLocation().tellEveryone(message, this).
     Czyli: dla kadego gracza w tej samej lokacji (ale nie do samego
     siebie) zrb player.putMessage(message)  */
  public void tellEveryoneInNowLocation(String message)
  {
    nowLocation().tellEveryone(message, this);
  }
  
  /** Po wytumaczenie po co jest ta metoda - wyjatkowo patrz specyfikacja
     tej metody w podklasie, w Player. W klasie Being (i we wszystkich 
     podklasach Being nie bdcych podklasami putMessage) ta metoda 
     to NOP. Ale mam t klas zdefiniowan tu dla Being bo to jest 
     wygodne - np. w attack zamiast pisa 
       if (b is instanceof Player) ((Player) b).putMessage
     mog pisa proste b.putMessage. */
  public void putMessage(String mesg)
  {
  }
  
  /** Given link MUST be one of nowLocation().links(). 
     This method does setNowLocation(link.target, link) so
     appropriate putMessage are done. */
  public void followLink(LinkToLocation link)
  {
    setNowLocation(link.target, link);
  }
    
  /* nextAction and related things ---------------------------------------- */

  private long fNextAction;

  /** This is timestamp (miliseconds after 01/01/1970 00:00),
     next action will be possible after this time. */
  public long nextAction() { return fNextAction; }

  /** Sort beings by delay.
     The LEAST creature in this order will make action FIRST
     (i.e. will have smallest {@link #nextAction}). */
  public int compareTo (Object o)
  {
    if (o instanceof Being)
    {
      Creature cr = (Creature) o;
      return (int) (this.nextAction() - cr.nextAction());
    } else
      throw new ClassCastException();
  }

  public long getDelay ()
  {
    /* Chc eby przecitny czowiek (dexterity 10) mia delay 2000 (2 sek).
       Dexterity 5 - 15 powinno dawa 2500 - 1500 delay ze zmian 
       proporcjonaln czyli zasadniczy wzr to 
         2000 - (dexterity-10)*100.
       Na potrzeby wynaturzonych istot ktrymi zazwyczaj gracze nie bd gra
       robimy proste clamp do 1000, 10000. */
    return Utils.clamped(2000 - (fullStats().dexterity() - 10)*100,
      1000, 10000);
    
    /* Old implementation, slightly different approach:
    final long BASEDELAY = 30000;
    return BASEDELAY / fullStats().dexterity();
    */
  }

  /** Wywouj po wykonaniu kadej akcji. Jest final bo jest uywane z 
     konstruktora Being (wic zanim jakakolwiek podklasa ma szans si 
     zainicjowa). */
  public void calcNextAction ()
  {
    /* Note: we're NOT doing (anymore) fNextAction += getDelay ();
       Adding to currentTime instead of fNextAction will fix this (nextAction 
       should be calculated with respect to last action time,
       not with respect to last action's earliest allowed time).
       It will also make it work when it is called from constructor. */
    fNextAction = System.currentTimeMillis() + getDelay ();
  }
  
  /** Checks is nextAction <= System.currentTimeMillis() */
  public boolean nextActionAllowed()
  {
    return nextAction() <= System.currentTimeMillis();
  }
  
  /* fighting ---------------------------------------- */
  
  /** Returns weapon.hit if {@link #weapon} is not null, else
     returns race.unarmedDamage. */
  public DiceRandom weaponHit()
  {
    if (weapon() != null) 
      return weapon().hit; else 
      return race.unarmedDamage;    
  }

  /** Returns race.skinDefense + (if armor not null) armor.defense. */
  public int armorDefense()
  {
    return race.skinDefense +(armor() != null ?  armor().defense : 0);
  }
  
  /** Attack target. Assumes that this can somehow reach target,
     which usually means that this and target must be in the same
     location. Calculates fighting ability as avg(strength, dexterity) + k6,
     compares, if this has greater fighting ability then target is
     damaged by max(0, this.weaponHit.random() - target.armorDefense). 
     
     <p>Returns -1 if there was no hit, else returns how much damage 
     was done (can be 0). 
     
     <p>Jeeli this == target to wypisuje odpowiedni komunikat przez 
     putMessage i zwraca -1. Innymi sowy uywajc tej metody nie mona
     zrani samego siebie.
     
     <p>Robi odpowiednim graczom ktrzy co zauwayli (i tym ktrzy
     uczestniczyli w tej walce te, naturalnie) odpowiednie putMessage.
     W szczeglnoci jest gwarantowane e zawsze w attack() jest przynajmniej
     raz wywoane putMessage dla this.
     
     <p>Nie sprawdza target.dead, a wic nie bdzie modyfikowa
     world.players() lub world.creatures() dla jakiej location. 
     To jest potrzebne bo ta metoda moe by wywoywana z 
     {@link Creature#action}. */
  public int attack(Being target)
  {
    if (this == target) 
    {
      putMessage("Nie moesz zaatakowa samego siebie. Nazywajmy rzeczy po "
        +"imieniu: jeli chcesz si zabi to uyj specjalnego polecenia " 
        +"\"samobojstwo\".");
      return -1;
    }
    
    DiceRandom d = new DiceRandom(1, 6);
    StatModifier thisS = fullStats();
    StatModifier targetS = target.fullStats();
    int fightThis   = (  thisS.strength() +   thisS.dexterity()) / 2 + d.random();
    int fightTarget = (targetS.strength() + targetS.dexterity()) / 2 + d.random();
    
    int result;
    String mesgEveryone;
    String mesgThis;
    String mesgTarget;
    
    if (fightThis > fightTarget)
    {
      int damage = this.weaponHit().random() - target.armorDefense();
      if (damage<0) damage = 0;
      target.life -= damage;

      mesgEveryone = "ranic za " +damage +" punktw";
      mesgThis = "trafi za " +damage +" punktw";
      mesgTarget = "i rani za " +damage +" punktw";

      result = damage;
    } else
    {
      mesgEveryone = "ale chybi";
      mesgThis = "chybi";
      mesgTarget = "ale chybi";
        
      result = -1;
    }
    
    /* zrb odpowiednie putMessage wszystkim zainteresowanym */      
    nowLocation().tellEveryone(new MessageFormatter(
      "%1 zaatakowa %2 " +mesgEveryone +".", this, target), this, target);
    putMessage("Twj atak na " +target.toString(this) +" " +mesgThis +".");
    target.putMessage("Zaatakowa Ci " +this.toString(target) +" " 
      +mesgTarget +".");
    
    return result;
  }
  
  /* dying, right after fighting ---------------------------------------- */
  
  /** Wywouj to aby sprawdzi czy being jest martwy. Sprawdzi czy life<=0. 
     NIE wykona adnych modyfikacji czegokolwiek - po prostu zwrci
     false lub true. Kiedy tylko moesz uywaj funkcji {@link #dead}
     ktra dodatkowo usuwa martwe istoty ze wiata gry. Jeeli deadCheck()
     zwrcio true to musisz jak najszybciej wywoa dead() na tym obiekcie
     aby go usunu ze wiata gry (a do tego czasu wszdzie przed zrobieniem
     na nim jakiej operacji powiniene sprawdza deadCheck()). */
  public boolean deadCheck()
  {
    return life <= 0;
  }

  /** Pokrywaj w podklasach aby zrobi co zanim being umrze lub przejdzie
     w stan nieaktywnoci, tzn. podczas tworzenia klasy {@link InactivePlayer}.
     Traktujemy tutaj umieranie i przechodzenie w stan nieaktywnoci jedn
     metod bo wymaga to wykonania podobnych czynnoci - chodzi o "odczenie"
     (unplug) gracza od reszty wiata, np. o usunicie go z listy istot
     znajdujcych si aktualnie w pomieszczeniu {@link #nowLocation}.
     Z drugiej strony s te pewne rnice - np. umierajc gracz upuszcza
     plecak na ziemie a przechodzc w stan nieaktywnoci - nie, co wynika
     z faktu e klasa {@link InactivePlayer} przechowuje stan plecaka gracza.
     Wanie po to masz parametr goingInactive.     
     Za kadym razem pokrywaj uywajc schematu 
     <pre>
     protected void beforeDeath(boolean goingInactive)
     {
       ...
       super.beforeDeath(goingInactive);
     }
     </pre>
     W ten sposb beforeDeath() nadklasy bdzie zawsze wykonywane na KOCU.
     To jest wane bo beforeDead() nadklasy nie potrzebuje elementw
     specyficznych dla podklasy ale na odwrt tak - beforeDeath() podklasy
     moe potrzebowa elementw nadklasy. */
  protected void beforeDeath(boolean goingInactive)
  {
    if (!goingInactive) 
    {
      for (int i = 0; i < USED_COUNT; i++) drop(i);
      
      /* nie upuszczaj na ziemi pustych niepodpisanych plecakw - 
         tylko by zamiecay teren */
      if (name != null || backpack.items().size() != 0)
        dropOnFloor(backpack);
    }
    
    /* setNowLocation MUSI by wywoane pod sam koniec, po wywoaniu 
       wszystkich innych metod; niektre inne metody mog nie by
       przygotowane na to e nowLocation() == null. Dlatego wanie
       dead() i goInactive() s final i mona pokrywa tylko beforeDeath(). */
    setNowLocation(null, null);
  }

  /** Sprawdzi czy being jest martwy (czy life<=0, czyli zwrci to samo co
     {@link #deadCheck}).
     Jeeli stwierdzi e tak to zanim zrobi return true wyczyci
     being z gry: zrzuci jego plecak i uywane przedmioty na ziemie 
     w nowLocation, w przypadku Player usunie go z {@link World#players 
     world.players()}, w przypadku Creature usunie go z {@link 
     World#creatures world.creatures()}.
     
     <p>Ta metoda produkuje komuniat o mierci stworzenia robic
     graczom w tej samej komnacie odpowiednie putMessage(...).
     (dla jakich specjalnych istot (jakitam Sauron itp.) bdziemy mogli 
     pokry beforeDeath aby przyj dodatkowe mechanizmy robienia putMessage, 
     np. dodatkowo dopisa wszystkim graczom w grze "Sauron zdech, 
     ziemia w caym Srdziemiu zadraa i w ogle bardzo Ci to grzeje".)
     
     <p>Do not call this method twice ! After calling this method
     with result true you should stop using this object in any way
     - in particular you should not call this method once more.
     To jest celowe ograniczenie stosowania tej metody - obiekt istoty
     ktra nie jest nigdzie w world.creatures() ani world.players()
     nie spenia podstawowych nakadanych na niego wymaga i nie powinien
     by ju w aden sposb dotykany. 
     
     <p>Nie pokrywaj tej metody - pokrywaj tylko beforeDeath().
     Na pocztku umierania gracza ta metoda wywoa beforeDeath(false). */
  final public boolean dead()
  {
    boolean result = deadCheck();
    if (result) 
    {
      /* Zrb komunikat o mierci gracza jeszcze przed wywoaniem 
         beforeDeath. To dlatego e obiekt musi by w penym stanie
         (a beforeDeath finalizuje wiele rzeczy, wyrzuca posta
         z world.players/creatures itd. - oglnie po beforeDeath nie moesz
         ju uy adnej wersji toString (poza toStringBase ale chcemy tego
         unikn ze wzgldu na to e toStringBase wypisuje bardzo okrojone
         informacje). */
      tellEveryoneInNowLocation(new MessageFormatter(
        "%1 umiera.", this));
      
      beforeDeath(false);
    };
    return result;
  }
  
  /** Wywoywane podczas konstrukcji obiektu {@link InactivePlayer} na
     podstawie tego obiektu. I znikd indziej nie powinno by uzywane.
     Ta metoda jest w klasie Being aby zaznaczy jej podobiestwo 
     do metody dead. By kiedy z tego skorzystam ale na razie ta metoda
     powinna by uywana tylko dla obiektw klasy Player.
     
     <p>Generalnie ta metoda dziaa podobnie jak dead() (gdy zwraca true)
     bo, jak wyjaniem przy opisie {@link #beforeDeath}, przechodzenie
     w stan nieaktywnoci jest bardzo podobne do umierania. Czyli,
     w skrcie (szerszy opis przy {@link #dead}):
     <ul>
       <li>gracz zostanie usunity z world.players() i nowLocation.players()
       <li>odpowiedni komunikat (przez putMessage) bdzie dodany wszystkim
         graczom w tej samej komnacie ktrzy zauwaaj zniknicie tego
         gracza
       <li>w ogle nie uywaj tego obiektu po wywoaniu goInactive() 
         (w szczeglnoci nie wywouj tej goInactive drugi raz)
       <li>ta metoda na pocztku wywouje beforeDeath(true). Nie pokrywaj
         jej w podklasach (zreszt nie moesz, jest final) - pokrywaj
         beforeDeath.
     </ul> */
  final public void goInactive()
  {
    tellEveryoneInNowLocation(new MessageFormatter(
      "%1 znika.", this));
    beforeDeath(true);
  }
}

/*
 * $Log: Being.java,v $
 * Revision 1.2  2004/02/19 21:54:23  michalis
 * + klasa ClientApplet ostatecznie zrobiona - dziaa !
 * * useStyles wycignite jako parametr konstruktorw IOSwing
 * + JamyUtils z pomocniczymi rzeczami gromadzcymi parametry do interfejsw,
 *   teraz i serwer i klient i single maj peen zestaw 3 parametrw --console, --simple-swing, --stylish-swing
 * * odpowiednie uaktualnienia w dokumentacji
 *
 * Revision 1.1  2004/02/18 14:37:28  michalis
 * * pl stringi
 * * --help output changed (to reflect that those classes will be run by bin/scripts)
 * preparations for ClientApplet:
 * * Client rozdzielony na Client i ClientStandalone
 * * znaczna cz pakietu common przeniesiona do nowego pakietu game
 * X* make jamy_i_nory_client_applet.jar
 *
 * Revision 1.80  2004/02/14 01:09:43  michalis
 * + przejcia w drug stron (LinkToLocation.wayBack, Location.createTwoSidedLinks)
 * * followLink -> Being
 * * lepsze komunkaty o przejciach w setNowLocation
 * * Troll osabiony
 *
 * Revision 1.79  2004/02/09 23:08:49  michalis
 * + GPL license clause i copyrighty (moje i gdzieniegdzie Michaa, sorry Rafa
 *   ale naprawd nic nie zostao) do wszystkich *.java
 *
 * Revision 1.78  2004/02/09 19:35:20  michalis
 * * Uaktualnienia dokumentacji
 *
 * Revision 1.77  2004/02/07 21:37:19  michalis
 * * malekie ulepszenia
 * * pusty plecak Beinga ktry ma name==null nie jest upuszczany na ziemi (bo po co?)
 *
 * Revision 1.76  2004/02/07 21:11:32  michalis
 * * StatModifier.makeFinalStat changed (bugfix: must be maxLife > 0,
 *   bez tego GrojecKiller czsto zaczyna ju jako trup)
 * * StatModifier.Modifs are named (as it was before) addModifs
 * * small improvement to Creature.action (to m.in. eliminuje problem ktry mi
 *   opisalicie e po zabiciu DungeonExplorera on jeszcze zdy (martwy)
 *   wyj z pomieszczenia)
 * + Utils.clamped
 * * Being.getDelay lepiej dostrojone
 *
 * Revision 1.75  2004/02/07 17:34:58  michalis
 * * jakies strzepy
 *
 * Revision 1.74  2004/02/06 04:44:39  michalis
 * + InactivePlayer
 * + dodatkowy konstruktor do Being
 * + dodatkowy konstruktor Player(World, InactivePlayer)
 * * beforeDeath rozszerzone o parametr goingInactive
 * + Being.goInactive
 * * listToString moved from World to Utils
 * + ServerCommand : polecenie "mrozonki"
 *
 * Revision 1.73  2004/02/05 23:27:39  michalis
 * * jeszcze mae poprawki w kwestii kiedy pisa TO-DO a kiedy to samo bez mylnika
 * * IOSwing konstruktor: parametr windowTitle
 * + Server obsuguje parametr --swing, ma pole IOSystem
 * + Utils.collectionToMultiLineString(Collection,String)
 * * Server mona ju kontrolowa interfejsem IOSystem, polecenia "x", "h", "p", "c"
 * + World.allCreatures/PlayersToString,
 * * UserCommand. "debug players/creatures" uywaj World.allCreatures/PlayersToString
 * * Poprawka w Server.ServerListener.run przy sleep
 *
 * Revision 1.72  2004/02/05 20:05:26  michalis
 * * Zrobiem eby w logach CVSa zapisanych w plikach byo wszdzie TO-DO
 *   z mylnikiem w rodku. Wprawdzie nieznacznie rozsynchronizowuje to
 *   zapisane w plikach logi z wynikiem 'cvs log' ale za to grep TO-DO
 *   (gdzie w wyrazie TO-DO naley usun mylnik) dziaa jak trzeba.
 *
 * Revision 1.71  2004/02/05 00:31:24  michalis
 * * docs improved
 *
 * Revision 1.70  2004/02/04 23:54:56  michalis
 * + dodany przedmiocik "szkatuka" do lokacji "Salon" z 3 piercieniami
 * * Being.beforeDeath - maa poprawka eby upuszcza plecak przez dropOnFloor
 *   (zeby byy komunikaty Player.putMessage)
 * * some Being interface clarifications
 * + dodane wywoanie putMessage(nowLocationToSingle()) do konstruktora Player,
 *   to pozwala nieco uproci PlayerConnection i JamySingle
 *
 * Revision 1.69  2004/02/04 22:30:32  michalis
 * * polecenie "samobojstwo" powoduje odpowiednie putMessage dla innych graczy w tym
 *   pomieszczeniu
 * + Being.findUsedItem, * Being.findItem i dropById uproszczone dziki temu
 * + Being.unpackItem
 * + UserCommand: polecenie "rozpakuj"
 *
 * Revision 1.68  2004/02/04 21:19:20  michalis
 * * polecenia "upusc" i "podnies" bardziej elegancko zapisane -
 *   powiadamianie o zmianach przez putMessage wepchnite w gd metod Being.
 *   Teraz fakt e np. polecenie "zaloz" moe w efekcie spowodowa podnie,
 *   potem waciwe za, a potem upu, jest widoczny w komunikatach dla gracza.
 *
 * Revision 1.67  2004/02/04 20:33:46  michalis
 * * lepszy komunikat bdu w UserCommand.parseId
 * * zapis komendy "zaloz" znacznie uproszczony, jej putMessage przerzucone do wearById
 *
 * Revision 1.66  2004/02/04 20:04:27  michalis
 * * wreszcie poprawione messages dla Being.attack
 * * Being.attack nie pozwala zaatakowa samego siebie
 * + UserCommand.parseId, uywane wszdzie w UserCommand.execute gdzie potrzebny id
 *
 * Revision 1.65  2004/02/04 16:21:09  michalis
 * * Being.dead wysya komunikaty uywajc toString zamiast toStringBase
 *   co jest pierwszym krokiem do wywalenia toStringBase
 *
 * Revision 1.64  2004/02/04 15:36:42  michalis
 * + Dziki pozbyciu si PlayersMessages moemy teraz wypisywa stringi
 *   o tym kto wchodzi/wychodzi z pomieszczenia w setNowLocation.
 * + Being.beforeDead, Being.toStringBase eby setNowLocation
 *   mogo take wywietla kto si pojawia (jako nowy gracz/potwr w wiecie)
 *   uywane z konstruktora Being i z dead()
 * * Server zosta uproszczony w wyniku powyszej zmiany, Being.dead te
 *   (chocia pojawia si niewielka strata funkcjonalnoci - patrz TO-DO przy
 *   toStringBase())
 *
 * Revision 1.63  2004/02/04 14:25:58  michalis
 * * dziki usuniciu PlayersMessages znowu mamy proste setNowLocation
 *   w Player i Creature i znowu mamy proste Player.dead i Creature.dead
 *
 * Revision 1.62  2004/02/04 14:00:24  michalis
 * * Being.tellEveryoneInNowLocation -> sayEveryoneInNowLocation
 *   (mniej mylca nazwa)
 * * Server powiadamia wszystkich graczy w komnacie e pojawi si nowy gracz
 * * PlayersMessages.tellEveryoneInLocation - docs updated,
 *   implemented in a cleaner way
 * + MessageGenerator
 * + MessageFormatter
 * * Being.attitude changed to realAttitude and observedAttitude
 * * Many fixes to use MessageFormatter when updating PlayersMessages.
 *   Many TO-DOs closed.
 * * Attitude.toStr translated to polish
 * * Poprawiony bug w Being.dead - Micha, nie mona tworzy PlayersMessages
 *   tu przed usuniciem gracza z gry - w czasie uywania PlayersMessages
 *   world.players() nie mog by modyfikowane. Z drugiej strony, to jest moja
 *   wina e jeszcze tego nie mona zrobi (bo to *powinno* by moliwe tylko
 *   musz zapisa inteligentniej PlayersMessages). OK, wanie mi przyszo do
 *   gowy jak to zrobi bardzo atwo i bardzo elegancko (klasa PlayersMessages
 *   w ogle nie bdzie potrzebna i unikniemy przerzucania wszdzie parametrw
 *   playersMessages) i w nastpnym commicie to bdzie zrobione (gupi, gupi,
 *   gupi jestem e od pocztku tego tak nie zrobiem).
 * * OK, serwer skunk.one.pl nie odpowiada wic nie bd commitowa, od razu
 *   zrobi to o czym mwiem powyej:
 *   + Player.getMessage, Player.putMessage
 *   + Location.tellEveryone (3 wersje)
 *   - PlayersMessages (funkcjonalno przerzucona w caoci do obiektw Player i
 *     Location)
 *   * masa uproszcze kodu (m.in. nie musimy ju przerzuca wszdzie
 *     parametrw PlayersMessages)
 *   - klasa Being.DeadAnswer (useless, teraz Being.dead() znowu zwraca prostego
 *     booleana)
 *   * w ogle Being.dead() jest teraz o niebo prostsze.
 *   Skoczyem! PlayersMessages usunite razem z wieloma niewygodami.
 *
 * Revision 1.61  2004/02/04 09:09:22  michalis
 * + contract of StatModifier.toString specified
 * + Location.statModifier(), odpowiednio rozszerzone Location.toString
 * * Being.fullStats odpowiednio rozszerzone
 * + lokacja Bagno (eby przetestowa Location.statModifier()),
 * * Goblin przeniesiony z tyka dinozura na zachodni balkon,
 *   Troll przeniesiony z tyka dinozura na wschodni balkon
 *   (eby ZPL nie pokazywa lokacji o nazwie "tyek dinozaura")
 *
 * Revision 1.60  2004/02/02 02:16:12  michalis
 * + PlayersMessages(Player,String) (unused for now)
 * + Item.useItem
 * + Item.ElfKillerWhistle implementujcy useItem
 * * Location.useItem wywouje Item.useItem
 * + Egzemplarz Item.ElfKillerWhistle dodany do salonu
 * * some docs clarified in Being
 *
 * Revision 1.59  2004/02/01 20:57:59  michalis
 * + UserCommandBlackTokens, wszystkie dotychczasowe komendy zmienione jako
 *   dziedziczce z UserCommandBlackTokens zamiast bezporednio z UserCommand
 * + Being.tellEveryoneInLocation
 * + UserCommand: komenda "powiedz"
 * + Goblin mwi e jest godny (naprawd)
 *
 * Revision 1.58  2004/02/01 19:58:46  michalis
 * + Being.classifyUsedItem
 * * wearById zaimplementowane bardziej elagancko uywajc Being.classifyUsedItem,
 *   zwraca teraz int
 * * polecenie "zaloz" zwraca teraz adny string - co i jako co jest noszone
 *
 * Revision 1.57  2004/02/01 19:14:07  michalis
 * * poleceine "uzyj" renamed to "zaloz"
 * + Being.findItem
 * + Location.useItem
 * + polecenie "uzyj" zaimplementowane jako "uzyj cos w danej lokacji"
 * - some TO-DOs in UserCommand removed (done!)
 *
 * Revision 1.56  2004/02/01 18:20:21  michalis
 * * Being.wearById changed (nie mona zakada e co co jest StatModifierItem
 *   jest na pewno Armor, Ring albo Weapon, poza tym mae uproszczenia zapisu)
 * + PlayersMessages.tellEveryoneInLocation(Location, String, Being)
 * * Being.dropOnFloor: docs slightly improved, implementation simplified
 * * Being.pickFromFloor: podobnie implementation simplified
 * - some comments from UserCommand removed, * some TO-DOs moved to Being
 *
 * Revision 1.55  2004/02/01 17:14:44  emes
 * * pickFromFloor made private
 *
 * Revision 1.54  2004/02/01 17:13:21  emes
 * * messages moved from UserCommand to Being
 *
 * Revision 1.53  2004/02/01 00:56:58  michalis
 * * Being.dead very small changes
 * * StringSocket small write improvement
 * + Server.Server processes messages from clients using UserCommand.
 *   Dziaa ! Merge JamySingle + Client zakoczony - mamy zrobione
 *   co co mona nazwa dziaajcym MUDem !
 *   (i udao mi si zachowa te JamySingle w wersji dziaajcej !)
 *
 * Revision 1.52  2004/02/01 00:08:54  michalis
 * + Being.DeadAnswer, dead() returns DeadAnswer
 * - Player/Creature.dead
 * * Player/Creature.setNowLocation: remove object from world.players/creatures list
 * * World.action implemented cleaner
 * * Creature.attack dostaje parametr PlayersMessages
 * + Creature.ElfKiller, dodany do World do lokacji "Jama"
 * + PlayersMessages.append
 * + PlayerConnection.server, + Connection.createPlayer "server" parameter
 * + gdy gracz umiera to jest komunikat dla System.out serwera, komunikat "Umierasz"
 *   dla umierajcego gracza i komunikat "kto-tam umiera" dla pozostaych graczy
 *   w tej lokacji
 * * another bugfix for checking dead state of players in server.
 *   Wszystko przez to e na razie mam doc niewygodne metody testowania tego kodu
 *   skoro gracze nie mog si rusza w wiecie. No, ale myl e obecnie
 *   wszystko ju jest jak trzeba - wszystkie elementy kodu zostay
 *   przetestowane dziki ElfKillerowi.
 *
 * Revision 1.51  2004/01/19 02:20:20  michalis
 * * setNowLocation interface slightly changed for subclasses
 *   (value can be null in one very very special case),
 * + dead calls setNowLocation(null)
 *
 * Revision 1.50  2004/01/19 01:35:25  michalis
 * + world.creatures()
 * * Location.creature() is now only calculated
 * * appropriate docs updated in Being
 * * Player.to[Detail]String removed and added to Being.to[Detail]String
 *
 * Revision 1.49  2004/01/18 23:58:20  michalis
 * + deadCheck, more comments for dead, attack and doAttack names exchanged
 *
 * Revision 1.48  2004/01/18 22:27:18  michalis
 * + StatModifier.MODIF_ consts added, * some things rewritten using these consts,
 * + StatModifier.makeFinalStat, used in Being.fullStats
 *
 * Revision 1.47  2004/01/18 22:03:25  michalis
 * * toString changed (to give *less* information - you can always use toDetailString
 *   if you really want all that stuff)
 *
 * Revision 1.46  2004/01/18 20:34:48  michalis
 * + docs updated
 *
 * Revision 1.45  2004/01/18 20:12:20  michalis
 * * Smaller BASEDELAY, smaller troll.maxLife. It makes testing easier...
 *
 * Revision 1.44  2004/01/18 19:57:09  michalis
 * * die removed, more general dead in Being, overriden in Player and Creature
 *
 * Revision 1.43  2004/01/18 19:52:22  emes
 * + important TO-DO
 *
 * Revision 1.42  2004/01/18 19:45:46  michalis
 * * Minor doc updates
 *
 * Revision 1.41  2004/01/18 19:38:41  emes
 * + die() method added
 *
 * Revision 1.40  2004/01/18 19:26:31  emes
 * - ok, ok, i'm moving it away :)
 *
 * Revision 1.39  2004/01/18 19:24:10  emes
 * + combat handling: attack() and loseLife() methods
 *   very incomplete yet.
 *
 * Revision 1.38  2004/01/18 19:14:09  michalis
 * * toDetailString improved, weaponHit fixed, attack implemented
 *
 * Revision 1.37  2004/01/18 18:44:56  michalis
 * + DiceRandom(int), Race.unarmedDamage, Race.skinDefense, Being.armorDefense, Being.weaponHit
 *
 * Revision 1.36  2004/01/18 06:46:26  michalis
 * + Race.randomStat, Player.raceRandomStat
 *
 * Revision 1.35  2004/01/15 20:55:41  emes
 * * bugfix
 *
 * Revision 1.34  2004/01/15 20:38:16  emes
 * * changed equipment/inventory management a lot (added methods using unique IDs)
 *
 * Revision 1.33  2004/01/15 19:47:00  emes
 * + added class IdedList for items with id
 * + method IdedList.byId() returns Ided object with specified id
 *
 * Revision 1.32  2004/01/15 19:19:17  emes
 * + show backpack contents
 *
 * Revision 1.31  2004/01/15 18:56:38  emes
 * * pickById() returns Item object instead of boolean val.
 *   it provides more functionality without any loss
 *
 * Revision 1.30  2004/01/15 18:33:34  michalis
 * * Small updates; strings made more consistent (only polish, with ogonki)
 *
 * Revision 1.29  2004/01/15 15:40:39  michalis
 * * Creating backpack in Being implemented a little shorter
 * + example assertion in Player (aName != null)
 *
 * Revision 1.28  2004/01/15 15:13:21  emes
 * * fixed stupid errors
 *
 * Revision 1.27  2004/01/15 15:10:49  emes
 * * pickById() returns boolean result
 *
 * Revision 1.26  2004/01/15 15:05:17  emes
 * * backpack is created in Being constructor
 * * name is passed to Being
 *
 * Revision 1.25  2004/01/15 13:58:22  michalis
 * + Being.toDetailString, pokryte w Player, uzyte w UsedCommand,
 *   docs update in Creature, BaseTests corrected (name and kind are lowercase)
 *
 * Revision 1.24  2004/01/15 13:18:59  michalis
 * + Being.doAction, Being.action i costam jeszcze
 *
 * Revision 1.23  2004/01/13 20:34:08  michalis
 * * Being.usedItem and related things finished. Nice and general
 *   implementation of wear, drop and fullStats methods.
 *
 * Revision 1.22  2004/01/13 15:13:07  michalis
 * + usedItem / setUsedItem / USED_xxx consts
 *
 * Revision 1.21  2004/01/13 14:33:11  michalis
 * + Being.toString(this) dziala ladnie, w UserCommand dodane "opisz miejsce/mnie"
 *
 * Revision 1.20  2004/01/13 14:15:36  michalis
 * * Rally really last update to using player.nextAction in JamySingle
 *
 * Revision 1.19  2004/01/13 13:09:50  michalis
 * * Fixes for nextAction
 *
 * Revision 1.18  2004/01/13 11:29:56  michalis
 * * calcNextAction used in JanySingle
 *
 * Revision 1.17  2004/01/13 06:17:21  michalis
 * * Mae przeprowadzki kodu i notki w dokumentacji w zwizku z calcNextAction
 *
 * Revision 1.16  2004/01/13 05:57:52  michalis
 * * nextAction things moved from Creature to Being
 *
 * Revision 1.15  2004/01/13 04:38:55  michalis
 * + JamySingle (single-user version of game), LinkToLocationList, UserCommand
 *
 * Revision 1.14  2004/01/12 04:08:11  michalis
 * + Creature.id moved to new class Ided, Ided is now the base class of
 *   Being, Item and LinkToLocation
 *
 * Revision 1.13  2004/01/12 02:20:04  michalis
 * * docs improved
 *
 * Revision 1.12  2004/01/12 02:02:44  michalis
 * + StatModifier.intelligence(), class Attitude, Being.attitude(),
 *   Being.toString(Being) extended to show attitude
 *
 * Revision 1.11  2004/01/12 01:22:45  michalis
 * + Location/Being.toString(Being), czyli obiekty z punktu
 *   widzenia danego Being
 *
 * Revision 1.10  2004/01/11 22:46:22  michalis
 * * Fixed dropWeapon/Armor for the case of weapon/armor = null
 *
 * Revision 1.9  2004/01/11 10:11:12  michalis
 * * docs corrections
 *
 * Revision 1.8  2004/01/11 10:06:17  michalis
 * * dropping weapon and armor implemented
 *
 * Revision 1.7  2004/01/11 09:34:26  michalis
 * + Class LinkToLocation, Location.links and necessary tests and docs
 *
 * Revision 1.6  2004/01/11 07:40:21  michalis
 * * Poprawki i ulepszenia dokumentacji
 *
 * Revision 1.5  2004/01/11 06:51:54  michalis
 * + World class, Being.nowLocation
 *
 * Revision 1.4  2004/01/10 09:39:47  michalis
 * * StatModifier interface and implementation changed;
 *   now it will be safer and easier to add new stats there
 *
 * Revision 1.3  2004/01/10 08:55:10  michalis
 * + StatModifier.maxLife and Being.life properties
 *
 * Revision 1.2  2004/01/10 07:38:23  michalis
 * + I forgot to add CVS Logs when I created those files; well, now they are
 *
 */
