/*
  Copyright 2004 Michalis Kamburelis, Micha Saaban.

  This file is part of "Jamy & Nory".

  "Jamy & Nory" is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  "Jamy & Nory" is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with "Jamy & Nory"; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
 
package jamy_i_nory.game;

import java.util.*;
import jamy_i_nory.common.*;

public abstract class Creature extends Being {

  int fWanderingFactor;

  /** Wandering factor, czyli tzw. "wspczynnik wasania si".
     Zawsze musi by &gt;= 0.
     W kadym wykonaniu doAction klasy Creature stworzenie ma szans
     1/wanderingFactor e si poruszy uywajc losowo wybranego linku
     z danej lokacji (o ile nie zrobi jakiej innej akcji, np. jeeli
     zaatakuje kogo w doAction to traci ju szans na ruch).
     Innymi sowy, gdy wanderingFactor = 1 to stworzenie poruszy si 
     zawsze gdy bdzie miao szane. Gdy wanderingFactor = 2 to 
     stworzenie ma szans 1/2 e si poruszy. Itd. Gdy wanderingFactor 
     jest bardzo duy to stworzenie bdzie si bardzo rzadko rusza.
     W wyjtkowym przypadku gdy wanderingFactor = 0 stworzenie nie bdzie
     si nigdy rusza. */
  public int wanderingFactor() { return fWanderingFactor; }
  
  /** Ustaw wanderingFactor. value musi by &gt;= 0.
     Patrz komentarz przy funkcji {@link #wanderingFactor} po interpretacj
     tego atrybutu. */
  public void setWanderingFactor(int value) 
  {
    fWanderingFactor = value;
  }

  public Creature (World aWorld, Location startLocation, Race aRace, String aName,
    int aWanderingFactor)
  {
    super(aWorld, startLocation, aRace, aName);
    aWorld.creatures().add(this);
    fWanderingFactor = aWanderingFactor;    
  }
  
  /** As was specified in the interface of {@link Being Being},
     this method takes care of {@link Being#nowLocation 
     Being.nowLocation()}.creatures(). */
  public void setNowLocation(Location value, LinkToLocation link)
  {
    if (nowLocation() != null) nowLocation().creatures().remove(this);
    super.setNowLocation(value, link);
    if (nowLocation() != null) nowLocation().creatures().add(this);       
  }

  protected void beforeDeath(boolean goingInactive)
  {
    /* dla Creature nigdy nie powinno nastpi wywoanie goInactive 
       a wic ta metoda nigdy nie powinna zosta wywoana z goingInactive 
       = true.*/
    assert !goingInactive;
    
    world.creatures().remove(this);
    super.beforeDeath(goingInactive);
  }
    
  /** Nigdy nie zwraca UNKNOWN i pokrywajc w podklasach nigdy nie moesz
     zwrci UNKNOWN. W ten sposb realAttitude mona wykorzysta do 
     implementacji sztucznej inteligencji ju w klasie Creature,
     tzn. w {@link Creature#doAction}. */
  public abstract int realAttitude (Being target);
  
  /** Pokrywaj w podklasach aby zrobi AI. Zrobi odpowiednie putMessage
     graczom ktrzy widzieli skutki tej akcji. 
     
     <p>Uwaga: ta metoda nie moe modyfikowa world.creatures() 
     dla jakiej lokacji ani world.players(). To dlatego e ta metoda 
     moe by wywoywana w trakcie iterowania po licie world.creatures(),
     a samo world.players() ma by stae eby mc je analizowa gdy trzeba
     bdzie doda co do playersMessages.
     W szczeglnoci wic nie mona std sprawdza dead. 
     Tym niemniej ta metoda powinna stara si uwzgldnia e niektre beings 
     (na listach creatures() lub players()) mog by martwe,
     co mona bezpiecznie sprawdza przez deadCheck.
     
     <p>Zwraca true jeeli w stworzenie wykonao ju jaki ruch. 
     W przypadku Player w UserCommand wymuszamy na nim eby zbyt czsto
     nie robi adnych ruchw - w przypadku Creature musimy si po prostu
     pilnowa i nigdy nie robi wicej ni jednego ruchu w doAction
     (przy czym chwilowo nie mamy tak dokadnie zdefiniowane co jest 
     "jednym ruchem"...). To oznacza e t metod trzeba pokrywa w podklasach 
     zawsze uywajc schematu
     <pre>
     if (super.doAction())
       return true; else
     {
       .. by moe rb jak akcj, na kocu zrb return false lub true ...
     }
     </pre>
     
     <p>W klasie Creature ta metoda atakuje (przez {@link #attack}) pierwszego
     z brzegu osobnika w komnacie {@link #nowLocation} dla ktrego realAttitude 
     zwrci HOSTILE. Jeeli nie ma takiego osobnika to wykonywane jest 
     losowanie na bazie {@link #wanderingFactor} i ew. stworzenie przechodzi
     po losowo wybranym linku z danej lokacji. */
  protected boolean doAction()
  {
    /* TODO: losowa. Pole attackRandomized obiektu zrobi -
       jeeli false to prbuje najpierw zaatakowa tego ktrego
       atakowa w ostatniej rundzie, bez losowania. 
       Na pewno nie chcemy uzalenia tego w kogo uderza od porzdku
       na licie nowLocation().beings() - ten jest do tendencyjny
       i odkrywa szczegy implementacji. */
  
    /* Szukaj na licie nowLocation().beings() ywego wroga */
    Iterator i = nowLocation().beings().iterator();
    while (i.hasNext())
    {
      Being b = (Being) i.next();
      if (realAttitude(b) == Attitude.HOSTILE && (!b.deadCheck()) && b!=this) 
      {
        attack(b);
        return true;
      }
    }
    
    /* By moe zrb ruch. */
    LinkToLocationList links = nowLocation().links();
    if (wanderingFactor() != 0 && 
        Utils.rand.nextInt(wanderingFactor()) == 0 &&
        links.size() != 0)
    {      
      followLink((LinkToLocation) 
        links.get( Utils.rand.nextInt(links.size()) ));
      return true;
    }
    
    return false;
  }
  
  /** Sprawdza {@link Being#nextActionAllowed} i ! {@link Being#deadCheck}
     i (jeeli si udao) robi doAction() i {@link Being#calcNextAction}.
     Innymi sowy zajmuje si rzeczami technicznymi ktre naprawd
     zawsze  musz by robione przy podejmowaniu akcji - dziki temu
     zdejmuje ten ciar z doAction. Nie pokrywaj tego w podklasach -
     po to jest doAction. 
     
     <p>Techniczna uwaga: zwracam uwag e robi calcNextAction() nawet gdy
     doAction() zwrcio false. Jest dyskusyjne czy kiedy nie rozwi
     tego inaczej (powinienem). Ale chwilowo jest jak jest bo to pozwala
     mi w doAction robi losowania, np. jeeli Random(wanderingFactor) 
     == 0 to zrb ruch itd. Gdybym tutaj nie uaktualnia calcNextAction()
     po takim nieudanym doAction (w ktrym nie wykonano ruchu) to takie
     postpowanie byoby bdne bo doAction mogoby by wywoywane dowolnie
     czsto. A postpowanie na zasadzie "jeeli Random(wanderingFactor) 
     to zrb ruch ale bez wzgldu na wynik tego Random zwr true"
     (czyli "sam fakt losowania znaczy e wykonae akcj") te jest bdne
     bo znaczy e doAction zmierza do tego aby zawsze zwraca false 
     (pomijajc fakt e wtedy doAction nie mogoby ju by pokrywane w
     podklasach, trzebaby robi co jak creatureSpecificDoAction
     w kadym punkcie hierarchii). Dobrym rozwizaniem tego byoby nie robi
     Random.nextInt tylko zaimplementowa co jak timedRandom ktre 
     raz na iletam milisekund testuje odpowiednie Random.nextInt 
     a w pozostaych przypadkach zawsze zwraca false (wtedy opnienie
     timedRandom nie byoby zdeterminowane przez getDelay stworzenia,
     co byoby zalet). Jeszcze pomyl, jak mi si zachce to to zrobi. TODO. */
  final public void action()
  {
    if (nextActionAllowed() && !deadCheck())
    { 
      doAction();
      calcNextAction();
    }
  }
  
  /* example creatures ---------------------------------------- */

  /** Bardzo silnie atakuje elfw, graczy i nie-graczy. Innych nie rusza. */  
  static public class ElfKiller extends Creature {
  
    public ElfKiller(World aWorld, Location startLocation, String aName)
    {
      super(aWorld, startLocation, new Race("Zabjca elfw", 
        new StatModifier(20, 20, 0, 10, 2), new DiceRandom(2, 20), 1), aName, 50);
    } 
    
    public int realAttitude(Being target)
    {
      return (target.race instanceof Race.Elf ? 
        Attitude.HOSTILE : Attitude.NEUTRAL);
    }
  } 
  
  /** Zabjca z Grjca. */  
  static public class GrojecKiller extends SimpleMindedCreature {
  
    public GrojecKiller(World aWorld, Location startLocation, String aName)
    {
      super(aWorld, startLocation, new Race("Zabjca z Grjca", 
        new StatModifier(1, 1, 0, 1, 2), new DiceRandom(1, 2), 1), aName,
        0, true, true);
    } 
  } 
  
  static public class Troll extends SimpleMindedCreature {

    public Troll(World aWorld, Location startLocation, String aName)
    {
      super(aWorld, startLocation, new Race("Troll", 
        new StatModifier(16, 5, 0, 10, 2), new DiceRandom(1, 2), 1), aName,
        50, true, true);
      wear(USED_WEAPON, new Weapon(10.0f, "maczuga", null, null, 
        new StatModifier(), new DiceRandom(1, 6)));
    }

    /* testy:    
    protected void doAction()
    {
      if (super.doAction())
        return true; else
      {
        for (int i = 0; i < playersMessages.length; i++)
        {     
          playersMessages[i].append(
            (((Player) (world.players().get(i))).nowLocation() == nowLocation() ?
             "Troll obcina sobie palec (tracc 2 punkty ycia) i wrzeszczy." :
             "Syszysz jak jaki troll wrzeszczy gdzie daleko.") +Utils.nl);
        }
        life -= 2;
        return true;
      }
    }
    */
  }
  
  static public class Goblin extends SimpleMindedCreature {
  
    public Goblin(World aWorld, Location startLocation, String aName) 
    {
      super(aWorld, startLocation, new Race("Goblin", 
        new StatModifier(2, 15, 0, 100, 2), new DiceRandom(2, 4, 1), 0), aName,
        50, false, true);  
    }
    
    protected boolean doAction()
    {
      if (super.doAction())
        return true; else
      {      
        if (Utils.rand.nextInt(4) == 0)
        {
          sayEveryoneInNowLocation("Jestem godny.");
          return true;
        }
        return false;
      }
    }
  }
  
  /** W realAttitude zawsze zwraca NEUTRAL. */
  static public class NonHostileCreature extends Creature {
  
    public NonHostileCreature (World aWorld, Location startLocation, 
      Race aRace, String aName, int aWanderingFactor)
    {
      super(aWorld, startLocation, aRace, aName, aWanderingFactor);
    }  
  
    public int realAttitude(Being target)
    {
      return Attitude.NEUTRAL;
    }
  }

  /** Completely non-hostile creature: human walking around.
     Demonstrates that a creature can move in the world. */
  static public class DungeonExplorer extends NonHostileCreature {
  
    public DungeonExplorer(World aWorld, Location startLocation)
    {
      super(aWorld, startLocation, Race.human, "Dungeon Explorer", 8);
    }
  }
}

/* 
 * $Log: Creature.java,v $
 * Revision 1.2  2004/02/19 15:30:28  michalis
 * * Creature.attackPlayers/Creatures -> SimpleMindedCreature
 * + NonHostileCreature
 * + ElfFairy, ElfFairyWhistle
 * + wanderingFactor, teraz prawie wszystkie creatures wasaj si po wiecie
 *
 * Revision 1.1  2004/02/18 14:37:28  michalis
 * * pl stringi
 * * --help output changed (to reflect that those classes will be run by bin/scripts)
 * preparations for ClientApplet:
 * * Client rozdzielony na Client i ClientStandalone
 * * znaczna cz pakietu common przeniesiona do nowego pakietu game
 * X* make jamy_i_nory_client_applet.jar
 *
 * Revision 1.52  2004/02/14 11:01:50  michalis
 * * demo otwierania i zamykania drzwi kluczem zrobione
 *
 * Revision 1.51  2004/02/14 01:09:43  michalis
 * + przejcia w drug stron (LinkToLocation.wayBack, Location.createTwoSidedLinks)
 * * followLink -> Being
 * * lepsze komunkaty o przejciach w setNowLocation
 * * Troll osabiony
 *
 * Revision 1.50  2004/02/09 23:08:50  michalis
 * + GPL license clause i copyrighty (moje i gdzieniegdzie Michaa, sorry Rafa
 *   ale naprawd nic nie zostao) do wszystkich *.java
 *
 * Revision 1.49  2004/02/07 21:11:32  michalis
 * * StatModifier.makeFinalStat changed (bugfix: must be maxLife > 0,
 *   bez tego GrojecKiller czsto zaczyna ju jako trup)
 * * StatModifier.Modifs are named (as it was before) addModifs
 * * small improvement to Creature.action (to m.in. eliminuje problem ktry mi
 *   opisalicie e po zabiciu DungeonExplorera on jeszcze zdy (martwy)
 *   wyj z pomieszczenia)
 * + Utils.clamped
 * * Being.getDelay lepiej dostrojone
 *
 * Revision 1.48  2004/02/07 20:05:15  michalis
 * + World.beingsFindById
 * + ServerCommand.parseBeingId
 * * ServerCommand polecenia "opisz" i "zabij" zapisane adniej, poprawione pl ogonki
 * * implementacja Zabjcy z Grjca uproszczona
 * + do dokumentacji o poleceniach serwera "opisz" i "zabij"
 *
 * Revision 1.47  2004/02/07 17:34:58  michalis
 * * jakies strzepy
 *
 * Revision 1.46  2004/02/06 04:44:40  michalis
 * + InactivePlayer
 * + dodatkowy konstruktor do Being
 * + dodatkowy konstruktor Player(World, InactivePlayer)
 * * beforeDeath rozszerzone o parametr goingInactive
 * + Being.goInactive
 * * listToString moved from World to Utils
 * + ServerCommand : polecenie "mrozonki"
 *
 * Revision 1.45  2004/02/05 20:05:26  michalis
 * * Zrobiem eby w logach CVSa zapisanych w plikach byo wszdzie TO-DO
 *   z mylnikiem w rodku. Wprawdzie nieznacznie rozsynchronizowuje to
 *   zapisane w plikach logi z wynikiem 'cvs log' ale za to grep TO-DO
 *   (gdzie w wyrazie TO-DO naley usun mylnik) dziaa jak trzeba.
 *
 * Revision 1.44  2004/02/04 16:21:09  michalis
 * * Being.dead wysya komunikaty uywajc toString zamiast toStringBase
 *   co jest pierwszym krokiem do wywalenia toStringBase
 *
 * Revision 1.43  2004/02/04 15:36:42  michalis
 * + Dziki pozbyciu si PlayersMessages moemy teraz wypisywa stringi
 *   o tym kto wchodzi/wychodzi z pomieszczenia w setNowLocation.
 * + Being.beforeDead, Being.toStringBase eby setNowLocation
 *   mogo take wywietla kto si pojawia (jako nowy gracz/potwr w wiecie)
 *   uywane z konstruktora Being i z dead()
 * * Server zosta uproszczony w wyniku powyszej zmiany, Being.dead te
 *   (chocia pojawia si niewielka strata funkcjonalnoci - patrz TO-DO przy
 *   toStringBase())
 *
 * Revision 1.42  2004/02/04 14:25:58  michalis
 * * dziki usuniciu PlayersMessages znowu mamy proste setNowLocation
 *   w Player i Creature i znowu mamy proste Player.dead i Creature.dead
 *
 * Revision 1.41  2004/02/04 14:00:24  michalis
 * * Being.tellEveryoneInNowLocation -> sayEveryoneInNowLocation
 *   (mniej mylca nazwa)
 * * Server powiadamia wszystkich graczy w komnacie e pojawi si nowy gracz
 * * PlayersMessages.tellEveryoneInLocation - docs updated,
 *   implemented in a cleaner way
 * + MessageGenerator
 * + MessageFormatter
 * * Being.attitude changed to realAttitude and observedAttitude
 * * Many fixes to use MessageFormatter when updating PlayersMessages.
 *   Many TO-DOs closed.
 * * Attitude.toStr translated to polish
 * * Poprawiony bug w Being.dead - Micha, nie mona tworzy PlayersMessages
 *   tu przed usuniciem gracza z gry - w czasie uywania PlayersMessages
 *   world.players() nie mog by modyfikowane. Z drugiej strony, to jest moja
 *   wina e jeszcze tego nie mona zrobi (bo to *powinno* by moliwe tylko
 *   musz zapisa inteligentniej PlayersMessages). OK, wanie mi przyszo do
 *   gowy jak to zrobi bardzo atwo i bardzo elegancko (klasa PlayersMessages
 *   w ogle nie bdzie potrzebna i unikniemy przerzucania wszdzie parametrw
 *   playersMessages) i w nastpnym commicie to bdzie zrobione (gupi, gupi,
 *   gupi jestem e od pocztku tego tak nie zrobiem).
 * * OK, serwer skunk.one.pl nie odpowiada wic nie bd commitowa, od razu
 *   zrobi to o czym mwiem powyej:
 *   + Player.getMessage, Player.putMessage
 *   + Location.tellEveryone (3 wersje)
 *   - PlayersMessages (funkcjonalno przerzucona w caoci do obiektw Player i
 *     Location)
 *   * masa uproszcze kodu (m.in. nie musimy ju przerzuca wszdzie
 *     parametrw PlayersMessages)
 *   - klasa Being.DeadAnswer (useless, teraz Being.dead() znowu zwraca prostego
 *     booleana)
 *   * w ogle Being.dead() jest teraz o niebo prostsze.
 *   Skoczyem! PlayersMessages usunite razem z wieloma niewygodami.
 *
 * Revision 1.40  2004/02/01 20:57:59  michalis
 * + UserCommandBlackTokens, wszystkie dotychczasowe komendy zmienione jako
 *   dziedziczce z UserCommandBlackTokens zamiast bezporednio z UserCommand
 * + Being.tellEveryoneInLocation
 * + UserCommand: komenda "powiedz"
 * + Goblin mwi e jest godny (naprawd)
 *
 * Revision 1.39  2004/02/01 17:13:21  emes
 * * messages moved from UserCommand to Being
 *
 * Revision 1.38  2004/02/01 00:08:54  michalis
 * + Being.DeadAnswer, dead() returns DeadAnswer
 * - Player/Creature.dead
 * * Player/Creature.setNowLocation: remove object from world.players/creatures list
 * * World.action implemented cleaner
 * * Creature.attack dostaje parametr PlayersMessages
 * + Creature.ElfKiller, dodany do World do lokacji "Jama"
 * + PlayersMessages.append
 * + PlayerConnection.server, + Connection.createPlayer "server" parameter
 * + gdy gracz umiera to jest komunikat dla System.out serwera, komunikat "Umierasz"
 *   dla umierajcego gracza i komunikat "kto-tam umiera" dla pozostaych graczy
 *   w tej lokacji
 * * another bugfix for checking dead state of players in server.
 *   Wszystko przez to e na razie mam doc niewygodne metody testowania tego kodu
 *   skoro gracze nie mog si rusza w wiecie. No, ale myl e obecnie
 *   wszystko ju jest jak trzeba - wszystkie elementy kodu zostay
 *   przetestowane dziki ElfKillerowi.
 *
 * Revision 1.37  2004/01/31 20:10:29  michalis
 * + IOSystem: some TO-DO with comments moved from UserCommand to IOSystem
 * * UserCommand docs improved
 * * UserCommand.s renamed to commandPattern
 * + PlayersMessages
 * * World.messageToPlayersInLocation moved to PlayersMessages.tellEveryoneInLocation
 * * World.action changed to use PlayersMessages (much cleaner code now!)
 * * Creature.action and doAction use PlayersMessages
 * * UserCommand.execute interface changed:
 *   - world (excessive, one can always use player.world)
 *   + playersMessages (to tell other players about effects of this command)
 *   + returns String (tell command invoker about effects of this command)
 * * every command's execute changed to use new interface:
 *   nothing uses JamySingle.io, nothing may call Player.dead()
 *
 * Revision 1.36  2004/01/19 02:22:13  michalis
 * * Creature.setNowLocation implemented (again) to update Location.creatures,
 *   but this time it's final
 *
 * Revision 1.35  2004/01/19 01:41:18  michalis
 * + world.creatures()
 * * Location.creature() is now only calculated
 * * appropriate docs updated in Creature,
 * - Creature.setNowLocation (useless for now)
 * * Creature.dead updated, constructor updated,
 * * Troll moved inside Creature.java
 * + new Creature subclass DungeonExplorer
 *
 * Revision 1.34  2004/01/19 00:19:49  michalis
 * * Fixed action() (do not attack yourself)
 *
 * Revision 1.33  2004/01/19 00:00:35  michalis
 * + more comments, action and doAction names exchanged, can attack other creatures
 *
 * Revision 1.32  2004/01/18 22:07:24  michalis
 * * action implemented partially (attacks other players)
 *
 * Revision 1.31  2004/01/18 20:34:48  michalis
 * + docs updated
 *
 * Revision 1.30  2004/01/18 19:57:09  michalis
 * * die removed, more general dead in Being, overriden in Player and Creature
 *
 * Revision 1.29  2004/01/17 16:45:47  michalis
 * * Minor doc improvement
 *
 * Revision 1.28  2004/01/16 20:05:12  michalis
 * + parametr StringBuffer[] playersMessages do metod doAction i action
 *
 * Revision 1.27  2004/01/15 15:05:18  emes
 * * backpack is created in Being constructor
 * * name is passed to Being
 *
 * Revision 1.26  2004/01/15 13:58:23  michalis
 * + Being.toDetailString, pokryte w Player, uzyte w UsedCommand,
 *   docs update in Creature, BaseTests corrected (name and kind are lowercase)
 *
 * Revision 1.25  2004/01/15 13:19:42  emes
 * * s/Pullic/Public/
 *
 * Revision 1.24  2004/01/15 13:18:59  michalis
 * + Being.doAction, Being.action i costam jeszcze
 *
 * Revision 1.23  2004/01/15 11:21:27  michalis
 * * Fixed indentation (Rafal: nie uzywaj znakow TAB w zrodlach !)
 *
 * Revision 1.22  2004/01/15 00:11:17  bularaider
 * * public int attitude, refaktoryzacja kodu
 *
 * Revision 1.21  2004/01/13 20:39:47  michalis
 * * code formatting fixed
 *
 * Revision 1.20  2004/01/13 06:17:21  michalis
 * * Mae przeprowadzki kodu i notki w dokumentacji w zwizku z calcNextAction
 *
 * Revision 1.19  2004/01/13 05:57:52  michalis
 * * nextAction things moved from Creature to Being
 *
 * Revision 1.18  2004/01/12 04:08:11  michalis
 * + Creature.id moved to new class Ided, Ided is now the base class of
 *   Being, Item and LinkToLocation
 *
 * Revision 1.17  2004/01/12 02:20:04  michalis
 * * docs improved
 *
 * Revision 1.16  2004/01/12 02:02:44  michalis
 * + StatModifier.intelligence(), class Attitude, Being.attitude(),
 *   Being.toString(Being) extended to show attitude
 *
 * Revision 1.15  2004/01/11 22:48:45  michalis
 * * Class Monster removed, Monster.attackhuman/monster moved to
 *   Creature.attackPlayers/Creatures
 *
 * Revision 1.14  2004/01/11 08:31:56  michalis
 * + Added some TO-DOs requesting documentation for things that
 *   I (Michalis) perceive as unnecessary
 *
 * Revision 1.13  2004/01/11 06:51:54  michalis
 * + World class, Being.nowLocation
 *
 * Revision 1.12  2004/01/10 09:39:47  michalis
 * * StatModifier interface and implementation changed;
 *   now it will be safer and easier to add new stats there
 *
 * Revision 1.11  2004/01/10 07:17:27  michalis
 * * base class Being separated from Creature
 *
 * Revision 1.10  2004/01/10 03:08:58  michalis
 * * Many small fixes; mainly improvements to javadoc comments
 *
 * Revision 1.9  2004/01/09 01:29:59  emes
 * - add rings' stats
 *
 * Revision 1.8  2004/01/09 00:37:51  michalis
 * * Poprawka compareTo, musi brac parametr klasy Object
 *
 * Revision 1.7  2004/01/09 00:21:09  emes
 * - ring slots
 *
 * Revision 1.6  2004/01/08 23:46:48  emes
 * - interfejs Comparable dla stworzen, sortujemy po opoznieniu
 *
 * Revision 1.5  2004/01/08 23:34:44  emes
 * - pole id dla kadej postaci
 *
 * Revision 1.4  2004/01/08 22:45:38  michalis
 * - Usunalem wlasny blad - te "reczne" logi ktore zaczalem tworzyc
 *
 * Revision 1.3  2004/01/08 22:37:42  michalis
 * + CVS Logs appended to all .java files
 *
 *
 */
