/*
  Copyright 2004 Michalis Kamburelis, Micha Saaban.

  This file is part of "Jamy & Nory".

  "Jamy & Nory" is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  "Jamy & Nory" is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with "Jamy & Nory"; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package jamy_i_nory.game;

import jamy_i_nory.common.*;

/** Abstract class representing any item in game. 

   <p>Ta klasa nie ma poczenia ze wiatem zewntrznym, tzn. nie ma
   pola world jak Being czy Location. To sprawia e mona j bez problemw
   agregowa w InactivePlayer. */
public abstract class Item extends Ided {

  /** Tzw. wewntrzna waga. W tej klasie internalWeight wyznacza
     co zwraca {@link #weight}. Ale w niektrych podklasach - w tym momencie
     tylko w {@link ItemsBag} - {@link #weight weight()}
     moe by wyznaczane jako inaczej. 
     Patrz tam po opis co w przypadku takich przedmiotw
     oznaczaj weight i internalWeight. 
     W kilogramach. */
  final public float internalWeight;
  
  /** Cakowita waga przedmiotu. W tej klasie jest brana z {@link 
     #internalWeight} ale w podklasach mona to pokry aby wylicza
     weight jako inaczej. */
  public float weight() { return internalWeight; }

  /** Short name for item kind, something like "miecz", "napj" etc. 
     Starts with lower letter, no newlines, should be one word
     (in special cases we allow a few words, but you rather should
     add more words to {@link #kind}). Must not be null or "". */
  final public String name;

  /** Additional description, e.g. "dugi" etc.
     Starts with lower letter, no newlines, may be a few words.
     Jeli nie ma description to daj null (nie ""). */
  final public String kind;

  /** Only for artifacts, ego-items, etc., null (nie "") for normal items. */
  final public String specialName;
  
  Item(float aInternalWeight, String aName, String aKind, String aSpecialName)
  {
    internalWeight = aInternalWeight;
    name = aName;
    kind = aKind;
    specialName = aSpecialName;
  }
  
  /** Zwraca string zawierajcy {@link #kind}, {@link #name} i {@link 
     #specialName} w eleganckiej postaci. Pierwsza litera jest uppercase
     (m.in. z tego wzgldu pokrywajc t metod w podklasach lepiej 
     dopisywa do super co z tyu ni z przodu). */
  public String toString()
  {  
    StringBuffer result = new StringBuffer();
    if (kind != null) result.append(kind +" ");
    result.append(name);    
    if (specialName != null) result.append(" \"" +specialName +"\"");
    Utils.capitalize(result);
    return "[id: " + id + "] " + result + ", " + weight() + "kg";
  }
  
  /** Uywa przedmiotu. Zwraca true jeeli dao si uy tego przedmiotu.
     Jeeli zwrcio true to wszyscy zainteresowani zostali
     powiadomieni przez putMessage i na pewno being dostao przynajmniej
     raz putMessage. Wpp. (gdy zwrcio false) to nikt nie dosta adnego
     putMessage.
  
     <p>Pamitaj e uywanie przedmiotw moe take by zapisane 
     w kodzie Location.useItem co oznacza e ta metoda powinna by 
     wywoywana TYLKO z Location.useItem - w kadej innej sytuacji uywaj 
     Location.useItem. 
     
     @return czy dao si tego przedmiotu jako uy. Patrz 
     {@link Location#useItem} po dokadniejsze komentarze. 
     W klasie Item zawsze zwraca false. */
  public boolean useItem(Being being)
  {
    return false;
  }
  
  /* example items ---------------------------------------- */
  
  static public class ElfKillerWhistle extends Item {
    
    ElfKillerWhistle()
    {
      super(0.01f, "gwizdek", null, "Przywoywacz Zabjcy elfw");
    }
    
    public boolean useItem(Being being)
    { 
      Creature c = new Creature.ElfKiller(being.world, 
        being.nowLocation(), null);
      
      /* TODO: jak tylko w momencie tworzenia Being graczom pojawi si
         bardziej pena informacja ni Being.toStringBase to z poniszego
         komunikatu naley wyci tekst " i w komnacie pojawia si %2"
         i przenie ten komunikat powyej tworzenia Creature.ElfKiller
         (eby byo wida e najpierw gwizdek jest uywany a dopiero
         pniej pojawia si ElfKiller). */
         
      being.tellEveryoneInNowLocation(new MessageFormatter(
        "%1 uywa " +this +" i w komnacie pojawia si %2", being, c));
      being.putMessage("Uywasz " +this +" i w komnacie pojawia si " +
        c.toString(being));
   
      return true;
    }
  }

  static public class ElfFairyWhistle extends Item {
    
    ElfFairyWhistle()
    {
      super(0.01f, "gwizdek", null, "Przywoywacz Elfowych wrek");
    }
    
    public boolean useItem(Being being)
    { 
      being.tellEveryoneInNowLocation(new MessageFormatter(
        "%1 uywa " +this +".", being));
      being.putMessage("Uywasz " +this +".");

      new Creature.NonHostileCreature(being.world, being.nowLocation(), 
        Race.elfFairy, null, 10);
        
      return true;
    }
  }

  static public class KeyNoraBiblioteka extends Item {
  
    public KeyNoraBiblioteka()
    {
      super(0.01f, "klucz", "otwierajcy drzwi pomidzy Nor a Bibliotek", null);
    }
  };
}
/*
 * $Log: Item.java,v $
 * Revision 1.2  2004/02/19 15:30:28  michalis
 * * Creature.attackPlayers/Creatures -> SimpleMindedCreature
 * + NonHostileCreature
 * + ElfFairy, ElfFairyWhistle
 * + wanderingFactor, teraz prawie wszystkie creatures wasaj si po wiecie
 *
 * Revision 1.1  2004/02/18 14:37:28  michalis
 * * pl stringi
 * * --help output changed (to reflect that those classes will be run by bin/scripts)
 * preparations for ClientApplet:
 * * Client rozdzielony na Client i ClientStandalone
 * * znaczna cz pakietu common przeniesiona do nowego pakietu game
 * X* make jamy_i_nory_client_applet.jar
 *
 * Revision 1.24  2004/02/15 15:24:35  michalis
 * + pdf generating
 * + COPYING file
 * + Utils.check
 * + Location.add/removeLink
 * + LocationWithDynLink
 * * NoraOrBiblioteka.useItem simplified
 * * teraz i w bibliotece i w norze s drzwi
 * - Makefiles rozsiane po katalogach.
 * * Globalny Makefile rzdzi.
 *
 * Revision 1.23  2004/02/09 23:08:50  michalis
 * + GPL license clause i copyrighty (moje i gdzieniegdzie Michaa, sorry Rafa
 *   ale naprawd nic nie zostao) do wszystkich *.java
 *
 * Revision 1.22  2004/02/09 19:35:20  michalis
 * * Uaktualnienia dokumentacji
 *
 * Revision 1.21  2004/02/07 12:35:45  michalis
 * * bugfix
 *
 * Revision 1.20  2004/02/04 16:21:09  michalis
 * * Being.dead wysya komunikaty uywajc toString zamiast toStringBase
 *   co jest pierwszym krokiem do wywalenia toStringBase
 *
 * Revision 1.19  2004/02/04 14:00:24  michalis
 * * Being.tellEveryoneInNowLocation -> sayEveryoneInNowLocation
 *   (mniej mylca nazwa)
 * * Server powiadamia wszystkich graczy w komnacie e pojawi si nowy gracz
 * * PlayersMessages.tellEveryoneInLocation - docs updated,
 *   implemented in a cleaner way
 * + MessageGenerator
 * + MessageFormatter
 * * Being.attitude changed to realAttitude and observedAttitude
 * * Many fixes to use MessageFormatter when updating PlayersMessages.
 *   Many TO-DOs closed.
 * * Attitude.toStr translated to polish
 * * Poprawiony bug w Being.dead - Micha, nie mona tworzy PlayersMessages
 *   tu przed usuniciem gracza z gry - w czasie uywania PlayersMessages
 *   world.players() nie mog by modyfikowane. Z drugiej strony, to jest moja
 *   wina e jeszcze tego nie mona zrobi (bo to *powinno* by moliwe tylko
 *   musz zapisa inteligentniej PlayersMessages). OK, wanie mi przyszo do
 *   gowy jak to zrobi bardzo atwo i bardzo elegancko (klasa PlayersMessages
 *   w ogle nie bdzie potrzebna i unikniemy przerzucania wszdzie parametrw
 *   playersMessages) i w nastpnym commicie to bdzie zrobione (gupi, gupi,
 *   gupi jestem e od pocztku tego tak nie zrobiem).
 * * OK, serwer skunk.one.pl nie odpowiada wic nie bd commitowa, od razu
 *   zrobi to o czym mwiem powyej:
 *   + Player.getMessage, Player.putMessage
 *   + Location.tellEveryone (3 wersje)
 *   - PlayersMessages (funkcjonalno przerzucona w caoci do obiektw Player i
 *     Location)
 *   * masa uproszcze kodu (m.in. nie musimy ju przerzuca wszdzie
 *     parametrw PlayersMessages)
 *   - klasa Being.DeadAnswer (useless, teraz Being.dead() znowu zwraca prostego
 *     booleana)
 *   * w ogle Being.dead() jest teraz o niebo prostsze.
 *   Skoczyem! PlayersMessages usunite razem z wieloma niewygodami.
 *
 * Revision 1.18  2004/02/02 02:16:12  michalis
 * + PlayersMessages(Player,String) (unused for now)
 * + Item.useItem
 * + Item.ElfKillerWhistle implementujcy useItem
 * * Location.useItem wywouje Item.useItem
 * + Egzemplarz Item.ElfKillerWhistle dodany do salonu
 * * some docs clarified in Being
 *
 * Revision 1.17  2004/01/18 22:48:45  michalis
 * * toString fixed - no space after "kg"
 *
 * Revision 1.16  2004/01/15 19:11:09  emes
 * + display weight
 *
 * Revision 1.15  2004/01/15 18:33:34  michalis
 * * Small updates; strings made more consistent (only polish, with ogonki)
 *
 * Revision 1.14  2004/01/15 14:21:22  emes
 * * id will be displayed by toString(); toStringWithId() removed.
 *
 * Revision 1.13  2004/01/15 14:16:43  emes
 * + added toStringWithId () method
 *
 * Revision 1.12  2004/01/15 13:19:42  emes
 * * s/Pullic/Public/
 *
 * Revision 1.11  2004/01/15 13:18:59  michalis
 * + Being.doAction, Being.action i costam jeszcze
 *
 * Revision 1.10  2004/01/14 19:53:41  michalis
 * * Item.weight splitted to internalWeight and weight(), + ItemsBag, + Player.backpack
 *
 * Revision 1.9  2004/01/14 18:50:04  michalis
 * * Item and all subclasses redesigned
 *
 * Revision 1.8  2004/01/12 04:08:11  michalis
 * + Creature.id moved to new class Ided, Ided is now the base class of
 *   Being, Item and LinkToLocation
 *
 * Revision 1.7  2004/01/11 07:40:21  michalis
 * * Poprawki i ulepszenia dokumentacji
 *
 * Revision 1.6  2004/01/10 07:18:21  michalis
 * * small cosmetic adjustments for javadoc
 *
 * Revision 1.5  2004/01/10 03:08:58  michalis
 * * Many small fixes; mainly improvements to javadoc comments
 *
 * Revision 1.4  2004/01/08 22:45:38  michalis
 * - Usunalem wlasny blad - te "reczne" logi ktore zaczalem tworzyc
 *
 * Revision 1.3  2004/01/08 22:37:42  michalis
 * + CVS Logs appended to all .java files
 *
 *
 */
