/*
  Copyright 2004 Michalis Kamburelis.

  This file is part of "Jamy & Nory".

  "Jamy & Nory" is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  "Jamy & Nory" is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with "Jamy & Nory"; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package jamy_i_nory.game;

/** Klasa reprezentujca przejcie od lokacji do lokacji (gdzie lokacja
   to obiekt klasy {@link Location Location}). Miejsce startowe
   poczenia bdzie znane implicite - obiekty tej klasy bd
   elementami skadowymi lokacji z ktrej wychodz.
   Miejsce docelowe poczenia to pole {@link #target target}.
   Caa reszta klasy to tylko opis przejcia dla usera.

   <p>Linki s jednokierunkowe. Ale uywajc {@link #wayBack} moesz poczy
   dwa linki ze sob aby w efekcie mie jakby link 2-kierunkowy.
*/
public class LinkToLocation extends Ided {

  /** Miejsce docelowe przejcia.

     <p>Zwracam uwag e to pole nie jest
     final wic w jakich specjalnych przypadkach moemy w czasie
     trwania gry podmieni target przejcia. Chocia
     zauwamy e bdziemy mogi zmienia list aktywnych przej
     w kadej lokacji, patrz {@link Location#links Location.links},
     wic moliwo zmiany pola target przejcia suy wygodzie,
     nie daje nam nic czego nie umielibymy i tak zrobi. */
  public Location target;

  /** Krtka nazwa przejcia - to powinien by cig nie-biaych znakw
     bez polskich ogonkw. Ma si zaczyna od maej litery.
     Ten string (a przynajmniej jego 1-znaczny prefix w danej lokacji)
     bdzie musia wprowadzi user, wic powinna to by nazwa
     "krtka acz treciwa". Nie czymy tutaj adnych skrtw
     ponad potrzeb i nie piszmy np. "z" zamiast "zachod" - podczas
     pobierania stringu od usera postaramy si ju da userowi
     odpowiednie narzdzia eby nie musia wpisywa caego shortName. */
  final public String shortName;

  /** Dusza nazwa przejcia. Powinien to by jeden krtki rwnowanik
     zdania (wic zaczyna si z duej litery i koczy kropk),
     kilka sw. W wyjtkowych przypadkach zezwalamy tu jednak na
     co duszego. Ale w adnym wypadku nie zezwalamy tu na znaki
     nowej linii. */
  final public String longName;

  /** Obvious constructor. Create object with specified properties.
     {@link #wayBack} bdzie ustawione na null. */
  public LinkToLocation(Location aTarget, String aShortName, String aLongName)
  {
    this(aTarget, aShortName, aLongName, null);
  }

  public LinkToLocation(Location aTarget, String aShortName, String aLongName,
    LinkToLocation aWayBack)
  {
    target = aTarget;
    shortName = aShortName;
    longName = aLongName;
    fWayBack = aWayBack;
  }

  /** Zwraca "[" {@link #shortName shortName} "] " {@link #longName longName} */
  public String toString()
  {
    return "[" +shortName +"] " +longName;
  }

  private LinkToLocation fWayBack;

  /** Okrela link w drug stron. Patrz {@link #setWayBack} aby zobaczy
     jakie zalenoci musi spenia ta warto. */
  public LinkToLocation wayBack()
  { return fWayBack; }

  /** Okrel link w drug stron. Moe by null co oznacza e nie ma linku
     w drug stron.

     <p>Podobnie jak location.links() tak i pola wayBack linkw mog si
     zmienia w czasie (tzn. raz wayBack linku moe wskazywa na jeden,
     a raz na inny link). Zawsze musi by zachowana zaleno
     "dla l:LinkToLocation, source:Location, jeli link l jest na licie
     source.links() i l.wayBack != null to l.target.links() musi zawiera link
     l.wayBack i l.wayBack.target musi by rwne source.". Intuicyjnie,
     link wayBack musi by naprawd linkiem w drug stron. Ponadto jest
     zalecane aby l.wayBack.wayBack = l ale nie jest to stricte wymagane
     (innymi sowy, linki ktre w istocie s 2-kierunkowe MOG
     by oznaczone odpowiednio polami wayBack ale nie MUSZ; tzn. nic si
     nie stanie jeli gra nie bdzie zdawaa sobie sprawy e jaki link jest
     2-kierunkowy).  */
  public void setWayBack(LinkToLocation value)
  { fWayBack = value; }
  
  /** To parenthesized string. I.e. returns 
     " (" + comment + " [" +shortName +"])"
     (zwracam uwag e cao jest poprzedzona spacj). */
  public String toParenthString(String comment)
  {
    return " (" +comment + " [" +shortName +"])";
  }
  
  /** Jeeli wayBack == null to zwraca "". Wpp. zwraca wayBack.toParenthString,
     np. dla wayBack().shortName == "polnoc" i comment == "droga z powrotem to"
     zwraca " (droga z powrotem to [polnoc])" . */
  public String wayBackToParenthString(String comment)
  {
    if (wayBack() != null) 
      return wayBack().toParenthString(comment); else
      return "";
  }
}

/*
 * $Log: LinkToLocation.java,v $
 * Revision 1.1  2004/02/18 14:37:28  michalis
 * * pl stringi
 * * --help output changed (to reflect that those classes will be run by bin/scripts)
 * preparations for ClientApplet:
 * * Client rozdzielony na Client i ClientStandalone
 * * znaczna cz pakietu common przeniesiona do nowego pakietu game
 * X* make jamy_i_nory_client_applet.jar
 *
 * Revision 1.5  2004/02/14 01:09:43  michalis
 * + przejcia w drug stron (LinkToLocation.wayBack, Location.createTwoSidedLinks)
 * * followLink -> Being
 * * lepsze komunkaty o przejciach w setNowLocation
 * * Troll osabiony
 *
 * Revision 1.4  2004/02/09 23:08:50  michalis
 * + GPL license clause i copyrighty (moje i gdzieniegdzie Michaa, sorry Rafa
 *   ale naprawd nic nie zostao) do wszystkich *.java
 *
 * Revision 1.3  2004/01/12 04:08:11  michalis
 * + Creature.id moved to new class Ided, Ided is now the base class of
 *   Being, Item and LinkToLocation
 *
 * Revision 1.2  2004/01/11 10:20:06  michalis
 * + Right after initial creation; CVS Log added
 *
 */
