/*
  Copyright 2004 Michalis Kamburelis.

  This file is part of "Jamy & Nory".

  "Jamy & Nory" is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  "Jamy & Nory" is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with "Jamy & Nory"; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package jamy_i_nory.game;

import jamy_i_nory.common.*;

/** Location with dynamic (changing at the game-time) link.
   Klasa pomocnicza, ma uproci tworzenie lokacji ktrej jeden z linkw
   jest dynamiczny, tzn. moe si pojawia i znika w czasie gry.
   Np. mog to by drzwi ktre mona otwiera i zamyka. 
   
   <p>Podaj dyn link ktry ma by przestawiany i wyczany w trakcie
   dziaania gry w konstruktorze obiektu. Od tego momentu stan tego linku
   na licie links() bdzie zarzdzany przez t klas, tzn. ty ju
   nie moesz sam wywoywa add/removeLink(dynLink()) i tym podobne,
   ta klasa sama si tym zajmie. */
public class LocationWithDynLink extends Location {
  
  final LinkToLocation fDynLink;   
  
  /** Link ktry podae przy tworzeniu klasy. */
  public LinkToLocation dynLink() { return fDynLink; }

  /** Returns is dynLink currently open, i.e. links().contains(dynLink). */
  public boolean dynLinkOpen()
  {
    /* may be optimized later */
    return links().contains(fDynLink);
  }
  
  /** Set open status of the link. If dynLinkOpen = false and value = true
     then it will do addLink, else if dynLinkOpen = true and value = false
     it will do removeLink, else nothing will be done. So note that 
     appropriate putMessage messages will be called ONLY when 
     dynLinkOpen != false, i.e. when dynLinkOpen value will change. */
  public void setDynLinkOpen(boolean value)
  {
    if (dynLinkOpen() && !value)
      Utils.check( removeLink(dynLink()), 
        "LocationWithDynLink.setDynLinkOpen"); else
    if (!dynLinkOpen() && value)
      addLink(dynLink());
  }
  
  /** Shortcut to setDynLinkOpen(true) */
  public void openDynLink() {  setDynLinkOpen(true); }
  
  /** Shortcut to setDynLinkOpen(false) */
  public void closeDynLink() {  setDynLinkOpen(false); }
  
  final public String constDescriptionPart;
  final public String descriptionWhenLinkOpen;
  final public String descriptionWhenLinkClosed;
  
  /** If {@link #dynLinkOpen} returns
     constDescriptionPart + " " + descriptionWhenLinkOpen, 
     else returns
     constDescriptionPart + " " + descriptionWhenLinkClosed */
  public String description()
  {
    return (dynLinkOpen() ?
      constDescriptionPart + " " + descriptionWhenLinkOpen :
      constDescriptionPart + " " + descriptionWhenLinkClosed );
  }

  public LocationWithDynLink(World aWorld, String aName, 
    StatModifier aStatModifier, LinkToLocation aDynLink,
    String aConstDescriptionPart,
    String aDescriptionWhenLinkOpen,
    String aDescriptionWhenLinkClosed,
    boolean aDynLinkOpen)
  {
    super(aWorld, aName, aStatModifier);
    fDynLink = aDynLink;
    constDescriptionPart = aConstDescriptionPart;
    descriptionWhenLinkOpen = aDescriptionWhenLinkOpen;
    descriptionWhenLinkClosed = aDescriptionWhenLinkClosed;
    
    if (aDynLinkOpen) 
      addLink(fDynLink);
  }
  
  public LocationWithDynLink(World aWorld, String aName, 
    LinkToLocation aDynLink,
    String aConstDescriptionPart,
    String aDescriptionWhenLinkOpen,
    String aDescriptionWhenLinkClosed,
    boolean aDynLinkOpen)
  {
    this(aWorld, aName, new StatModifier(), aDynLink, 
      aConstDescriptionPart, aDescriptionWhenLinkOpen, 
      aDescriptionWhenLinkClosed, aDynLinkOpen);
  }
}

/*
 * $Log: LocationWithDynLink.java,v $
 * Revision 1.1  2004/02/18 14:37:28  michalis
 * * pl stringi
 * * --help output changed (to reflect that those classes will be run by bin/scripts)
 * preparations for ClientApplet:
 * * Client rozdzielony na Client i ClientStandalone
 * * znaczna cz pakietu common przeniesiona do nowego pakietu game
 * X* make jamy_i_nory_client_applet.jar
 *
 * Revision 1.1  2004/02/15 15:24:35  michalis
 * + pdf generating
 * + COPYING file
 * + Utils.check
 * + Location.add/removeLink
 * + LocationWithDynLink
 * * NoraOrBiblioteka.useItem simplified
 * * teraz i w bibliotece i w norze s drzwi
 * - Makefiles rozsiane po katalogach.
 * * Globalny Makefile rzdzi.
 *
 */
