/*
  Copyright 2004 Michalis Kamburelis.

  This file is part of "Jamy & Nory".

  "Jamy & Nory" is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  "Jamy & Nory" is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with "Jamy & Nory"; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package jamy_i_nory.game;

import jamy_i_nory.common.*;

/** Gracz; istota w wiecie gry sterowana przez czowieka. */
public class Player extends Being {
 
  /** Gromadzi wsplny kod wykonywany na kocu dwch konstruktorw
     Player. */
  private void postConstructor()
  {
    world.players().add(this);
    assert name != null;
    putMessage(nowLocationToString());  
  }

  /** Obvious constructor. Jak powiedziaem przy {@linkplain
     Being#Being(World,Location,Race,String) konstruktorze Being}
     ten konstruktor dodaje sam obiekt do aWorld.players().
     Gracz na pocztku gry jest ustawiany w lokacji zgodnie z
     {@link World#playerStartLocation aWorld.playerStartLocation(race)}.
     Gracz dostaje te pocztkowe wyposaenie w tej metodzie. 
     Note: player's name must not be null. 
     
     <p>Wysya te (wysya w sensie robienia putMessage) odpowiedni powitalny 
     komunikat do postaci - w tym momencie opisuje on poczatkow komnat 
     w ktrej znajduje si posta (to nie moe by robione w konstruktorze
     Being bo putMessage jest polimorficzne i wymaga zainicjowanego
     obiektu Player aby dziaa).  */
  public Player(World aWorld, Race race, String aName)
  {
    super(aWorld, aWorld.playerStartLocation(race), race, aName);    
    wear(USED_WEAPON, new Weapon(3.0f, "miecz", "krtki", null, 
      new StatModifier(), new DiceRandom(1, 6)));    
    postConstructor();
  }
  
  /** Konstruktor inicjujcy danego gracza z podanego inactivePlayer.
     Innymi sowy uywajc tego konstruktora zmieniasz nieaktywnego 
     gracza w aktywnego. Po skonstruowaniu w ten sposb obiektu Player
     musisz przesta uywa gdziekolwiek w programie podanego obiektu
     inactivePlayer. */     
  public Player(World aWorld, InactivePlayer inactivePlayer)
  {
    super(aWorld, inactivePlayer.nowLocation, inactivePlayer.race, 
      inactivePlayer.name, inactivePlayer.backpack, inactivePlayer.life,
      inactivePlayer.raceRandomStat);
    for (int i = 0; i < USED_COUNT; i++)
      setUsedItem(i, inactivePlayer.usedItem(i));
    postConstructor();  
  }
  
  /** As was specified in the interface of {@link Being Being},
     this method takes care of {@link Being#nowLocation 
     Being.nowLocation()}.players(). 
     
     <p>If this was not called from dead() (when value == null) or
     constructor (when starting nowLocation() == null) then
     after calling super.setNowLocation, this method does 
     putMessage(nowLocationToString()) to tell player where he is now. */
  public void setNowLocation(Location value, LinkToLocation link)
  {
    boolean casualCall = (nowLocation() != null && value != null);      
  
    if (nowLocation() != null) nowLocation().players().remove(this);
    super.setNowLocation(value, link);
    if (nowLocation() != null) nowLocation().players().add(this);
    
    if (casualCall) putMessage(nowLocationToString());
  }
  
  protected void beforeDeath(boolean goingInactive)
  {
    world.players().remove(this);
    super.beforeDeath(goingInactive);
  }
  
  /** W klasie Player ta metoda zawsze zwraca UNKNOWN bo przecie nie znamy
     zamiarw gracza. */
  public int realAttitude(Being target)
  {
    return Attitude.UNKNOWN;
  }
  
  /* rzeczy zwizane z messages do gracza ------------------------------ */
  
  private StringBuffer fMessage = new StringBuffer();
  
  /** Kady gracz ma w atrybut message w ktrym pamitamy przeznaczone 
     dla niego wiadomoci ktre naley do niego wysa w najbliszym 
     dogodnym momencie. Ta metoda zwraca albo string pusty (tzn. zerowej
     dugoci) oznaczajcy e do gracza nie przyszy adne nowe wiadomoci 
     albo string wielolinijkowy gdzie kada wiadomo jest
     zakoczona znakiem nowej linii (i w zwizku z tym caoc te jest
     zakoczona now lini). 
     
     <p>Wywoanie tej metody <b>czyci</b> zapamitane wiadomoci dla
     gracza (tzn. jeeli zaraz wywoasz znowu getMessage() to dostaniesz
     w odpowiedzi string pusty). To jest celowa niewygoda: jeeli wywoujesz
     t metod to powiniene to robic TYLKO po to aby wypisa (albo gdzie
     wysa) otrzymany String. W ten sposb unikamy te niebezpieczestwa 
     e kiedy zapomnisz wywoa co w rodzaju "clearMessages" (bo w ten
     sposb nie ma czego takiego jak clearMessages). */
  public String getMessage()
  {
    String result = fMessage.toString();
    fMessage.setLength(0);
    return result;
  }
  
  /** Dopisz do listy wiadomoci getMessage() wiadomoc mesg. 
     mesg nie powinno zawiera znakw nowej linii (ale w ostatecznoci moe)
     a ju zwaszcza nie moe si koczy znakiem nowej linii. 
     Jest final w tej klasie bo jest uywane z konstruktora (a wic zanim
     cokolwiek w podklasie zdy si zainicjowa). */
  final public void putMessage(String mesg)
  {
    fMessage.append(mesg +Utils.nl);
  }
}

/*
 * $Log: Player.java,v $
 * Revision 1.1  2004/02/18 14:37:30  michalis
 * * pl stringi
 * * --help output changed (to reflect that those classes will be run by bin/scripts)
 * preparations for ClientApplet:
 * * Client rozdzielony na Client i ClientStandalone
 * * znaczna cz pakietu common przeniesiona do nowego pakietu game
 * X* make jamy_i_nory_client_applet.jar
 *
 * Revision 1.36  2004/02/14 01:09:43  michalis
 * + przejcia w drug stron (LinkToLocation.wayBack, Location.createTwoSidedLinks)
 * * followLink -> Being
 * * lepsze komunkaty o przejciach w setNowLocation
 * * Troll osabiony
 *
 * Revision 1.35  2004/02/09 23:08:50  michalis
 * + GPL license clause i copyrighty (moje i gdzieniegdzie Michaa, sorry Rafa
 *   ale naprawd nic nie zostao) do wszystkich *.java
 *
 * Revision 1.34  2004/02/06 04:44:40  michalis
 * + InactivePlayer
 * + dodatkowy konstruktor do Being
 * + dodatkowy konstruktor Player(World, InactivePlayer)
 * * beforeDeath rozszerzone o parametr goingInactive
 * + Being.goInactive
 * * listToString moved from World to Utils
 * + ServerCommand : polecenie "mrozonki"
 *
 * Revision 1.33  2004/02/05 20:05:26  michalis
 * * Zrobiem eby w logach CVSa zapisanych w plikach byo wszdzie TO-DO
 *   z mylnikiem w rodku. Wprawdzie nieznacznie rozsynchronizowuje to
 *   zapisane w plikach logi z wynikiem 'cvs log' ale za to grep TO-DO
 *   (gdzie w wyrazie TO-DO naley usun mylnik) dziaa jak trzeba.
 *
 * Revision 1.32  2004/02/04 23:54:57  michalis
 * + dodany przedmiocik "szkatuka" do lokacji "Salon" z 3 piercieniami
 * * Being.beforeDeath - maa poprawka eby upuszcza plecak przez dropOnFloor
 *   (zeby byy komunikaty Player.putMessage)
 * * some Being interface clarifications
 * + dodane wywoanie putMessage(nowLocationToSingle()) do konstruktora Player,
 *   to pozwala nieco uproci PlayerConnection i JamySingle
 *
 * Revision 1.31  2004/02/04 18:56:43  michalis
 * + some improvements in Client (waitForMessageAndPrint, sSocketRead, sScoketWrite)
 * + Player.followLink
 * * UserCommand.execute does not return String (it returns void).
 *   It should simply append something to player.getMessage().
 *   Wiele uproszcze zapisu w UserCommand - np. komenda "idz" jest teraz
 *   krciutka i wszystkim zajmuje si Player.followLink i Player.setNowLocation.
 * + PlayerConnection.pushMessage, * PlayerConnection.dead slightly improved
 * - Server.sendMessage, - Server.broadcastMessage (useless - we use only
 *   sendPlayersMessages and PlayerConnection.pushMessage)
 *
 * Revision 1.30  2004/02/04 15:36:42  michalis
 * + Dziki pozbyciu si PlayersMessages moemy teraz wypisywa stringi
 *   o tym kto wchodzi/wychodzi z pomieszczenia w setNowLocation.
 * + Being.beforeDead, Being.toStringBase eby setNowLocation
 *   mogo take wywietla kto si pojawia (jako nowy gracz/potwr w wiecie)
 *   uywane z konstruktora Being i z dead()
 * * Server zosta uproszczony w wyniku powyszej zmiany, Being.dead te
 *   (chocia pojawia si niewielka strata funkcjonalnoci - patrz TO-DO przy
 *   toStringBase())
 *
 * Revision 1.29  2004/02/04 14:25:58  michalis
 * * dziki usuniciu PlayersMessages znowu mamy proste setNowLocation
 *   w Player i Creature i znowu mamy proste Player.dead i Creature.dead
 *
 * Revision 1.28  2004/02/04 14:00:25  michalis
 * * Being.tellEveryoneInNowLocation -> sayEveryoneInNowLocation
 *   (mniej mylca nazwa)
 * * Server powiadamia wszystkich graczy w komnacie e pojawi si nowy gracz
 * * PlayersMessages.tellEveryoneInLocation - docs updated,
 *   implemented in a cleaner way
 * + MessageGenerator
 * + MessageFormatter
 * * Being.attitude changed to realAttitude and observedAttitude
 * * Many fixes to use MessageFormatter when updating PlayersMessages.
 *   Many TO-DOs closed.
 * * Attitude.toStr translated to polish
 * * Poprawiony bug w Being.dead - Micha, nie mona tworzy PlayersMessages
 *   tu przed usuniciem gracza z gry - w czasie uywania PlayersMessages
 *   world.players() nie mog by modyfikowane. Z drugiej strony, to jest moja
 *   wina e jeszcze tego nie mona zrobi (bo to *powinno* by moliwe tylko
 *   musz zapisa inteligentniej PlayersMessages). OK, wanie mi przyszo do
 *   gowy jak to zrobi bardzo atwo i bardzo elegancko (klasa PlayersMessages
 *   w ogle nie bdzie potrzebna i unikniemy przerzucania wszdzie parametrw
 *   playersMessages) i w nastpnym commicie to bdzie zrobione (gupi, gupi,
 *   gupi jestem e od pocztku tego tak nie zrobiem).
 * * OK, serwer skunk.one.pl nie odpowiada wic nie bd commitowa, od razu
 *   zrobi to o czym mwiem powyej:
 *   + Player.getMessage, Player.putMessage
 *   + Location.tellEveryone (3 wersje)
 *   - PlayersMessages (funkcjonalno przerzucona w caoci do obiektw Player i
 *     Location)
 *   * masa uproszcze kodu (m.in. nie musimy ju przerzuca wszdzie
 *     parametrw PlayersMessages)
 *   - klasa Being.DeadAnswer (useless, teraz Being.dead() znowu zwraca prostego
 *     booleana)
 *   * w ogle Being.dead() jest teraz o niebo prostsze.
 *   Skoczyem! PlayersMessages usunite razem z wieloma niewygodami.
 *
 * Revision 1.27  2004/02/01 17:13:21  emes
 * * messages moved from UserCommand to Being
 *
 * Revision 1.26  2004/02/01 00:08:54  michalis
 * + Being.DeadAnswer, dead() returns DeadAnswer
 * - Player/Creature.dead
 * * Player/Creature.setNowLocation: remove object from world.players/creatures list
 * * World.action implemented cleaner
 * * Creature.attack dostaje parametr PlayersMessages
 * + Creature.ElfKiller, dodany do World do lokacji "Jama"
 * + PlayersMessages.append
 * + PlayerConnection.server, + Connection.createPlayer "server" parameter
 * + gdy gracz umiera to jest komunikat dla System.out serwera, komunikat "Umierasz"
 *   dla umierajcego gracza i komunikat "kto-tam umiera" dla pozostaych graczy
 *   w tej lokacji
 * * another bugfix for checking dead state of players in server.
 *   Wszystko przez to e na razie mam doc niewygodne metody testowania tego kodu
 *   skoro gracze nie mog si rusza w wiecie. No, ale myl e obecnie
 *   wszystko ju jest jak trzeba - wszystkie elementy kodu zostay
 *   przetestowane dziki ElfKillerowi.
 *
 * Revision 1.25  2004/01/19 02:27:03  michalis
 * + Player.setNowLocation updates nowLocation.players()
 *
 * Revision 1.24  2004/01/19 01:45:32  michalis
 * * Player.to[Detail]String removed and added to Being.to[Detail]String
 *   (because name is now in Being, not only in Player)
 *
 * Revision 1.23  2004/01/18 19:57:09  michalis
 * * die removed, more general dead in Being, overriden in Player and Creature
 *
 * Revision 1.22  2004/01/18 06:26:33  michalis
 * * HitModifier ranamed to DiceRandom (more general name, this class may
 *   be used in many other cases when we need the AkB+C kind of randomness),
 *   small changes: + constantBonus, - parameterless constructor (useless).
 *
 * Revision 1.21  2004/01/15 20:35:15  michalis
 * * docs improved, doc/Makefile changed to generate docs for jamy_i_nory.client
 *
 * Revision 1.20  2004/01/15 19:58:38  michalis
 * - Player.newInteractive removed (to JamySingle)
 *
 * Revision 1.19  2004/01/15 18:33:34  michalis
 * * Small updates; strings made more consistent (only polish, with ogonki)
 *
 * Revision 1.18  2004/01/15 15:40:39  michalis
 * * Creating backpack in Being implemented a little shorter
 * + example assertion in Player (aName != null)
 *
 * Revision 1.17  2004/01/15 15:05:18  emes
 * * backpack is created in Being constructor
 * * name is passed to Being
 *
 * Revision 1.16  2004/01/15 13:58:23  michalis
 * + Being.toDetailString, pokryte w Player, uzyte w UsedCommand,
 *   docs update in Creature, BaseTests corrected (name and kind are lowercase)
 *
 * Revision 1.15  2004/01/15 13:41:21  emes
 * * Armor & Weapon are not abstract anymore
 *
 * Revision 1.14  2004/01/15 13:18:59  michalis
 * + Being.doAction, Being.action i costam jeszcze
 *
 * Revision 1.13  2004/01/14 22:22:52  michalis
 * + Player.dead
 *
 * Revision 1.12  2004/01/14 19:53:41  michalis
 * * Item.weight splitted to internalWeight and weight(), + ItemsBag, + Player.backpack
 *
 * Revision 1.11  2004/01/14 18:50:04  michalis
 * * Item and all subclasses redesigned
 *
 * Revision 1.10  2004/01/13 14:33:11  michalis
 * + Being.toString(this) dziala ladnie, w UserCommand dodane "opisz miejsce/mnie"
 *
 * Revision 1.9  2004/01/13 04:38:55  michalis
 * + JamySingle (single-user version of game), LinkToLocationList, UserCommand
 *
 * Revision 1.8  2004/01/12 02:02:44  michalis
 * + StatModifier.intelligence(), class Attitude, Being.attitude(),
 *   Being.toString(Being) extended to show attitude
 *
 * Revision 1.7  2004/01/12 01:22:45  michalis
 * + Location/Being.toString(Being), czyli obiekty z punktu
 *   widzenia danego Being
 *
 * Revision 1.6  2004/01/11 23:51:54  michalis
 * * Location.world added and necessary changes made, some Player docs improved
 *
 * Revision 1.5  2004/01/11 10:03:13  michalis
 * + some TO-DO
 *
 * Revision 1.4  2004/01/11 07:40:21  michalis
 * * Poprawki i ulepszenia dokumentacji
 *
 * Revision 1.3  2004/01/11 06:51:55  michalis
 * + World class, Being.nowLocation
 *
 * Revision 1.2  2004/01/10 07:38:23  michalis
 * + I forgot to add CVS Logs when I created those files; well, now they are
 *
 */
