/*
  Copyright 2004 Michalis Kamburelis, Micha Saaban.

  This file is part of "Jamy & Nory".

  "Jamy & Nory" is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  "Jamy & Nory" is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with "Jamy & Nory"; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package jamy_i_nory.game;

/** Rasa istoty. Ta klasa jest immutable (wszystkie rzeczy wewnatrz s
   final), bo przeciez to czym jest dana rasa nie moe si zmienia
   w czasie gry. 
   
   <p>Zwracam uwag e tworzc podklasy Race mona tworzy hierarchie
   ras - np. rasa Elf moe mie "podrasy" Elfowa wrka, Elfowy dziadek,
   Elfowy cherubinek itd. W ten sposb ElfKiller moe je wszystkie
   zabija. Yeah. Mona te w ten sposb wyznacza w kodzie jakie
   wspczynniki s wsplne np. dla wszystkich elfw a jakie mog by
   rne dla rnych elfowych klas. 
   
   <p>Aha, moe te przyj do gowy pomys e rasy w zasadzie monaby
   zorganizowa jako hierarchi klas zakorzenion w Being. 
   Ale nie daoby si w takiej hierarchii umieci Creature i Player
   (i ich podklasy). */
public class Race {

  /** Nazwa rasy (powinno by jednym, ew. kilkoma sowami w rodzaju 
     "Elfowy ciapciak"). Not null. Zaczyna si z duej litery. */
  final public String name;
  /** Staly modyfikator rasy. To jest zestaw bazowych modyfikatorw
     z ktrymi zaczyna gr posta. */
  final public StatModifier stat;
  
  /** Damage that this race can make with bare hands (or tentacles,
     or tail, etc. - whatever is appropriate for this race). */
  final public DiceRandom unarmedDamage;
  
  /** Defense of skin of this race. */
  final public int skinDefense;

  /** Chwilowo wszystkie wspolczynniki maja taki sam modyfikator
     losowy. Ale to czysty przypadek, na pewno nie zawsze tak
     bedzie, wic prosz tu w aden sposob nie "pakowac" zapisu. 
     Musi by randomStatDices.length == StatModifier.MODIFS_COUNT,
     to jest sprawdzane asercj niej w kodzie.     
     
     By moe pniej przeniesiemy randomStatDices do jakiejs podklasy
     HumanLikeRace a w bazowej klasie Race bdziemy w randomStat()
     zwraca same zera. Na razie niepotrzebne, wic zostawiam jak jest. */
  final private DiceRandom[] randomStatDices = {
      new DiceRandom(1, 5, -3) /* -2 .. +2 */,
      new DiceRandom(1, 5, -3),
      new DiceRandom(1, 5, -3),
      new DiceRandom(1, 5, -3),
      new DiceRandom(1, 5, -3),      
    };
  
  /** Istota danej rasy przy "narodzinach" dostaje StatModifier
     wylosowany przez t funkcj. (mozna to pokryc w podklasach -
     - inne rasy mog mie tu inne wymagania). 
     Do tego zawsze ma by dodawany stay {@link #stat} danej rasy. */
  public StatModifier randomStat()
  {
    int[] randomMods = new int[StatModifier.MODIFS_COUNT];
    for (int i = 0; i < StatModifier.MODIFS_COUNT; i++)
      randomMods[i] = randomStatDices[i].random();
    return new StatModifier(randomMods);
  }

  public Race(String aName, StatModifier aStat, DiceRandom anUnarmedDamage, 
    int aSkinDefense)
  {
    assert randomStatDices.length == StatModifier.MODIFS_COUNT;
    name = aName;
    stat = aStat;
    unarmedDamage = anUnarmedDamage;
    skinDefense = aSkinDefense;
  }
  
  /** Describes {@link #name}, {@link #stat} and how {@link #randomStat}
     works in this class. No newlines. */
  public String toDetailString()
  {
    StringBuffer result = new StringBuffer(name +" [");
    
    for (int i = 0; i < StatModifier.MODIFS_COUNT; i++)
      result.append(StatModifier.addModifsNames[i] +": " 
        +(stat.getModifier(i) +randomStatDices[i].constantBonus)
        +"+"
        +new DiceRandom(randomStatDices[i].dicesCount,
                    randomStatDices[i].diceFaces) +"; ");
                    
    if (skinDefense != 0) result.append("ochrona skry: " +skinDefense +"; ");
    /* Prosze nie optymalizowac zapisu poniej; z czasem bdziemy z pewnoci
       chcieli jeszcze co tu doda wic niech nam bdzie wygodnie. */
    result.append("sia ataku bez broni: " +unarmedDamage +"; ");
    
    result.setLength(result.length() - 2);
    result.append("]");
    return result.toString();
  }  

  /** Returns {@link #name name} (does not return {@link #stat stat}
     just because usually you will not want it here). */
  public String toString()
  {
    return name;
  }
  
  /* example races ---------------------------------------- */
  
  static public class Elf extends Race {
    public Elf(String aName) 
    {
      super(aName, new StatModifier(8, 12, 10, 18, 10), new DiceRandom(1), 0);
    }
  }
  
  /* player races ---------------------------------------- */
  
  static final public Race elf = new Elf("Elf");
  static final public Race elfFairy = new Elf("Elfowa wrka");
  static final public Race human = 
    new Race("Czowiek", new StatModifier(10, 10, 8, 20, 10), 
      new DiceRandom(1), 0);
  static final public Race dwarf = 
    new Race("Krasnolud", new StatModifier(12, 8, 5, 22, 10), 
      new DiceRandom(1), 0); 
    
  /** Races that a Player can choose. Maybe later we will organize 
     this information differently, but for now it looks sufficient. */  
  static final public Race playerRaces[] = { human, elf, dwarf };  
}

/* 
 * $Log: Race.java,v $
 * Revision 1.2  2004/02/19 15:30:28  michalis
 * * Creature.attackPlayers/Creatures -> SimpleMindedCreature
 * + NonHostileCreature
 * + ElfFairy, ElfFairyWhistle
 * + wanderingFactor, teraz prawie wszystkie creatures wasaj si po wiecie
 *
 * Revision 1.1  2004/02/18 14:37:32  michalis
 * * pl stringi
 * * --help output changed (to reflect that those classes will be run by bin/scripts)
 * preparations for ClientApplet:
 * * Client rozdzielony na Client i ClientStandalone
 * * znaczna cz pakietu common przeniesiona do nowego pakietu game
 * X* make jamy_i_nory_client_applet.jar
 *
 * Revision 1.23  2004/02/09 23:08:50  michalis
 * + GPL license clause i copyrighty (moje i gdzieniegdzie Michaa, sorry Rafa
 *   ale naprawd nic nie zostao) do wszystkich *.java
 *
 * Revision 1.22  2004/02/07 21:11:32  michalis
 * * StatModifier.makeFinalStat changed (bugfix: must be maxLife > 0,
 *   bez tego GrojecKiller czsto zaczyna ju jako trup)
 * * StatModifier.Modifs are named (as it was before) addModifs
 * * small improvement to Creature.action (to m.in. eliminuje problem ktry mi
 *   opisalicie e po zabiciu DungeonExplorera on jeszcze zdy (martwy)
 *   wyj z pomieszczenia)
 * + Utils.clamped
 * * Being.getDelay lepiej dostrojone
 *
 * Revision 1.21  2004/01/18 19:45:46  michalis
 * * Minor doc updates
 *
 * Revision 1.20  2004/01/18 18:44:57  michalis
 * + DiceRandom(int), Race.unarmedDamage, Race.skinDefense, Being.armorDefense, Being.weaponHit
 *
 * Revision 1.19  2004/01/18 17:49:59  michalis
 * * DiceRandom.toString slightly improved (+ tests for it in BaseTests),
 * + Race.toDetailString (* used in JamySingle)
 *
 * Revision 1.18  2004/01/18 06:54:10  michalis
 * * some docs added
 *
 * Revision 1.17  2004/01/18 06:46:27  michalis
 * + Race.randomStat, Player.raceRandomStat
 *
 * Revision 1.16  2004/01/15 20:01:17  michalis
 * * Race.playerRaces made public (needed because now they are used in
 *   JamySingle.newPlayerInteractive)
 *
 * Revision 1.15  2004/01/15 13:19:42  emes
 * * s/Pullic/Public/
 *
 * Revision 1.14  2004/01/12 02:02:44  michalis
 * + StatModifier.intelligence(), class Attitude, Being.attitude(),
 *   Being.toString(Being) extended to show attitude
 *
 * Revision 1.13  2004/01/12 00:28:22  michalis
 * * elf/dwarf/human made public
 *
 * Revision 1.12  2004/01/12 00:26:08  michalis
 * * docs correction
 *
 * Revision 1.11  2004/01/12 00:22:22  michalis
 * + Race.elf/human/dwarf, comfortable for tests in BaseTests
 *
 * Revision 1.10  2004/01/11 10:11:12  michalis
 * * docs corrections
 *
 * Revision 1.9  2004/01/11 10:05:33  michalis
 * * Race.toString defined precisely
 *
 * Revision 1.8  2004/01/10 08:55:10  michalis
 * + StatModifier.maxLife and Being.life properties
 *
 * Revision 1.7  2004/01/10 07:19:37  michalis
 * + playerRaces i toString
 *
 * Revision 1.6  2004/01/10 06:23:48  michalis
 * * name made final
 *
 * Revision 1.5  2004/01/09 00:58:42  emes
 * - constructor made public
 *
 * Revision 1.4  2004/01/08 22:45:38  michalis
 * - Usunalem wlasny blad - te "reczne" logi ktore zaczalem tworzyc
 *
 * Revision 1.3  2004/01/08 22:37:42  michalis
 * + CVS Logs appended to all .java files
 *
 *
 */
