/*
  Copyright 2004 Michalis Kamburelis.

  This file is part of "Jamy & Nory".

  "Jamy & Nory" is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  "Jamy & Nory" is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with "Jamy & Nory"; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package jamy_i_nory.game;

import jamy_i_nory.common.*;

/** Commands available to user. This class handles parsing String
   to a command and executing this command. This class will not
   do any input/output (ani przy pomocy System.out/err/in ani przy
   pomocy jakiej instancji {@link jamy_i_nory.common.iosystem.IOSystem}).
   W ten sposb t klas bdzie mona uy zarwno w 
   niesieciowej wersji gry ({@link jamy_i_nory.single.JamySingle}) 
   jak i w architekturze klient-serwer.
   
   <p>Aby stworzy konkretn komend stwrz podklas tej klasy
   pokrywajc jej niestatyczn metod {@link #execute(Player, String[])} 
   i dodaj jej instancj do prywatnego pola availCommands.
   
   <p>Aby w typowy sposb uy tej klasy (czyli przekaza jej Stringa z komend
   i pozwoli aby ta klasa sama zaj곱 si parsowaniem komendy i wykonywaniem
   metody execute odpowiedniej klasy na licie availCommands)
   uyj statycznej metody {@link #execute(Player, String)}. 
   
   <p>TODO: niech opisz <przedmiot-id> dziaao te na przedmiotach
   w plecaku i niech wypisuje przedmioty w rodku plecaka (takie 
   "zabacz co bym dosta gdybym rozpakowa"). */
public abstract class UserCommand {

  /** Rzuc tym wyjatkiem zeby wyjsc z gry (unikajac pytania
     "czy chcesz zagrac jeszcze raz innym graczem"). */     
  static public class EndGameException extends Exception { }

  /* non-static members ---------------------------------------- */

  /** true means that this command should not be documented by the
     "help" command. */
  final public boolean hidden;

  /** Czy ta komenda jest worldAction ? Jezeli tak to
     1. przed uruchomieniem {@link #execute(Player, String[])}
     danej klasy {@link #execute(Player, String)}
     sprawdzi czy player.nextActionAllowed.
     Jezeli nie to wypisze odpowiedni komunikat i koniec roboty.
     2. Jezeli tak to odpowiednie execute konkretnej komendy
     zostanie wykonane, a potem zostanie wykonane
     player.calcNextAction(). */
  final public boolean worldAction;
  
  /** Krtka nazwa polecenia - to powinien by pierwszy token polecenia. 
     Uywane do polecenia "pomoc ARG1". */
  final public String name;

  UserCommand(boolean aHidden, boolean aWorldAction, String aName)
  {
    hidden = aHidden;
    worldAction = aWorldAction;
    name = aName;
  }

  /** Podaje komend rozbit na cig tokenw. */
  public abstract boolean matches(String[] command);

  public abstract String docLine();

  /** Execute given command. Pokryj w podklasach aby zaimplementowa
     konkretn komend. W czasie wykonywania tej metody zawartoc
     player.world.players() nie moe ulec zmianie (bo serwer uywa tej metody
     iterujc po licie world.players()); w szczeglnoci wic nie uywaj 
     w tej metodzie metody dead() dla obiektw klasy Player.
     
     <p>Ta metoda MUSI wywoa player.putMessage chocia raz z czym niepustym, 
     tzn. gracz ktry uy tej komendy musi dosta jak odpowied. 
     Klient polega na tym e na kad wysan komend dostaje jak 
     odpowied (w waitForMessageAndPrint). Dziki uyciu player.putMessage
     poniej (zamiast np. wymaga od tej metody zwracania Stringa)
     mamy 
     <ul>
       <li>prostot zapisu, wszystko do gracza lduje w jego getMessage.
         Wpp. czsto musielibymy kombinowa eby np. Being.attack zwracao
         Stringa zamiast po prostu inta (i niech sobie uaktualnia getMessage
         samo).
       <li>mamy pewno e wiadomoc zwracana przez t metod ley w dobrym
         miejscu pord innych wiadomoci do tego gracza; gdybymy uywali 
         stringa zawsze byby problem "co wypisywa pierwsze - warto zwrcon
         przez execute czy te aktualne player.getMessage()" ktry to problem
         nie daje si rozstrzygn elegancko bo wiadomoci powinny by po prostu
         wypisywane chronologiczne, tzn. w takiej kolejnoci w jakiej zachodz
         wywoujce je zdarzenia.
     </ul>
   
     @param player gracz ktry uruchomi t komend
     @param command komenda rozbita na tokeny
       (musi pasowa do komendy w sensie {@link #matches})*/
  public abstract void execute(Player player, String[] command)
  throws EndGameException;
  
  /** Metoda pomocnicza do implementacji rnych execute rnych komend:
     zamienia string na identyfikator, czyli Ided.id, czyli liczb typu long. 
     Jeli by bd to zwraca null i wypisuje odpowiedni komunikat 
     (przez player.putMessage) wpp. zwraca zboxowan warto long. */
  protected Long parseId(Player player, String s)
  {
    try {
      return Long.valueOf(s);
    } catch (NumberFormatException e) {
      player.putMessage("Nieprawidowy format identyfikatora " 
        +"(liczby cakowitej): " +e.getMessage());
      return null;
    }
  }

  /* static members ---------------------------------------- */

  final private static UserCommand availCommands[] =
  {
    new UserCommandBlackTokens("pomoc", "poka list dostpnych polece", 
      false, false) {
      public void execute(Player player, String[] command)
      {
        player.putMessage(
           "Oto spis dostpnych polece. "
          +"Polecenia ktrych nie mona wykonywa dowolnie czsto "
          +"(bo oddziauj na wiat gry) oznaczone s \"(*)\". "
          +"Zauwa e nie wszystkie polecenia s poniej wymienione - "
          +"w grze s pewne ukryte polecenia ktre musisz sam odkry.");
        for (int i = 0; i < UserCommand.availCommands.length; i++)
          if (! UserCommand.availCommands[i].hidden)
            player.putMessage("  " +UserCommand.availCommands[i].docLine());
      }
    },
    
    new UserCommandBlackTokens(new String[] { "pomoc", null },
      "wypisz polecenia ktrych nazwa (pierwszy czon) brzmi ARG1", 
      false, false) {
      public void execute(Player player, String[] command)
      {
        player.putMessage("Polecenia o nazwie \"" +command[1] +"\" :");
        for (int i = 0; i < UserCommand.availCommands.length; i++)
          if (UserCommand.availCommands[i].name.equals(command[1]))
            player.putMessage("  " +UserCommand.availCommands[i].docLine());
      }
    },    

    new UserCommandBlackTokens("koniec", 
      "zakocz gr (moesz zakoczy te gr koczc "+
      "strumien wejciowy (stdin) dla klienta konsolowego lub zamykajc "+
      "okienko z WindowManagera dla klienta w Swingu; wszystko zostanie "+
      "zakoczone \"czysto\")", false, false) {
      public void execute(Player player, String[] command)
      throws EndGameException
      {
        throw new EndGameException();
      }
    },

    new UserCommandBlackTokens(new String[] { "idz", null },
      "wyjd przejciem ARG1 z tej lokacji", false, true) {
      public void execute(Player player, String[] command)
      {
        LinkToLocation link = player.nowLocation().links().findShortName(command[1]);
        if (link == null)
          player.putMessage("Nie ma std przejcia \"" + command[1]+"\"."); else
          player.followLink(link);
      }
    },    

    new UserCommandBlackTokens(new String[] { "opisz", "miejsce" },
      "wywietla pen informacj o miejscu w ktrym si znajdujesz", 
      false, false) {
      public void execute(Player player, String[] command)
      {
        player.putMessage(player.nowLocationToString());
      }
    },

    new UserCommandBlackTokens(new String[] { "opisz", "mnie" },
      "wywietla pen informacj o Tobie", false, false) {
      public void execute(Player player, String[] command)
      {
        player.putMessage(player.toDetailString());
      }
    },
    
    new UserCommandBlackTokens(new String[] { "opisz", null },
      "wywietla pen informacj o przedmiocie lub przejciu lub istocie "
        +"w biecej lokacji o id = ARG1", false, false) {
      public void execute(Player player, String[] command)
      {
        Long ido = parseId(player, command[1]);
        if (ido == null) return;
        long id = ido.longValue();
        
        Being b = (Being) player.nowLocation().beings().byId(id);
        if (b != null) { player.putMessage(b.toDetailString(player)); return; }
        
        LinkToLocation l = (LinkToLocation) player.nowLocation().links().byId(id);
        if (l != null) { player.putMessage(""+l); return; }
        
        Item i = (Item) player.nowLocation().items().byId(id);
        if (i != null) { player.putMessage(""+i); return; }
        
        player.putMessage("Identyfikator " +id +" nie okrela adnego "
          +"przedmiotu, przejcia lub istoty w tym miejscu.");
      }
    },    

    new UserCommandBlackTokens(new String[] { "upusc", null },
      "upuszcza na ziemi ARG1. ARG1 = " +
        Utils.stringArrayConcat(Being.usedItemInputName, " | ") +
        " lub identyfikator przedmiotu noszonego lub w plecaku",
        false, true) {
      public void execute(Player player, String[] command)
      {
        int usedItemNum = Utils.stringArrayIndexOf(Being.usedItemInputName,
          command[1]);
        if (usedItemNum == -1) 
        {
          Long ido = parseId(player, command[1]);
          if (ido == null) return;
          
          player.dropById (ido.longValue());
        } else
        {            
          /* Moglibymy tu nie sprawdza czy usedItem == null, drop()
             i tak zadziaa jak NOP. Ale my chcemy powiedzie o tym userowi. */
          if (player.usedItem(usedItemNum) != null) 
            player.drop(usedItemNum); else
            player.putMessage("Nie uywasz przedmiotu rodzaju \"" +
              Being.usedItemName[usedItemNum] +"\" wic nie moesz go upuci.");
        }
      }
    },

    new UserCommandBlackTokens(new String[] { "podnies", null },
            "podnosi przedmiot o podanym ID i umieszcza w plecaku",
            false, true) {
      public void execute (Player player, String[] command)
      {
        Long ido = parseId(player, command[1]);
        if (ido == null) return;

        player.pickById (ido.longValue());
      }
    },

    new UserCommandBlackTokens (new String[] { "zaloz", null },
            "element o podanym ID staje sie czci wyposaenia, zamieniajc si "+
            "z obecnie uywanym",
            false, true) {
      public void execute (Player player, String[] command)      
      {
        Long ido = parseId(player, command[1]);
        if (ido == null) return;
          
        player.wearById (ido.longValue());
      }
    },
    
    new UserCommandBlackTokens (new String[] { "uzyj" , null },
      "uyj posiadany przedmiot o id = ARG1 w danej lokacji, np. uycie "+
      "klucza otwiera/zamyka drzwi itd.",
      false, true) {
      public void execute (Player player, String[] command)      
      {  
        Long ido = parseId(player, command[1]);
        if (ido == null) return;
        long id = ido.longValue();
        
        Item item = player.findItem(id);
        if (item == null) 
          player.putMessage("Nie posiadasz przedmiotu o id = " +id); else
          player.nowLocation().useItem(player, item);
      }
    },
    
    new UserCommandBlackTokens (new String[] { "rozpakuj" , null },
      "szuka przedmiotu ktry zawiera w sobie inne przedmioty (plecaki itp.) "+
      "o id = ARG1 wrd posiadanych przedmiotw (tzn. we wasnym plecaku) i "+
      "wrd przedmiotw lecych na ziemi i rozpakowuje go - tzn. wyjmuje "+
      "wszystkie przedmioty ze rodka i odkada je do wasnego plecaka lub "+
      "na ziemi.",
      false, true) {
      public void execute (Player player, String[] command)      
      {  
        Long ido = parseId(player, command[1]);
        if (ido == null) return;
        
        player.unpackItem(ido.longValue());
      }
    },
    
    new UserCommandBlackTokens (new String[] {"atakuj", null},
            "atakuj stworzenie o id = ARG1",
            false, true) {
      public void execute (Player player, String[] command)
      {
        Long ido = parseId(player, command[1]);
        if (ido == null) return;
        long id = ido.longValue();

        Being target = (Being) player.nowLocation().beings().byId (id);
        if (target == null) {
            player.putMessage("Nie ma postaci o id = " + id);
            return;
        }

        player.attack (target);
      }
    },
    
    new UserCommand(false, false, "powiedz") {
    
      public boolean matches(String[] command)
      {        
        return (command.length>0) && command[0].equals(name);
      }

      public String docLine()
      {
        return name +" S - powiedz do wszystkich graczy w tej lokacji S";
      }
      
      public void execute(Player player, String[] command)
      {
        String message = Utils.stringArrayConcat(command, 1, command.length-1, " ");
        player.sayEveryoneInNowLocation(message);
        /* Nie powinnimy graczowi robi echa message. */
        player.putMessage("Powiedziae.");
      }
    },

    new UserCommandBlackTokens("samobojstwo",
      "popenia samobjstwo; w ten sposb moesz zaczc gr od nowa nowym graczem",
      false, true) {
      public void execute(Player player, String[] command)
      {
        player.life = 0;
        player.tellEveryoneInNowLocation(new MessageFormatter(
          "%1 popenia samobjstwo.", player));
        player.putMessage("Popeniasz samobjstwo.");
      }
    },
    
    new UserCommandBlackTokens(new String[] { "debug", "players" },
      "show all players in world",
      true, false) {
      public void execute(Player player, String[] command)
      {
        player.putMessage(player.world.allPlayersToString());
      }
    },
    
    new UserCommandBlackTokens(new String[] { "debug", "creatures" },
      "show all creatures in world",
      true, false) {
      public void execute(Player player, String[] command)
      {
        player.putMessage(player.world.allCreaturesToString());
      }
    },    
      
  };

  final static private int whichCommand(String[] command)
  {
    for (int i = 0; i < availCommands.length; i++)
      if (availCommands[i].matches(command)) return i;
    return -1;
  }

  /** For the given commandStr, execute the right action. 
     Parametr player znaczy to samo co dla niestatycznej wersji 
     {@link #execute(Player,String[])}. If commandStr is not valid, it will do 
     player.putMessage(some message like "Invalid command.").
     
     <p>It will also check (for commands that have worldAction == true)
     that player.nextActionAllowed() - if it is, it will do a command 
     and call player.calcNextAction(). Else, it will do
     player.putMessage(some message like "you must wait longer.").
     
     <p>Note: the assertion "execute ALWAYS does at least once 
     player.putMessage()" is valid. */
  final static public void execute(Player player, String commandStr)
  throws EndGameException
  {
    String[] command = commandStr.split("\\s+");

    int whichCommand = whichCommand(command);
    if (whichCommand == -1)
    {
      player.putMessage("Nieprawidowe polecenie \"" +commandStr +"\". "+
        "Uyj \"pomoc\" aby zobaczy list dostpnych polece.");
    } else
    {
      UserCommand c = availCommands[whichCommand];
      if (c.worldAction)
      {
        if (! player.nextActionAllowed())
        {
          float waitSecs = ((float) (player.nextAction() -
            System.currentTimeMillis())) / 1000.0f;
          /* Line below is only to not show user negative time (in case
             some large amount of time elapsed since calling System.currentTimeMillis()
             inside player.nextActionAllowed) */
          if (waitSecs < 0.0f) waitSecs = 0.0f;
          player.putMessage("Nie moesz jeszcze wykona tej akcji. "+
            "Musisz poczeka " +waitSecs +" sekund.");
        } else
        {
          c.execute(player, command);
          player.calcNextAction();
        }
      } else
        c.execute(player, command);
    }
  }
}

/*
 * $Log: UserCommand.java,v $
 * Revision 1.1  2004/02/18 14:37:34  michalis
 * * pl stringi
 * * --help output changed (to reflect that those classes will be run by bin/scripts)
 * preparations for ClientApplet:
 * * Client rozdzielony na Client i ClientStandalone
 * * znaczna cz pakietu common przeniesiona do nowego pakietu game
 * X* make jamy_i_nory_client_applet.jar
 *
 * Revision 1.26  2004/02/15 20:11:45  michalis
 * * lepszy interfejs w Swingu (JTextPane ze stylami, echo komendy bold'em,
 *   pole do dalszych upiksze)
 * * bugfix do "powiedz" w UserCommand (dla przypadku command.length == 0)
 * * linki do nory i biblioteki poczone przez wayBack
 *
 * Revision 1.25  2004/02/15 15:24:35  michalis
 * + pdf generating
 * + COPYING file
 * + Utils.check
 * + Location.add/removeLink
 * + LocationWithDynLink
 * * NoraOrBiblioteka.useItem simplified
 * * teraz i w bibliotece i w norze s drzwi
 * - Makefiles rozsiane po katalogach.
 * * Globalny Makefile rzdzi.
 *
 * Revision 1.24  2004/02/14 11:01:51  michalis
 * * demo otwierania i zamykania drzwi kluczem zrobione
 *
 * Revision 1.23  2004/02/09 23:08:51  michalis
 * + GPL license clause i copyrighty (moje i gdzieniegdzie Michaa, sorry Rafa
 *   ale naprawd nic nie zostao) do wszystkich *.java
 *
 * Revision 1.22  2004/02/09 19:35:20  michalis
 * * Uaktualnienia dokumentacji
 *
 * Revision 1.21  2004/02/05 23:27:39  michalis
 * * jeszcze mae poprawki w kwestii kiedy pisa TO-DO a kiedy to samo bez mylnika
 * * IOSwing konstruktor: parametr windowTitle
 * + Server obsuguje parametr --swing, ma pole IOSystem
 * + Utils.collectionToMultiLineString(Collection,String)
 * * Server mona ju kontrolowa interfejsem IOSystem, polecenia "x", "h", "p", "c"
 * + World.allCreatures/PlayersToString,
 * * UserCommand. "debug players/creatures" uywaj World.allCreatures/PlayersToString
 * * Poprawka w Server.ServerListener.run przy sleep
 *
 * Revision 1.20  2004/02/05 20:05:26  michalis
 * * Zrobiem eby w logach CVSa zapisanych w plikach byo wszdzie TO-DO
 *   z mylnikiem w rodku. Wprawdzie nieznacznie rozsynchronizowuje to
 *   zapisane w plikach logi z wynikiem 'cvs log' ale za to grep TO-DO
 *   (gdzie w wyrazie TO-DO naley usun mylnik) dziaa jak trzeba.
 *
 * Revision 1.19  2004/02/04 22:30:32  michalis
 * * polecenie "samobojstwo" powoduje odpowiednie putMessage dla innych graczy w tym
 *   pomieszczeniu
 * + Being.findUsedItem, * Being.findItem i dropById uproszczone dziki temu
 * + Being.unpackItem
 * + UserCommand: polecenie "rozpakuj"
 *
 * Revision 1.18  2004/02/04 21:19:20  michalis
 * * polecenia "upusc" i "podnies" bardziej elegancko zapisane -
 *   powiadamianie o zmianach przez putMessage wepchnite w gd metod Being.
 *   Teraz fakt e np. polecenie "zaloz" moe w efekcie spowodowa podnie,
 *   potem waciwe za, a potem upu, jest widoczny w komunikatach dla gracza.
 *
 * Revision 1.17  2004/02/04 20:33:47  michalis
 * * lepszy komunikat bdu w UserCommand.parseId
 * * zapis komendy "zaloz" znacznie uproszczony, jej putMessage przerzucone do wearById
 *
 * Revision 1.16  2004/02/04 20:04:27  michalis
 * * wreszcie poprawione messages dla Being.attack
 * * Being.attack nie pozwala zaatakowa samego siebie
 * + UserCommand.parseId, uywane wszdzie w UserCommand.execute gdzie potrzebny id
 *
 * Revision 1.15  2004/02/04 18:56:43  michalis
 * + some improvements in Client (waitForMessageAndPrint, sSocketRead, sScoketWrite)
 * + Player.followLink
 * * UserCommand.execute does not return String (it returns void).
 *   It should simply append something to player.getMessage().
 *   Wiele uproszcze zapisu w UserCommand - np. komenda "idz" jest teraz
 *   krciutka i wszystkim zajmuje si Player.followLink i Player.setNowLocation.
 * + PlayerConnection.pushMessage, * PlayerConnection.dead slightly improved
 * - Server.sendMessage, - Server.broadcastMessage (useless - we use only
 *   sendPlayersMessages and PlayerConnection.pushMessage)
 *
 * Revision 1.14  2004/02/04 14:00:25  michalis
 * * Being.tellEveryoneInNowLocation -> sayEveryoneInNowLocation
 *   (mniej mylca nazwa)
 * * Server powiadamia wszystkich graczy w komnacie e pojawi si nowy gracz
 * * PlayersMessages.tellEveryoneInLocation - docs updated,
 *   implemented in a cleaner way
 * + MessageGenerator
 * + MessageFormatter
 * * Being.attitude changed to realAttitude and observedAttitude
 * * Many fixes to use MessageFormatter when updating PlayersMessages.
 *   Many TO-DOs closed.
 * * Attitude.toStr translated to polish
 * * Poprawiony bug w Being.dead - Micha, nie mona tworzy PlayersMessages
 *   tu przed usuniciem gracza z gry - w czasie uywania PlayersMessages
 *   world.players() nie mog by modyfikowane. Z drugiej strony, to jest moja
 *   wina e jeszcze tego nie mona zrobi (bo to *powinno* by moliwe tylko
 *   musz zapisa inteligentniej PlayersMessages). OK, wanie mi przyszo do
 *   gowy jak to zrobi bardzo atwo i bardzo elegancko (klasa PlayersMessages
 *   w ogle nie bdzie potrzebna i unikniemy przerzucania wszdzie parametrw
 *   playersMessages) i w nastpnym commicie to bdzie zrobione (gupi, gupi,
 *   gupi jestem e od pocztku tego tak nie zrobiem).
 * * OK, serwer skunk.one.pl nie odpowiada wic nie bd commitowa, od razu
 *   zrobi to o czym mwiem powyej:
 *   + Player.getMessage, Player.putMessage
 *   + Location.tellEveryone (3 wersje)
 *   - PlayersMessages (funkcjonalno przerzucona w caoci do obiektw Player i
 *     Location)
 *   * masa uproszcze kodu (m.in. nie musimy ju przerzuca wszdzie
 *     parametrw PlayersMessages)
 *   - klasa Being.DeadAnswer (useless, teraz Being.dead() znowu zwraca prostego
 *     booleana)
 *   * w ogle Being.dead() jest teraz o niebo prostsze.
 *   Skoczyem! PlayersMessages usunite razem z wieloma niewygodami.
 *
 * Revision 1.13  2004/02/02 00:28:21  michalis
 * + Utils.stringArrayConcat(String, int, int, String)
 * * UserCommand some TO-DO in komenda "powiedz"  removed
 *
 * Revision 1.12  2004/02/02 00:08:09  michalis
 * * Tylko zmieniem kolejno komend w availCommands eby "pomoc" wywietlaa
 *   "samobojstwo" na koncu
 *
 * Revision 1.11  2004/02/01 20:57:59  michalis
 * + UserCommandBlackTokens, wszystkie dotychczasowe komendy zmienione jako
 *   dziedziczce z UserCommandBlackTokens zamiast bezporednio z UserCommand
 * + Being.tellEveryoneInLocation
 * + UserCommand: komenda "powiedz"
 * + Goblin mwi e jest godny (naprawd)
 *
 * Revision 1.10  2004/02/01 19:58:46  michalis
 * + Being.classifyUsedItem
 * * wearById zaimplementowane bardziej elagancko uywajc Being.classifyUsedItem,
 *   zwraca teraz int
 * * polecenie "zaloz" zwraca teraz adny string - co i jako co jest noszone
 *
 * Revision 1.9  2004/02/01 19:14:07  michalis
 * * poleceine "uzyj" renamed to "zaloz"
 * + Being.findItem
 * + Location.useItem
 * + polecenie "uzyj" zaimplementowane jako "uzyj cos w danej lokacji"
 * - some TO-DOs in UserCommand removed (done!)
 *
 * Revision 1.8  2004/02/01 18:46:43  michalis
 * * funkcjonalno polecenia "wyrzu" doczona do polecenia "upusc"
 *
 * Revision 1.7  2004/02/01 18:20:22  michalis
 * * Being.wearById changed (nie mona zakada e co co jest StatModifierItem
 *   jest na pewno Armor, Ring albo Weapon, poza tym mae uproszczenia zapisu)
 * + PlayersMessages.tellEveryoneInLocation(Location, String, Being)
 * * Being.dropOnFloor: docs slightly improved, implementation simplified
 * * Being.pickFromFloor: podobnie implementation simplified
 * - some comments from UserCommand removed, * some TO-DOs moved to Being
 *
 * Revision 1.6  2004/02/01 17:17:23  emes
 * + 'wyrzuc' command added
 *
 * Revision 1.5  2004/02/01 17:13:21  emes
 * * messages moved from UserCommand to Being
 *
 * Revision 1.4  2004/02/01 02:03:33  michalis
 * + PlayersMessages.tellEveryoneInLocation(Location loc, String message, Player except)
 * * UserCommand "podnies" i "upusc" dopisuja odpowiednie rzeczy do playersMessages
 *
 * Revision 1.3  2004/02/01 01:16:55  michalis
 * * small docs improvements
 * - Client testing message ("Enter lines ...") removed
 *
 * Revision 1.2  2004/02/01 00:08:54  michalis
 * + Being.DeadAnswer, dead() returns DeadAnswer
 * - Player/Creature.dead
 * * Player/Creature.setNowLocation: remove object from world.players/creatures list
 * * World.action implemented cleaner
 * * Creature.attack dostaje parametr PlayersMessages
 * + Creature.ElfKiller, dodany do World do lokacji "Jama"
 * + PlayersMessages.append
 * + PlayerConnection.server, + Connection.createPlayer "server" parameter
 * + gdy gracz umiera to jest komunikat dla System.out serwera, komunikat "Umierasz"
 *   dla umierajcego gracza i komunikat "kto-tam umiera" dla pozostaych graczy
 *   w tej lokacji
 * * another bugfix for checking dead state of players in server.
 *   Wszystko przez to e na razie mam doc niewygodne metody testowania tego kodu
 *   skoro gracze nie mog si rusza w wiecie. No, ale myl e obecnie
 *   wszystko ju jest jak trzeba - wszystkie elementy kodu zostay
 *   przetestowane dziki ElfKillerowi.
 *
 * Revision 1.1  2004/01/31 20:47:48  michalis
 * * JamySingle parameter console changed to --console
 * * UserCommand moved from tests/ to common/
 * * UserCommand small changes (very small doc updates, some things made public
 *   to be available from outside of package jamy_i_nory.common)
 * * tests/SimpleClient.java moved to client/Client.java
 * * tests/local_client.sh moved to client/ and modified
 * * Server: StringBuffer[] changed to PlayerMessages everywhere
 *
 * Logs as tests/UserCommand.java:
 * Revision 1.27  2004/01/31 20:10:31  michalis
 * + IOSystem: some TO-DO with comments moved from UserCommand to IOSystem
 * * UserCommand docs improved
 * * UserCommand.s renamed to commandPattern
 * + PlayersMessages
 * * World.messageToPlayersInLocation moved to PlayersMessages.tellEveryoneInLocation
 * * World.action changed to use PlayersMessages (much cleaner code now!)
 * * Creature.action and doAction use PlayersMessages
 * * UserCommand.execute interface changed:
 *   - world (excessive, one can always use player.world)
 *   + playersMessages (to tell other players about effects of this command)
 *   + returns String (tell command invoker about effects of this command)
 * * every command's execute changed to use new interface:
 *   nothing uses JamySingle.io, nothing may call Player.dead()
 *
 * Revision 1.26  2004/01/19 01:51:53  michalis
 * + "debug creatures" command
 *
 * Revision 1.25  2004/01/18 22:53:26  michalis
 * + "opisz <id>" command - describes in detail item, link or being in this
 *   location (and maybe we should even extend it later to more ids, like
 *   items in player's backpack ?)
 *
 * Revision 1.24  2004/01/18 22:13:03  michalis
 * + some TO-DO
 *
 * Revision 1.23  2004/01/18 20:05:42  michalis
 * * Small fix for last update, sorry, last cvs commit process was killed
 *
 * Revision 1.22  2004/01/18 20:00:29  michalis
 * + target.dead checked in UserCommand after "atakuj"
 *   (not in attack !! never check dead in attack -
 *   nowLocation.creatures() and world.players() lists must not be modified
 *   from there - it will be required when Creatures will be able to attack),
 *   message for user
 *
 * Revision 1.21  2004/01/18 19:47:11  emes
 * * we can also attack players
 *
 * Revision 1.20  2004/01/18 19:30:18  emes
 * * "atakuj" modified
 *
 * Revision 1.19  2004/01/18 19:24:35  emes
 * + "atakuj" command added
 *
 * Revision 1.18  2004/01/17 16:57:56  michalis
 * * More docs for command "koniec"
 *
 * Revision 1.17  2004/01/16 20:51:19  michalis
 * + Wypisujc "nieprawidowe polecenie" teraz pokazuje *jakie* polecenie otrzyma.
 *
 * Revision 1.16  2004/01/16 00:00:52  michalis
 * * System.out.println -> JamySingle.io.println
 *
 * Revision 1.15  2004/01/15 20:55:20  emes
 * * 'uzyj' command to put item into equipment slot
 *
 * Revision 1.14  2004/01/15 20:04:38  michalis
 * + jamy_i_nory.client.IOSystem
 * + jamy_i_nory.client.IOStd
 * + JamySingle.io, JamySingle.newPlayerInteractive (from Player.newInteractive)
 * * JamySingle and UserCommand use JamySingle.io instead of System.in/out
 *
 * Revision 1.13  2004/01/15 19:00:18  emes
 * + added 'podnies' command
 *
 * Revision 1.12  2004/01/15 18:33:36  michalis
 * * Small updates; strings made more consistent (only polish, with ogonki)
 *
 * Revision 1.11  2004/01/15 15:12:09  emes
 * + message when dropping item
 *
 * Revision 1.10  2004/01/15 13:58:25  michalis
 * + Being.toDetailString, pokryte w Player, uzyte w UsedCommand,
 *   docs update in Creature, BaseTests corrected (name and kind are lowercase)
 *
 * Revision 1.9  2004/01/14 22:29:23  michalis
 * + UserCommand.EndGameException, "debug players" i "samobojstwo" w UserCommand,
 *   JamySingle much cleaner now
 *
 * Revision 1.8  2004/01/13 20:37:02  michalis
 * + UserCommand "upusc" ogolna, BaseTests updated (weapon -> weapon())
 *
 * Revision 1.7  2004/01/13 14:44:44  michalis
 * + polecenia "upusc bron/zbroje"
 *
 * Revision 1.6  2004/01/13 14:33:13  michalis
 * + Being.toString(this) dziala ladnie, w UserCommand dodane "opisz miejsce/mnie"
 *
 * Revision 1.5  2004/01/13 14:15:38  michalis
 * * Rally really last update to using player.nextAction in JamySingle
 *
 * Revision 1.4  2004/01/13 13:09:52  michalis
 * * Fixes for nextAction
 *
 * Revision 1.3  2004/01/13 11:29:58  michalis
 * * calcNextAction used in JanySingle
 *
 * Revision 1.2  2004/01/13 06:17:23  michalis
 * * Mae przeprowadzki kodu i notki w dokumentacji w zwizku z calcNextAction
 *
 * Revision 1.1  2004/01/13 04:39:00  michalis
 * + JamySingle (single-user version of game), LinkToLocationList, UserCommand
 *
 */
