/*
  Copyright 2004 Michalis Kamburelis.

  This file is part of "Jamy & Nory".

  "Jamy & Nory" is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  "Jamy & Nory" is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with "Jamy & Nory"; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package jamy_i_nory.game;

import java.util.*;
import jamy_i_nory.common.*;

/** Ta klasa opisuje cay wiat gry. wiat gry to zasadniczo zbir lokacji
   (obiektw klasy {@link Location Location}) w ktrych rozrzucone s 
   rne przedmioty (obiekty klasy {@link Item Item}) i postacie 
   ({@link Being Being}, czyli postacie zarwno sterowane przez komputer
   ({@link Creature Creature}) jak i przez ludzi ({@link Player Player})).
   
   <p>Ta klasa nie ma zawiera adnych technicznych szczegw zwizanych
   z tym e jest to gra sieciowa, e jest gdzies jaki serwer gry,
   e rni gracze cz si poprzez sie itd. W szczeglnoci za pomoc
   tej klasy musi si da elegancko zaimplementowa nie-sieciow
   wersj naszej gry, tzn. wersj dla jednego gracza ktry chce sobie
   po prostu pogra z komputerem. */
public class World {

  /** Wszystkie lokacje gry. 
  
     <p>Ta tablica moe zawiera tylko rne (w sensie referencji)
     elementy klasy {@link Location Location}. adnych nulli.  

     <p>Read-only after object World creation. (making it non read-only
     would cause some unnecessary troubles and uncertainties;
     zamiast tego bdziemy pozwalali sobie modyfikowa w czasie
     gry <b>poczenia</b> pomidzy lokacjami
     ({@link LinkToLocation obiekty klasy LinkToLocation})). 
     
     <p>Wszystkie lokacje na tej licie mog zawiera wrd swoich 
     {@link Location#links links()} tylko takie obiekty klasy 
     {@link LinkToLocation LinkToLocation} ktre prowadz innych lokacji 
     w tym samym wiecie, czyli ich {@link LinkToLocation#target target} 
     te musi by wtedy na tej licie. */
  final public Location[] locations;
  
  private IdedList fPlayers = new IdedList();
  
  /** Wszyscy gracze ktrzy aktualnie znajduj si w wiecie. 
     
     <p>Podobnie jak przy {@link Location#creatures Location.creatures()} 
     nie okrelamy tutaj konkretnej implementacji interfejsu List i
     nie okrelamy czy ta funkcja nie bdzie musiaa przeprowadza
     jakiego skomplikowanego wyliczania swojej wartoci.

     <p>Ta lista moe zawiera tylko rne (w sensie referencji)
     elementy klasy {@link Player Player}. adnych nulli.

     <p>Ta lista moe by bezporednio modyfikowana 
     tylko przez {@link Player#Player konstruktory klasy Player}
     (a dokadniej Player.postConstructor) i metod {@link Player#beforeDeath}
     (wywoywan z {@link Being#dead} i {@link Being#goInactive}). */
  public IdedList players() { return fPlayers; }
  
  private IdedList fCreatures = new IdedList();
  
  /** Wszyscy gracze ktrzy aktualnie znajduj si w wiecie. 
     
     <p>Podobnie jak przy {@link Location#creatures Location.creatures()} 
     nie okrelamy tutaj konkretnej implementacji interfejsu List i
     nie okrelamy czy ta funkcja nie bdzie musiaa przeprowadza
     jakiego skomplikowanego wyliczania swojej wartoci.

     <p>Ta lista moe zawiera tylko rne (w sensie referencji)
     elementy klasy {@link Player Player}. adnych nulli.

     <p>Ta lista moe by bezporednio modyfikowana 
     tylko przez {@link Creature#Creature konstruktor klasy Creature} i
     metod {@link Creature#beforeDeath} 
     (wywoywan z {@link Being#dead} i {@link Being#goInactive}). */
  public IdedList creatures() { return fCreatures; }

  /** To samo co beings().byId() gdzie beings() to suma {@link #creatures}
     i {@link #players}. (chwilowo nie ma w tej klasie zaimplementowanej
     metody beings() ale nawet jeli bdzie to niniejsza funkcja zachowa wano 
     bo jej implementacja nie wymaga konstrukcji po drodze adnej pomocniczej
     listy beings()) */
  public Being beingsFindById(long id)
  {
    Being result = (Being) players().byId(id);
    if (result == null) 
      result = (Being) creatures().byId(id);    
    return result;
  }
  
  /** fPlayersStartLocation must be a member of aLocations. */
  final private Location fPlayersStartLocation;

  /** Startowa lokacja dla gracza danej rasy. 
     Na pewno bdzie to jeden z elementw {@link #locations locations}.
     W tym momencie wszyscy gracze bd po prostu zaczyna w tej samej lokacji,
     czyli parametr race nie jest uywany.
     
     <p>TODO: By moe pniej zrobimy eby np. elfy zaczynay gr
     w jakim lesie, ludzie w knajpie itp. 
     Dopuszczalne jest te eby ta funkcja robia jakie losowanie,
     np. 60% ludzi zaczyna gr w knajpie a 40% ludzi - w innej knajpie. */
  public Location playerStartLocation(Race playerRace)
  {
    return fPlayersStartLocation;
  }
  
  /** Wywouj to moliwie czsto. Zajmie si realizacj caego AI.
     Zrobi odpowiednie putMessages opisujce kademu graczowi
     zaobserwowane przez niego zmiany w wiecie (np. potwr wszed
     do komnaty w ktrej znajduje si gracz itp.).
     
     <p>Ta metoda nie moe modyfikowa tablicy players() (eby nie narusza
     sensownoci zwracanej wartoci) dlatego po wywoaniu tej metody 
     (i przesaniu/wywietleniu wiadomoci odpowiednim graczom) powiniene
     jak najszybciej wywoa dead() dla wszystkich players(). */
  public void action()
  {
    Iterator i = creatures().iterator();
    while (i.hasNext())
      ((Creature) i.next()).action();
        
    /* remove dead creatures. TODO: optimize this ? Should use iterator.
       But cannot - we use dead(). */
    int j = 0;
    while (j < creatures().size())
    {
      Creature c = (Creature) creatures().get(j);
      if (!c.dead()) j++;
    }
  }
  
  /** Wypisz wielinijkowy (zakoczony newline) spis wszystkich creatures w grze.
     Tylko do celw testowych (czyli dostpne z serwera oraz z klienta przez 
     "debug") */
  public String allCreaturesToString()
  {
    return Utils.listToString(creatures(), "Stworzenia");
  }
  
  /** Wypisz wielinijkowy (zakoczony newline) spis wszystkich players w grze.
     Tylko do celw testowych (czyli dostpne z serwera oraz z klienta przez 
     "debug") */
  public String allPlayersToString()
  {
    return Utils.listToString(players(), "Gracze");
  }  
    
  /** Obvious constructor. 
     
     <p>TODO: for now World just creates some testing locations (and their 
     links). Creating Location objects here
     ensures that each Location will have proper world field.
     In the future we will 1. give here as a parameter a list of something 
     like LocationSkeleton - Location stripped down of some fields
     (like world) or 2. give something like filename (xml?) from
     which to load locations description. */
  public World()
  {
    /* Poniszy kod to demo e mona zrobi otwieranie/zamykanie drzwi poleceniem
       "uzyj" klienta. Polecenie "uzyj" na przedmiocie KeyNoraBiblioteka
       otwiera i zamyka drzwi, czyli aktywuje i deaktywuje przejcia
       linkToBiblioteka i linkToNora. 
       
       TODO: to powinno by zapisywalne prociej. Wyabstrahowa przypadek
       otwierania i zamykania drzwi w dwch lokacjach w jaki oglny sposb. 
       Najlepiej - dodajc atrybut active do linku, to zniweluje te
       potrzeb konstruowania description w tak fikuny sposb. */
    
    LinkToLocation linkToBiblioteka = new LinkToLocation(null, "drzwi", 
      "Przejd do pomieszczenia za drzwiami ktre wyglda jak biblioteka.");
    LinkToLocation linkToNora = new LinkToLocation(null, "drzwi", 
      "Przejd do pomieszczenia za drzwiami ktre wyglda jak nora.");
    
    class NoraOrBiblioteka extends LocationWithDynLink {
    
      public boolean useItem(Being being, Item item)
      {
        if (item instanceof Item.KeyNoraBiblioteka)
        {
          if (dynLinkOpen())
          {
            being.tellEveryoneInNowLocation(new MessageFormatter(
              "%1 zamyka drzwi.", being));
            being.putMessage("Zamykasz drzwi.");
            closeDynLink();
            ((NoraOrBiblioteka) dynLink().target).closeDynLink();
          } else
          { 
            being.tellEveryoneInNowLocation(new MessageFormatter(
              "%1 otwiera drzwi.", being));
            being.putMessage("Otwierasz drzwi.");
            openDynLink();
            ((NoraOrBiblioteka) dynLink().target).openDynLink();
          }
          return true;
        } else
          return super.useItem(being, item);
      }
      
      public NoraOrBiblioteka(World aWorld, String aName, 
        LinkToLocation aDynLink,
        String aConstDescriptionPart,
        String aDescriptionWhenLinkOpen,
        String aDescriptionWhenLinkClosed)
      {
        super(aWorld, aName, aDynLink, aConstDescriptionPart,
          aDescriptionWhenLinkOpen,aDescriptionWhenLinkClosed, false);
      }
    }

    /* lista lokacji */    
    locations = new Location[] {
/* 0 */ new LocationConstDescr(this, "Tyek dinozaura", 
        "Stoisz na rodku tyka dinozaura i zastanawiasz si nad sensem " +
        "swojego ycia. Na olbrzymich poladkach jaki dowcipny turysta " +
        "wyry napis \"Tarzan wuz here\". Nie czujesz si zbyt pewnie. "),
        
/* 1 */ new LocationConstDescr(this, "Jama", "Jeste w jamie."),

/* 2 */ new NoraOrBiblioteka(this, "Nora", linkToBiblioteka,
        "Jeste w norze.",
        "S tu otwarte drzwi.",
        "S tu zamknite drzwi. Na drzwiach wisi tabliczka "+
        "z napisem \"Kilka egzemplarzy klucza do tych drzwi jest w Jamie.\""),

/* 3 */ new LocationConstDescr(this, "Salon", 
        "Znajdujesz si w wykwintnie urzdzonym "+
        "salonie. ciany pokryte s przepiknie zdobionymi drapieriami, "+
        "staromodne meble mieni si zoceniami. Salon owietlony jest "+
        "blaskiem kilkunastu rozstawionych tu i wdzie wiecznikw. "+
        "Na zachd i na wschd znajduj si drzwi, na pnoc znajduje si "+
        "przejcie do bibiloteki."),
        
/* 4 */ new LocationConstDescr(this, "Zachodni taras", "Wychodzisz na taras. "+ 
        "Chodne powietrze nocy orzewia Ci. O balkon oparta jest drabina "+
        "po ktrej moesz zej na d."),
        
/* 5 */ new LocationConstDescr(this, "Wschodni taras", 
        "Wychodzisz na taras. Chodne powietrze nocy orzewia Ci."),
        
/* 6 */ new NoraOrBiblioteka(this, "Biblioteka", linkToNora,
        "Tere-fere, bibiloteki si zachciao. "+
        "Na obrazie na cianie dostrzegasz olbrzymi tyek dinozaura "+
        "i to by byo na tyle jeli chodzi o bibliotek.",
        "S tu otwarte drzwi prowadzce do jakiej nory.",
        "S tu zamknite drzwi."),
        
/* 7 */ new LocationConstDescr(this, "Bagno", 
        "Zapadasz si po kolana w mulistym bocie. "+
        "Nie ma tu adnego owietlenia i widoczno jest ograniczona do "+
        "kilku metrw. Zdecydowanie nie byoby zbyt bezpiecznie oddala "+
        "si od stojcej tu drabiny.",
        new StatModifier(0, -2, 0, 0, 0)),
    };
    
    fPlayersStartLocation = locations[3];
    
    /* set up links between locations */       
    linkToNora      .target = locations[2];
    linkToBiblioteka.target = locations[6];
    linkToNora      .setWayBack(linkToBiblioteka);
    linkToBiblioteka.setWayBack(linkToNora);
    
    locations[0].createTwoSidedLinks(
      locations[1], "wschod", "Na wschd znajduje si jama.",
                    "zachod", "Na zachd znajduje si tyek dinozaura.");
    locations[0].createTwoSidedLinks(
      locations[2], "zachod", "Na zachd znajduje si nora.",
                    "wschod", "Na wschd znajduje si tyek dinozaura.");

    locations[3].createTwoSidedLinks(
      locations[4], "zachod", "Przejd przez drzwi na zachd.",
                    "wschod", "Wr do salonu.");
    locations[3].createTwoSidedLinks(
      locations[5], "wschod", "Przejd przez drzwi na wschd.",
                    "zachod", "Wr do salonu.");
    locations[3].createTwoSidedLinks(
      locations[6], "polnoc", "Przejd na pnoc do biblioteki.",
                    "poludnie", "Przejd na poudnie do salonu.");      
      
    locations[4].createTwoSidedLinks(
      locations[7], "dol", "Zejd po drabinie na d.",
                    "gora", "Wejd po drabinie na gr.");

    locations[6].links().add( new LinkToLocation(
      locations[0], "obraz", "W magiczny sposb przenie swoje szanowne "
        +"jestestwo do wntrza obrazu z tykiem dinozaura."));

    /* add some creatures */
    new Creature.Troll(this, locations[5], null);
    new Creature.GrojecKiller(this, locations[7], null);
    new Creature.Goblin(this, locations[4], null);
    new Creature.DungeonExplorer(this, locations[0]);
    
    /* add some items */
    for (int i=0; i<5; i++)
      locations[1].items().add(new Item.KeyNoraBiblioteka());
      
    locations[3].items().add(new Item.ElfKillerWhistle());
    locations[3].items().add(new Item.ElfFairyWhistle());
    ItemsBag shinyBox = new ItemsBag(0.5f, "szkatuka", "misternie zdobiona", null);
    locations[3].items().add(shinyBox);
    shinyBox.items().add(new Ring("elazny", "Piercie guptakw", 
      new StatModifier(2, 2, 0, 2, -6)));
    shinyBox.items().add(new Ring("zoty", "Piercie krli i innych takich", 
      new StatModifier(0, 0, 0, 0, -6)));
    shinyBox.items().add(new Ring("wysadzany diamentami", "Piercie ycia", 
      new StatModifier(0, 0, 0, 3, 0)));
  }      
}

/*
 * $Log: World.java,v $
 * Revision 1.2  2004/02/19 15:30:28  michalis
 * * Creature.attackPlayers/Creatures -> SimpleMindedCreature
 * + NonHostileCreature
 * + ElfFairy, ElfFairyWhistle
 * + wanderingFactor, teraz prawie wszystkie creatures wasaj si po wiecie
 *
 * Revision 1.1  2004/02/18 14:37:35  michalis
 * * pl stringi
 * * --help output changed (to reflect that those classes will be run by bin/scripts)
 * preparations for ClientApplet:
 * * Client rozdzielony na Client i ClientStandalone
 * * znaczna cz pakietu common przeniesiona do nowego pakietu game
 * X* make jamy_i_nory_client_applet.jar
 *
 * Revision 1.36  2004/02/15 20:11:45  michalis
 * * lepszy interfejs w Swingu (JTextPane ze stylami, echo komendy bold'em,
 *   pole do dalszych upiksze)
 * * bugfix do "powiedz" w UserCommand (dla przypadku command.length == 0)
 * * linki do nory i biblioteki poczone przez wayBack
 *
 * Revision 1.35  2004/02/15 15:24:35  michalis
 * + pdf generating
 * + COPYING file
 * + Utils.check
 * + Location.add/removeLink
 * + LocationWithDynLink
 * * NoraOrBiblioteka.useItem simplified
 * * teraz i w bibliotece i w norze s drzwi
 * - Makefiles rozsiane po katalogach.
 * * Globalny Makefile rzdzi.
 *
 * Revision 1.34  2004/02/14 11:01:51  michalis
 * * demo otwierania i zamykania drzwi kluczem zrobione
 *
 * Revision 1.33  2004/02/14 01:09:44  michalis
 * + przejcia w drug stron (LinkToLocation.wayBack, Location.createTwoSidedLinks)
 * * followLink -> Being
 * * lepsze komunkaty o przejciach w setNowLocation
 * * Troll osabiony
 *
 * Revision 1.32  2004/02/09 23:08:53  michalis
 * + GPL license clause i copyrighty (moje i gdzieniegdzie Michaa, sorry Rafa
 *   ale naprawd nic nie zostao) do wszystkich *.java
 *
 * Revision 1.31  2004/02/09 19:35:20  michalis
 * * Uaktualnienia dokumentacji
 *
 * Revision 1.30  2004/02/07 21:37:19  michalis
 * * malekie ulepszenia
 * * pusty plecak Beinga ktry ma name==null nie jest upuszczany na ziemi (bo po co?)
 *
 * Revision 1.29  2004/02/07 20:05:15  michalis
 * + World.beingsFindById
 * + ServerCommand.parseBeingId
 * * ServerCommand polecenia "opisz" i "zabij" zapisane adniej, poprawione pl ogonki
 * * implementacja Zabjcy z Grjca uproszczona
 * + do dokumentacji o poleceniach serwera "opisz" i "zabij"
 *
 * Revision 1.28  2004/02/07 17:34:58  michalis
 * * jakies strzepy
 *
 * Revision 1.27  2004/02/06 04:44:40  michalis
 * + InactivePlayer
 * + dodatkowy konstruktor do Being
 * + dodatkowy konstruktor Player(World, InactivePlayer)
 * * beforeDeath rozszerzone o parametr goingInactive
 * + Being.goInactive
 * * listToString moved from World to Utils
 * + ServerCommand : polecenie "mrozonki"
 *
 * Revision 1.26  2004/02/06 01:24:33  michalis
 * * implementacja UserCommandBlackTokens.matches przeniesiona do Utils.stringArraysNullEqual
 * + ServerCommand
 * * Server.ioCheckInput uproszczone dziki temu
 * * komendy serwera na polski, konsekwentnie
 * + ServerCommand "zabij ID"
 *
 * Revision 1.25  2004/02/05 23:27:39  michalis
 * * jeszcze mae poprawki w kwestii kiedy pisa TO-DO a kiedy to samo bez mylnika
 * * IOSwing konstruktor: parametr windowTitle
 * + Server obsuguje parametr --swing, ma pole IOSystem
 * + Utils.collectionToMultiLineString(Collection,String)
 * * Server mona ju kontrolowa interfejsem IOSystem, polecenia "x", "h", "p", "c"
 * + World.allCreatures/PlayersToString,
 * * UserCommand. "debug players/creatures" uywaj World.allCreatures/PlayersToString
 * * Poprawka w Server.ServerListener.run przy sleep
 *
 * Revision 1.24  2004/02/05 20:05:26  michalis
 * * Zrobiem eby w logach CVSa zapisanych w plikach byo wszdzie TO-DO
 *   z mylnikiem w rodku. Wprawdzie nieznacznie rozsynchronizowuje to
 *   zapisane w plikach logi z wynikiem 'cvs log' ale za to grep TO-DO
 *   (gdzie w wyrazie TO-DO naley usun mylnik) dziaa jak trzeba.
 *
 * Revision 1.23  2004/02/04 23:54:57  michalis
 * + dodany przedmiocik "szkatuka" do lokacji "Salon" z 3 piercieniami
 * * Being.beforeDeath - maa poprawka eby upuszcza plecak przez dropOnFloor
 *   (zeby byy komunikaty Player.putMessage)
 * * some Being interface clarifications
 * + dodane wywoanie putMessage(nowLocationToSingle()) do konstruktora Player,
 *   to pozwala nieco uproci PlayerConnection i JamySingle
 *
 * Revision 1.22  2004/02/04 14:00:25  michalis
 * * Being.tellEveryoneInNowLocation -> sayEveryoneInNowLocation
 *   (mniej mylca nazwa)
 * * Server powiadamia wszystkich graczy w komnacie e pojawi si nowy gracz
 * * PlayersMessages.tellEveryoneInLocation - docs updated,
 *   implemented in a cleaner way
 * + MessageGenerator
 * + MessageFormatter
 * * Being.attitude changed to realAttitude and observedAttitude
 * * Many fixes to use MessageFormatter when updating PlayersMessages.
 *   Many TO-DOs closed.
 * * Attitude.toStr translated to polish
 * * Poprawiony bug w Being.dead - Micha, nie mona tworzy PlayersMessages
 *   tu przed usuniciem gracza z gry - w czasie uywania PlayersMessages
 *   world.players() nie mog by modyfikowane. Z drugiej strony, to jest moja
 *   wina e jeszcze tego nie mona zrobi (bo to *powinno* by moliwe tylko
 *   musz zapisa inteligentniej PlayersMessages). OK, wanie mi przyszo do
 *   gowy jak to zrobi bardzo atwo i bardzo elegancko (klasa PlayersMessages
 *   w ogle nie bdzie potrzebna i unikniemy przerzucania wszdzie parametrw
 *   playersMessages) i w nastpnym commicie to bdzie zrobione (gupi, gupi,
 *   gupi jestem e od pocztku tego tak nie zrobiem).
 * * OK, serwer skunk.one.pl nie odpowiada wic nie bd commitowa, od razu
 *   zrobi to o czym mwiem powyej:
 *   + Player.getMessage, Player.putMessage
 *   + Location.tellEveryone (3 wersje)
 *   - PlayersMessages (funkcjonalno przerzucona w caoci do obiektw Player i
 *     Location)
 *   * masa uproszcze kodu (m.in. nie musimy ju przerzuca wszdzie
 *     parametrw PlayersMessages)
 *   - klasa Being.DeadAnswer (useless, teraz Being.dead() znowu zwraca prostego
 *     booleana)
 *   * w ogle Being.dead() jest teraz o niebo prostsze.
 *   Skoczyem! PlayersMessages usunite razem z wieloma niewygodami.
 *
 * Revision 1.21  2004/02/04 09:09:23  michalis
 * + contract of StatModifier.toString specified
 * + Location.statModifier(), odpowiednio rozszerzone Location.toString
 * * Being.fullStats odpowiednio rozszerzone
 * + lokacja Bagno (eby przetestowa Location.statModifier()),
 * * Goblin przeniesiony z tyka dinozura na zachodni balkon,
 *   Troll przeniesiony z tyka dinozura na wschodni balkon
 *   (eby ZPL nie pokazywa lokacji o nazwie "tyek dinozaura")
 *
 * Revision 1.20  2004/02/02 02:16:12  michalis
 * + PlayersMessages(Player,String) (unused for now)
 * + Item.useItem
 * + Item.ElfKillerWhistle implementujcy useItem
 * * Location.useItem wywouje Item.useItem
 * + Egzemplarz Item.ElfKillerWhistle dodany do salonu
 * * some docs clarified in Being
 *
 * Revision 1.19  2004/02/02 01:40:37  michalis
 * + wicej lokacji i przej testowych
 *
 * Revision 1.18  2004/02/01 20:57:59  michalis
 * + UserCommandBlackTokens, wszystkie dotychczasowe komendy zmienione jako
 *   dziedziczce z UserCommandBlackTokens zamiast bezporednio z UserCommand
 * + Being.tellEveryoneInLocation
 * + UserCommand: komenda "powiedz"
 * + Goblin mwi e jest godny (naprawd)
 *
 * Revision 1.17  2004/02/01 00:08:54  michalis
 * + Being.DeadAnswer, dead() returns DeadAnswer
 * - Player/Creature.dead
 * * Player/Creature.setNowLocation: remove object from world.players/creatures list
 * * World.action implemented cleaner
 * * Creature.attack dostaje parametr PlayersMessages
 * + Creature.ElfKiller, dodany do World do lokacji "Jama"
 * + PlayersMessages.append
 * + PlayerConnection.server, + Connection.createPlayer "server" parameter
 * + gdy gracz umiera to jest komunikat dla System.out serwera, komunikat "Umierasz"
 *   dla umierajcego gracza i komunikat "kto-tam umiera" dla pozostaych graczy
 *   w tej lokacji
 * * another bugfix for checking dead state of players in server.
 *   Wszystko przez to e na razie mam doc niewygodne metody testowania tego kodu
 *   skoro gracze nie mog si rusza w wiecie. No, ale myl e obecnie
 *   wszystko ju jest jak trzeba - wszystkie elementy kodu zostay
 *   przetestowane dziki ElfKillerowi.
 *
 * Revision 1.16  2004/01/31 20:10:29  michalis
 * + IOSystem: some TO-DO with comments moved from UserCommand to IOSystem
 * * UserCommand docs improved
 * * UserCommand.s renamed to commandPattern
 * + PlayersMessages
 * * World.messageToPlayersInLocation moved to PlayersMessages.tellEveryoneInLocation
 * * World.action changed to use PlayersMessages (much cleaner code now!)
 * * Creature.action and doAction use PlayersMessages
 * * UserCommand.execute interface changed:
 *   - world (excessive, one can always use player.world)
 *   + playersMessages (to tell other players about effects of this command)
 *   + returns String (tell command invoker about effects of this command)
 * * every command's execute changed to use new interface:
 *   nothing uses JamySingle.io, nothing may call Player.dead()
 *
 * Revision 1.15  2004/01/31 06:02:09  michalis
 * + Server.world
 * + SimpleClient.newPlayer
 * * Idea of Connection changed : now it does not inherit from StringSocket.
 *   It just holds stringSocket inside, and can create PlayerConnection.
 * + PlayerConnection : inherits from Player, contains StringSocket.
 *   (This is what was eariler meant to be in Connection)
 * * Server.users split to connections and world.players()
 * + Server adds ready connections to world.players() using Connection.createPlayer
 * - Server.findById (it was just a TO-DO) removed,
 *   now it can be implemented using IdedList.byId
 * + important comments at world.action
 * * StringSocket.read bug (when receiving two messages *fast*) fixed
 * + Utils.InternalError(String, Throwable)
 * * some Server docs improved
 *
 * Revision 1.14  2004/01/19 02:27:58  michalis
 * * Fixed messageForPlayersInLocation (added that Utils.nl)
 *
 * Revision 1.13  2004/01/19 01:50:12  michalis
 * + world.creatures()
 * * Location.creatures() is now only calculated
 * * world.players are IdedList (not LinkedList) now
 * * messageForPlayersInLocation interface changed (don't add Utils.nl at the end now)
 * * world.action changed to use world.creatures(), no longer any need
 *   to iterate over locations[]
 * + testing World will contain creature DungeonExplorer (to see that creatures
 *   can move in our world)
 *
 * Revision 1.12  2004/01/19 00:19:00  michalis
 * + Goblin that attacks Troll (to test Creature vs Creature fighting)
 *
 * Revision 1.11  2004/01/19 00:04:01  michalis
 * + action and doAction names exchanged
 *
 * Revision 1.10  2004/01/18 21:20:36  michalis
 * * Troll obcina sobie palec (i w kocu umiera... bardzo mdry troll...)
 * + World.messageForPlayersInLocation,
 * + World.doAction sprawdza creatures().dead w kadej lokacji
 *
 * Revision 1.9  2004/01/16 20:54:27  michalis
 * + Troll class, Troll tests in BaseTests, initial World contains one Troll
 *   instance. Finally we can see first Creature AI in action !
 *
 * Revision 1.8  2004/01/16 20:06:16  michalis
 * + World.doAction
 *
 * Revision 1.7  2004/01/14 22:24:24  michalis
 * + docs update
 *
 * Revision 1.6  2004/01/11 23:51:54  michalis
 * * Location.world added and necessary changes made, some Player docs improved
 *
 * Revision 1.5  2004/01/11 09:53:21  michalis
 * + Small docs update
 *
 * Revision 1.4  2004/01/11 09:34:26  michalis
 * + Class LinkToLocation, Location.links and necessary tests and docs
 *
 * Revision 1.3  2004/01/11 07:48:52  michalis
 * + CVS Log added
 *
 * Revision 1.2  2004/01/11 07:40:21  michalis
 * * Poprawki i ulepszenia dokumentacji
 *
 * Revision 1.1  2004/01/11 06:51:55  michalis
 * + World class, Being.nowLocation
 *
 */
