/*
  Copyright 2004 Michalis Kamburelis.

  This file is part of "Jamy & Nory".

  "Jamy & Nory" is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  "Jamy & Nory" is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with "Jamy & Nory"; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package jamy_i_nory.server;

import java.nio.channels.*;
import java.io.*;
import jamy_i_nory.common.*;
import jamy_i_nory.game.*;

/** Klasa reprezentujca poczenie z klientem ktry jeszcze nie 
   zainicjowa odpowiadajcego mu w wiecie gracza. */
class Connection {   

  final private StringSocket sSocket;

  /** Podaj aSockChannel na uytek tego obiektu i pniejszego obiektu
     PlayerConnection. Wymagania nakadane na przekazany tu aSockChannel
     oraz uwagi o rzucanych przez ten konstruktor wyjtkach - patrz
     {@link jamy_i_nory.common.StringSocket#StringSocket}. */
  Connection(SocketChannel aSockChannel) 
  throws IOException
  {
    sSocket = new StringSocket(aSockChannel);
  }
  
  /* Informacje ktre ta klasa musi zgromadzi, potrzebne do wywoania
     konstruktora Player. */
  private String playerName;
  /* raceNumber to numer rasy na Race.playerRaces albo -1 (ktre oznacza
     e gracza naley zainicjowa z inactivePlayers) */
  private Integer raceNumber;
  private String password;
  
  /** Wywoaj to aby sprawdzi czy mona wywoa metod {@link #createPlayer}. 
     Wywoanie tej metody moe pocign za sob prb odczytu danych z gniazda
     czcego z klientem. Jeli odczytano od klienta wszystkie informacje
     i w zwizku z tym metoda createPlayer moe dziaa to zwraca true. */
  public boolean canCreatePlayer()
  {
    if (playerName == null) 
    {
      String s;
      if ((s = sSocket.read()) != null) 
      { 
        playerName = s;
        return canCreatePlayer();
      } else 
        return false;
    } else
    if (raceNumber == null)
    {
      String s;
      if ((s = sSocket.read()) != null) 
      {
        try {
          raceNumber = Integer.valueOf(s);
          return canCreatePlayer();
        } catch (NumberFormatException e) {
          System.err.println("Communication error with client " +sSocket 
            +": invalid race number: " +e);
          return false;
        }
      } else
        return false;        
    } else
    if (password == null) 
    {
      String s;
      if ((s = sSocket.read()) != null) 
      { 
        password = s;
        return canCreatePlayer();
      } else 
        return false;
    } 
      /* Tutaj mona dopisywa dalsze waciwoci ktre trzeba bdzie
         przekaza przy konstruowaniu Player. */    
      else
      return true;
  }
  
  static public class CannotActivatePlayerException extends Exception {
    CannotActivatePlayerException(String message) { super(message); }
  };
  
  /** Podaj message (bez kocowej kropki ani newline'a).
     Wyle odpowiedni message do sSocket, zamknie sSocket, rzuci
     new CannotActivatePlayerException(message).
     
     Zwraca PlayerConnection tylko dla picu (eby mona go byo wygodnie
     uy z createPlayer, eby Java nie mwia "nie uye return").
     Zawsze wychodzi z wyjtkiem. */
  private PlayerConnection failCreatePlayer(String message)
  throws CannotActivatePlayerException
  {
    sSocket.write(message +"." +Utils.nl);
    sSocket.close();
    throw new CannotActivatePlayerException(message);
  }
  
  /** Tworzy obiekt klasy {@link PlayerConnection} ktry odtd bdzie
     reprezentowa to poczenie z klientem. 
     Wywouj tylko jeli {@link #canCreatePlayer} = true. 
     Po wywoaniu tej metody nie moesz ju uywa tego obiektu -
     od tego momentu poczenie z klientem powinno ju by uywane tylko
     przez uzyskany obiekt klasy {@link PlayerConnection}. 
     
     <p>Jeeli klient chce gra nowym graczem to ta metoda doda nowego
     gracza do gry, tzn. uyje konstruktora {@link 
     PlayerConnection#PlayerConnection(World, Race, String, StringSocket, 
     Server, String)}.
     
     <p>Jeeli klient zeche gra dalej graczem tymczasowo zamrozonym
     (tzn. na licie server.inactivePlayers) to uyje konstruktora 
     {@link PlayerConnection#PlayerConnection(World, InactivePlayerConnection, 
     StringSocket, Server)} wycigajc (tzn. znajdujc i usuwajc) 
     odpowiedni obiekt z listy server.inactivePlayers. 
     Jeeli klient zada gry ju istniejcym graczem ale nie poda imienia
     gracza ktry jest ju zamroony (tzn. poda ze imi gracza, ktrego
     nie znajdziemy wrd server.inactivePlayers) to zostanie zgoszony
     wyjtek CannotActivatePlayerException, do klienta zostanie wysany
     odpowiedni komunikat bdu i poczenie zostanie zerwane.
     Podobnie jeeli klient poda ze haso do gracza. 
     
     <p>W przypadku wyjcia z tego konstruktora z
     wyjtkiem CannotActivatePlayerException lista server.inactivePlayers
     (ani cokolwiek innego, np. stan wiata gry) nie zostanie zmodyfikowana.
     Natomiast zasada "nie uywaj tego obiektu (klasy Connection) po
     wywoaniu tej metody" zostaje zachowana. */
  public PlayerConnection createPlayer(World world, Server server) 
  throws CannotActivatePlayerException
  {
    assert canCreatePlayer();
    if (raceNumber.intValue() == -1) 
    {
      InactivePlayerConnection inactivePlayer = (InactivePlayerConnection)
        server.inactivePlayers.findByName(playerName);
      if (inactivePlayer == null)
        return failCreatePlayer("Nie istnieje zamroony"+ 
          " (tzn. chwilowo nieaktywny) gracz o imieniu \"" + playerName +"\"");
      else
      if (!inactivePlayer.passwd.equals(password))
        return failCreatePlayer("Niestety, podae ze haso");
      else
      {
        server.inactivePlayers.remove(inactivePlayer);
        return new PlayerConnection(world, inactivePlayer, sSocket, server);
      }
    } else 
    {
      return new PlayerConnection(world, Race.playerRaces[raceNumber.intValue()], 
        playerName, sSocket, server, password);
    }
  }
  
  /** Zwraca toString opakowywanego obiektu StringSocket. */
  public String toString()
  {
    return sSocket.toString();
  }
}

/*
 * $Log: Connection.java,v $
 * Revision 1.13  2004/02/19 07:32:56  michalis
 * * small docs corrections
 * * sekcja "Przed uruchomieniem" skoczona
 * * some improvements to jar generating (one step closer to making client Applet)
 *
 * Revision 1.12  2004/02/18 14:37:37  michalis
 * * pl stringi
 * * --help output changed (to reflect that those classes will be run by bin/scripts)
 * preparations for ClientApplet:
 * * Client rozdzielony na Client i ClientStandalone
 * * znaczna cz pakietu common przeniesiona do nowego pakietu game
 * X* make jamy_i_nory_client_applet.jar
 *
 * Revision 1.11  2004/02/10 00:39:42  michalis
 * * po kilku bezproduktywnych commitach (jakie reorganizacje ...) co konkretnego:
 *   Zamroony na serwerze gracz jest chroniony hasem.
 *
 * Revision 1.10  2004/02/09 23:08:57  michalis
 * + GPL license clause i copyrighty (moje i gdzieniegdzie Michaa, sorry Rafa
 *   ale naprawd nic nie zostao) do wszystkich *.java
 *
 * Revision 1.9  2004/02/06 06:21:04  michalis
 * + PlayerConnection: dodatkowy konstruktor
 * + InactivePlayersList
 * * rozszerzona funkcjonalnoc Connection (musi umie wywoa oba konstruktory
 *   PlayerConnection)
 *   Dziaa ! Zamraanie i odmraanie graczy zrobione !
 *
 * Revision 1.8  2004/02/01 00:08:56  michalis
 * + Being.DeadAnswer, dead() returns DeadAnswer
 * - Player/Creature.dead
 * * Player/Creature.setNowLocation: remove object from world.players/creatures list
 * * World.action implemented cleaner
 * * Creature.attack dostaje parametr PlayersMessages
 * + Creature.ElfKiller, dodany do World do lokacji "Jama"
 * + PlayersMessages.append
 * + PlayerConnection.server, + Connection.createPlayer "server" parameter
 * + gdy gracz umiera to jest komunikat dla System.out serwera, komunikat "Umierasz"
 *   dla umierajcego gracza i komunikat "kto-tam umiera" dla pozostaych graczy
 *   w tej lokacji
 * * another bugfix for checking dead state of players in server.
 *   Wszystko przez to e na razie mam doc niewygodne metody testowania tego kodu
 *   skoro gracze nie mog si rusza w wiecie. No, ale myl e obecnie
 *   wszystko ju jest jak trzeba - wszystkie elementy kodu zostay
 *   przetestowane dziki ElfKillerowi.
 *
 * Revision 1.7  2004/01/31 06:02:10  michalis
 * + Server.world
 * + SimpleClient.newPlayer
 * * Idea of Connection changed : now it does not inherit from StringSocket.
 *   It just holds stringSocket inside, and can create PlayerConnection.
 * + PlayerConnection : inherits from Player, contains StringSocket.
 *   (This is what was eariler meant to be in Connection)
 * * Server.users split to connections and world.players()
 * + Server adds ready connections to world.players() using Connection.createPlayer
 * - Server.findById (it was just a TO-DO) removed,
 *   now it can be implemented using IdedList.byId
 * + important comments at world.action
 * * StringSocket.read bug (when receiving two messages *fast*) fixed
 * + Utils.InternalError(String, Throwable)
 * * some Server docs improved
 *
 * Revision 1.6  2004/01/29 19:55:47  michalis
 * + code from server/Connection moved to common/StringSocket
 *   (with some minor improvements and clarification of interface)
 * - Connection class is empty for now - it only inherits everything from
 *   StringSocket, nothing more
 * * SimpleServer.sendMessage(Connection,String) implemented (using new
 *   Connection/StringSocket interface), so all sendMessage methods uncommented again;
 *   and they don't throw IOException anymore - this is already handled
 *   in StringSocket.write()
 * * SimpleServer.closeServerSession: some minor improvemnets
 * - SimpleServer comment : @author Rafal Wawrzkowicz removed - myl e
 *   zrobiem wystarczajco duo w tej klasie eby mie co do powiedzenia.
 *   Nie twrzmy szczegowych list autorw do kadej klasy - w logach kady
 *   widzi co kto zrobi; a oryginalny autor pliku z klas nie zawsze jest osob
 *   ktra doprowadzia klas do stanu wzgldnej uywalnoci...
 *
 * Revision 1.5  2004/01/29 14:13:18  michalis
 * * SimpleServer and Connection changed to use java.nio - ServerSocketChannel and
 *   SocketChannel instead ServerSocket and Socket. This allows to use non-blocking
 *   checking for end of stream, so Connection.isConnected works now.
 * * sendMessage temporarily broken
 *
 * Revision 1.4  2004/01/29 12:15:40  michalis
 * + Connection.isConnected, getMessage
 * * Connection.closeSocket does not throw IOException anymore
 *
 * Revision 1.3  2004/01/29 04:00:55  michalis
 * + Connection.toString
 * + SimpleClient wysya bajty (ze Stringa)
 * + SimpleServer odbiera bajty i wypisuje na konsol (jako String) - co i od kogo dosta
 * * Now SimpleServer can be stopped only by C-c (i.e., killing the process).
 *   Functionality to close server "gently" will be restored soon.
 *
 * Revision 1.2  2004/01/26 20:09:55  michalis
 * * line endings CRLF -> LF
 * * main Makefile modified to clean server/ dir
 * + basic Makefile in server/
 * * class SimpleServerProtocol renamed to SimpleGameProtocol to match filename
 *   (Rafa, przecie to si w ogle nie kompilowao)
 * + required imports to Connection
 *   (Rafa, przecie to si w ogle nie kompilowao)
 * + package jamy_i_nory.server; na pocztku Message.java
 *   (Rafa, przecie to si w ogle nie kompilowao)
 * * SimpleServer.closeServerSession() fixed (downcast to Connection needed)
 *   (Rafa, przecie to si w ogle nie kompilowao)
 *
 * Revision 1.1  2004/01/26 19:41:03  michalis
 * Klasy serwera Rafaa:
 * * TABy zamienione na spacje (Rafa, do k.n., nie uywaj TABw)
 * * Windows-1250 zamieniony na ISO-8859-2 (Rafa, do k.n., nie uywaj
 *   Windows-1250; uywaj ogonkw w ISO albo wcale)
 * * "Server" -> "server" (all files removed from Server/ and added again to server/)
 * + Logi CVSa dodane
 *
 */
