/*
  Copyright 2004 Michalis Kamburelis.

  This file is part of "Jamy & Nory".

  "Jamy & Nory" is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  "Jamy & Nory" is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with "Jamy & Nory"; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package jamy_i_nory.server;

import java.nio.channels.*;
import java.io.*;
import jamy_i_nory.common.*;
import jamy_i_nory.game.*;

/** Reprezentuje poczenie z klientem ktry ju ma przypisanego gracza.
   Dziedziczy z klasy Player co pozwala m.in. wrzuca obiekty tej klasy 
   na list World.players(). */
class PlayerConnection extends Player {

  /** Poczenie z danym klientem. Nie ma ogranicze co do tego jak moesz
     je wykorzysta z zewntrz (dlatego te to pole jest publiczne). */
  final public StringSocket sSocket;
  
  final public Server server;
  
  private String fPasswd;
  public String passwd() { return fPasswd; }
  /* unused for now; TODO - we should allow user to change it's password,
     then (probably?) we will use method below 
  public void setPasswd(String value) { fPasswd = value; }
  */
  
  /** Gromadzi wsplny kod wykonywany na kocu dwch konstruktorw
     PlayerConnection. */
  private void postConstructor(boolean madeFromInactive)
  {
    /* Ustawianie sSocket i server nie jest robione tutaj bo wtedy Java
       nie zauwazy e ta metoda jest wywoywana tylko z konstruktora
       i powinna mc ustawia pola final. Ale to nic strasznego,
       Java ostrzee nas jesli zapomnimy w konstruktorach zainicjowa
       tych pl. */
    server.io.println((madeFromInactive ? 
      "Gracz " +this +" przyczony do gry (odmroony)" :
      "Nowy gracz " +this)
      +" z adresu " +sSocket);
  }

  /** Tworzy polaczenie z graczem. Wypisuje te na server.io
     (a wic dla admina serwera) o tym e mamy nowego gracza w wiecie. */
  public PlayerConnection(World aWorld, Race aRace, String aName, 
    StringSocket aSSocket, Server aServer, String aPasswd)
  {
    super(aWorld, aRace, aName);
    sSocket = aSSocket;
    server = aServer;
    fPasswd = aPasswd;
    postConstructor(false);
  }
  
  /** Tworzy polaczenie z graczem odmraajc gracza zapamitanego w
     InactivePlayer. Uwagi podobne jak przy {@link Player#Player(World,
     InactivePlayer)} - po zrobieniu czego takiego musisz przesta
     odwoywa si skdkolwiek w programie do przekazanego tu
     obiektu inactivePlayer. Wypisuje te na server.io
     (a wic dla admina serwera) o tym e mamy nowego gracza w wiecie. */
  public PlayerConnection(World aWorld, InactivePlayerConnection inactivePlayer, 
    StringSocket aSSocket, Server aServer)
  {
    super(aWorld, inactivePlayer);
    sSocket = aSSocket;
    server = aServer;
    fPasswd = inactivePlayer.passwd;
    postConstructor(true);
  }
  
  /** Robi getMessage() i (jeeli dosta co niepustego) wysya do klienta 
     przez gniazdo. Innymi sowy, zapewne nigdy nie bdziesz mia potrzeby
     robi getMessage na obiekcie klasy PlayerConnection. Po prostu co jaki
     czas musisz wywoywa pushMessage(). */
  public void pushMessage()
  {
    String s = getMessage();
    if (s.length() != 0) sSocket.write(s);
  }
  
  /** Po pierwsze, patrz specyfikacja tej motody w nadklasie. 
     Tutaj powiem tylko e poniewa ta klasa jest czci serwera 
     (a nie klienta albo np. JamySingle) wic pozwala 
     sobie na wykonanie czci roboty specyficznej dla serwera naszego MUDa :
     <ol>
       <li>wypisuje komunikat na server.io, a wic komunikat dla 
         admina serwera gry, co w rodzaju "Gracz taki-a-taki umar"/
         "odczy si"
       <li>Robi putMessage("Umierasz"/"Odczasz si.") + pushMessage(). 
         W ten sposb wysya do gracza komunikat "Umierasz.""Odczasz si." 
         i wszystkie wczeniejsze jeszcze nie wysane komunikaty.
       <li>finalizuje poczenie z klientem, tzn. zamyka sSocket
     </ol> */
  protected void beforeDeath(boolean goingInactive)
  {
    server.io.println("Gracz " +this +" " 
      +(goingInactive ? "odcza si" : "umiera") +".");
    
    putMessage(goingInactive ? "Odczasz si od gry." : "Umierasz.");
    pushMessage();
    sSocket.close();
    super.beforeDeath(goingInactive);
  }  
}

/*
 * $Log: PlayerConnection.java,v $
 * Revision 1.14  2004/02/18 14:37:37  michalis
 * * pl stringi
 * * --help output changed (to reflect that those classes will be run by bin/scripts)
 * preparations for ClientApplet:
 * * Client rozdzielony na Client i ClientStandalone
 * * znaczna cz pakietu common przeniesiona do nowego pakietu game
 * X* make jamy_i_nory_client_applet.jar
 *
 * Revision 1.13  2004/02/10 00:39:42  michalis
 * * po kilku bezproduktywnych commitach (jakie reorganizacje ...) co konkretnego:
 *   Zamroony na serwerze gracz jest chroniony hasem.
 *
 * Revision 1.12  2004/02/09 23:08:57  michalis
 * + GPL license clause i copyrighty (moje i gdzieniegdzie Michaa, sorry Rafa
 *   ale naprawd nic nie zostao) do wszystkich *.java
 *
 * Revision 1.11  2004/02/06 06:21:04  michalis
 * + PlayerConnection: dodatkowy konstruktor
 * + InactivePlayersList
 * * rozszerzona funkcjonalnoc Connection (musi umie wywoa oba konstruktory
 *   PlayerConnection)
 *   Dziaa ! Zamraanie i odmraanie graczy zrobione !
 *
 * Revision 1.10  2004/02/06 04:44:42  michalis
 * + InactivePlayer
 * + dodatkowy konstruktor do Being
 * + dodatkowy konstruktor Player(World, InactivePlayer)
 * * beforeDeath rozszerzone o parametr goingInactive
 * + Being.goInactive
 * * listToString moved from World to Utils
 * + ServerCommand : polecenie "mrozonki"
 *
 * Revision 1.9  2004/02/05 23:27:44  michalis
 * * jeszcze mae poprawki w kwestii kiedy pisa TO-DO a kiedy to samo bez mylnika
 * * IOSwing konstruktor: parametr windowTitle
 * + Server obsuguje parametr --swing, ma pole IOSystem
 * + Utils.collectionToMultiLineString(Collection,String)
 * * Server mona ju kontrolowa interfejsem IOSystem, polecenia "x", "h", "p", "c"
 * + World.allCreatures/PlayersToString,
 * * UserCommand. "debug players/creatures" uywaj World.allCreatures/PlayersToString
 * * Poprawka w Server.ServerListener.run przy sleep
 *
 * Revision 1.8  2004/02/04 23:55:01  michalis
 * + dodany przedmiocik "szkatuka" do lokacji "Salon" z 3 piercieniami
 * * Being.beforeDeath - maa poprawka eby upuszcza plecak przez dropOnFloor
 *   (zeby byy komunikaty Player.putMessage)
 * * some Being interface clarifications
 * + dodane wywoanie putMessage(nowLocationToSingle()) do konstruktora Player,
 *   to pozwala nieco uproci PlayerConnection i JamySingle
 *
 * Revision 1.7  2004/02/04 18:56:45  michalis
 * + some improvements in Client (waitForMessageAndPrint, sSocketRead, sScoketWrite)
 * + Player.followLink
 * * UserCommand.execute does not return String (it returns void).
 *   It should simply append something to player.getMessage().
 *   Wiele uproszcze zapisu w UserCommand - np. komenda "idz" jest teraz
 *   krciutka i wszystkim zajmuje si Player.followLink i Player.setNowLocation.
 * + PlayerConnection.pushMessage, * PlayerConnection.dead slightly improved
 * - Server.sendMessage, - Server.broadcastMessage (useless - we use only
 *   sendPlayersMessages and PlayerConnection.pushMessage)
 *
 * Revision 1.6  2004/02/04 15:36:44  michalis
 * + Dziki pozbyciu si PlayersMessages moemy teraz wypisywa stringi
 *   o tym kto wchodzi/wychodzi z pomieszczenia w setNowLocation.
 * + Being.beforeDead, Being.toStringBase eby setNowLocation
 *   mogo take wywietla kto si pojawia (jako nowy gracz/potwr w wiecie)
 *   uywane z konstruktora Being i z dead()
 * * Server zosta uproszczony w wyniku powyszej zmiany, Being.dead te
 *   (chocia pojawia si niewielka strata funkcjonalnoci - patrz TO-DO przy
 *   toStringBase())
 *
 * Revision 1.5  2004/02/04 14:00:28  michalis
 * * Being.tellEveryoneInNowLocation -> sayEveryoneInNowLocation
 *   (mniej mylca nazwa)
 * * Server powiadamia wszystkich graczy w komnacie e pojawi si nowy gracz
 * * PlayersMessages.tellEveryoneInLocation - docs updated,
 *   implemented in a cleaner way
 * + MessageGenerator
 * + MessageFormatter
 * * Being.attitude changed to realAttitude and observedAttitude
 * * Many fixes to use MessageFormatter when updating PlayersMessages.
 *   Many TO-DOs closed.
 * * Attitude.toStr translated to polish
 * * Poprawiony bug w Being.dead - Micha, nie mona tworzy PlayersMessages
 *   tu przed usuniciem gracza z gry - w czasie uywania PlayersMessages
 *   world.players() nie mog by modyfikowane. Z drugiej strony, to jest moja
 *   wina e jeszcze tego nie mona zrobi (bo to *powinno* by moliwe tylko
 *   musz zapisa inteligentniej PlayersMessages). OK, wanie mi przyszo do
 *   gowy jak to zrobi bardzo atwo i bardzo elegancko (klasa PlayersMessages
 *   w ogle nie bdzie potrzebna i unikniemy przerzucania wszdzie parametrw
 *   playersMessages) i w nastpnym commicie to bdzie zrobione (gupi, gupi,
 *   gupi jestem e od pocztku tego tak nie zrobiem).
 * * OK, serwer skunk.one.pl nie odpowiada wic nie bd commitowa, od razu
 *   zrobi to o czym mwiem powyej:
 *   + Player.getMessage, Player.putMessage
 *   + Location.tellEveryone (3 wersje)
 *   - PlayersMessages (funkcjonalno przerzucona w caoci do obiektw Player i
 *     Location)
 *   * masa uproszcze kodu (m.in. nie musimy ju przerzuca wszdzie
 *     parametrw PlayersMessages)
 *   - klasa Being.DeadAnswer (useless, teraz Being.dead() znowu zwraca prostego
 *     booleana)
 *   * w ogle Being.dead() jest teraz o niebo prostsze.
 *   Skoczyem! PlayersMessages usunite razem z wieloma niewygodami.
 *
 * Revision 1.4  2004/02/02 12:24:55  michalis
 * + dodane wysylanie poczatkowego stringa opisujacego startowa lokacje klientowi
 *
 * Revision 1.3  2004/02/01 00:25:31  michalis
 * * Client wypisuje otrzymane od serwera stringi wprost (bez jakiego prefixu
 *   "Serwer przysa wiadomo" i bez dodawania newline'a na kocu)
 * * PlayerConnection: po wysaniu "Umierasz." dodaje newline
 *
 * Revision 1.2  2004/02/01 00:08:56  michalis
 * + Being.DeadAnswer, dead() returns DeadAnswer
 * - Player/Creature.dead
 * * Player/Creature.setNowLocation: remove object from world.players/creatures list
 * * World.action implemented cleaner
 * * Creature.attack dostaje parametr PlayersMessages
 * + Creature.ElfKiller, dodany do World do lokacji "Jama"
 * + PlayersMessages.append
 * + PlayerConnection.server, + Connection.createPlayer "server" parameter
 * + gdy gracz umiera to jest komunikat dla System.out serwera, komunikat "Umierasz"
 *   dla umierajcego gracza i komunikat "kto-tam umiera" dla pozostaych graczy
 *   w tej lokacji
 * * another bugfix for checking dead state of players in server.
 *   Wszystko przez to e na razie mam doc niewygodne metody testowania tego kodu
 *   skoro gracze nie mog si rusza w wiecie. No, ale myl e obecnie
 *   wszystko ju jest jak trzeba - wszystkie elementy kodu zostay
 *   przetestowane dziki ElfKillerowi.
 *
 * Revision 1.1  2004/01/31 17:36:25  michalis
 * + PlayerConnection.java; wybaczcie, to miaa by cz mojego commita
 *   datowanego 2004/01/31 06:02:10 ale po prostu zapomniaem zrobi cvs add...
 *
 */
