/*
  Copyright 2004 Michalis Kamburelis.

  This file is part of "Jamy & Nory".

  "Jamy & Nory" is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  "Jamy & Nory" is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with "Jamy & Nory"; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package jamy_i_nory.server;

import java.net.*;
import java.nio.channels.*;
import java.io.*;
import java.lang.*;
import java.util.*;
import jamy_i_nory.common.*;
import jamy_i_nory.common.iosystem.*;
import jamy_i_nory.game.*;

/** Serwer gry. Zarzdza wiatem gry (pole {@link #world}) i poczeniami 
   (uywajc klasy ServerListener do nasuchiwania na gniedzie serwera;
   kady klient najpierw trafia na list newUsers, potem connections,
   a wreszcie do world.players() jako obiekt klasy PlayerConnection).
   
   <p>Ma zdefiniowan metod {@link #main} ktra uruchomi 
   cay serwer. Serwer moe dziaa na dowolnym porcie, domylnie
   dziaa na {@link jamy_i_nory.common.Utils#DEFAULT_JAMY_PORT} ale
   mona to zmieni parametrem --port w linii polece).
   W ten sposb mona uruchomi kilka serwerw na tym samym komputerze 
   prowadzcych odrbne gry. */
public class Server 
{
  /** Klasa ServerListener jest klas wewnetrzn klasy Server.
     Jest wtkiem ktry nawizuje i kolejkuje poczenia. Wtek
     glwny (Server) obsuguje (ewentualnie koczy) polaczenia nawizane przez
     ServerListener. Dostp do listy oczekujcych pocze, new Users, 
     jest moliwy tylko przez zsynchronizowane metody add/removeNewUser. */
  private class ServerListener extends Thread {
  
    /** Lista nowych klientw, obiekty typu SocketChannel (no nulls). 
       SimpleListener do niej dodaje, a gwny  wtek serwera z niej wyciga 
       (i zamienia do Connection i dodaje do {@link Server#connections}). 
       Dlatego dostp do tej listy musi by synchronizowany - 
       uywaj {@link #addNewUser} i {@link #removeNewUser}. */
    private LinkedList newUsers = new LinkedList();  
    
    /** Zwraca klienta ktrego zgoszenie zaakceptowa serwer a ktry
       jeszcze nie zosta obsuony ("obsuony" znaczy dodany do 
       Server#connections). Zwraca null jeli nie ma takiego klienta. */
    synchronized SocketChannel removeNewUser()
    {
      try {
        return (SocketChannel) newUsers.removeFirst();
      } catch (NoSuchElementException e) {
        return null;
      }
    }
    
    synchronized void addNewUser(SocketChannel value)
    {
      newUsers.add(value);
    }    

    private ServerSocketChannel serverSocketChannel;

    /** Obiekt ServerSocket ktry tutaj podasz bdzie ju odtd zarzdzany
       we wntrzu tej klasy w osobnym watku; nie rb z nim ju nic
       z pocztkowego wtku. Ponadto nie moesz ju pisa do System.err
       z gwnego wtku (ale do System.out moesz, w szczeglnoci moesz
       pisa do System.out przez IOStd). */
    ServerListener(ServerSocketChannel aServerSocketChannel)
    { serverSocketChannel = aServerSocketChannel; }

    public void run()
    {         
      try {
        while (!isInterrupted())
        {
          SocketChannel clSocket;
          try {
            clSocket = serverSocketChannel.accept();            
          } catch (IOException e) {
            System.err.println("Bd podczas nasuchiwania na poczenie : " +e);
            break;
          }          
          if (clSocket != null) addNewUser(clSocket);
          
          /* We can do some small sleep here to ease CPU load of our server.
             We don't expect 1.000.000 new users connecting each second,
             so this will not be problem. 
             
             Note: if we're interrupted while in sleep() we will get 
             InterruptedException but our isInterrupted flag will NOT
             be set. That is why we're doing here break. */
          try {
            sleep(100);
          } catch (InterruptedException e) { break; }
        } 
      }
      finally {
        try {
          serverSocketChannel.socket().close();
          System.err.println("Gniazdo serwera zamknite.");
        } catch (IOException e) {
          System.err.println("Bd podczas zamykania gniazda serwera : " +e);
        }
      }
    }
  }
    
  /** Lista aktywnych pocze klientw ktrzy jeszcze nie zainicjowali 
     swoich graczy, czyli obiektw klasy Connection (no nulls). */
  private LinkedList connections = new LinkedList();  
  
  /** To jest lista graczy ktrzy swego czasu byli w grze (world.players()) 
     ale si odczyli i zamroziem ich w InactivePlayerConnection 
     (tak, na tej licie wszystkie obiekty maj klas nie tylko
     InactivePlayer ale nawet InactivePlayerConnection).
     Elementy s dodawane do tej listy w gwnej ptli gry i wyjmowane z listy w
     konstruktorze Connection. */
  public InactivePlayersList inactivePlayers = new InactivePlayersList();

  ServerListener serverListenerThread;
  
  static class ServerSocketSetupException extends Exception {
    ServerSocketSetupException(String message, Throwable cause)
    { super(message, cause); }    
  };
  
  /** IOSystem (uywany tylko w gwnym wtku serwera). */
  final IOSystem io;
  
  /** wiat zarzdzany przez ten serwer. 
  
    <p>Lista players() tego wiata bdzie zawieraa jedynie obiekty klasy 
    {@link PlayerConnection}.
     
    <p>Pole final (zamiast metody world()) 
    bo jeden serwer obsuguje ju zawsze tylko jeden wiat, przeczanie serwera 
    na inne wiaty nie miaoby sensu - przed takim przeczeniem trzebaby zamkn
    wszystkie poczenia z klientami (bo ci s zwizani z graczami 
    a gracze - ze wiatem). */  
  final public World world;

  /** Konstruktor.     
    
     @param port numer portu uslugi
      
     @throws ServerSocketSetupException gdy nie mona bylo odpowiednio 
       zainicjowa gniazda serwera. ServerSocketSetupException ju ma 
       odpowiednio dokadny komunikat bdu, w main moemy go wywietla
       wprost. */
  public Server(int port, int ioKind)
  throws ServerSocketSetupException
  { 
    /* init io */
    io = JamyUtils.newIO(ioKind, "> ", "Jamy & Nory - serwer");
    try {

      world = new World();
      /* W (niedalekiej?) przyszoci to w jaki sposb jest tworzony nowy
         wiat bdzie kontrolowane command-line parametrami dla serwera -
         np. z jakiego pliku zaadowa wiat ? */
      io.println("Stworzono nowy wiat.");

      ServerSocketChannel serverSocketChannel;
      try {  
        serverSocketChannel = ServerSocketChannel.open();
        serverSocketChannel.socket().bind(new InetSocketAddress(port));
      } catch (IOException e) {
        throw new ServerSocketSetupException("Can't create and bind server socket: " +e, e);
      }

      try {
        serverSocketChannel.configureBlocking(false);
      } catch (IOException e) {
        throw new ServerSocketSetupException(
          "Error while setting server's socket to non-blocking mode: " +e, e);
      }

      io.println("Serwer nasuchuje w porcie " +port +".");
      serverListenerThread = new ServerListener(serverSocketChannel);
      serverListenerThread.start();

      io.lineBegin();

      while (true)
      {
        /* Przerzu newUsers do connections. */
        {
          SocketChannel clSocket;
          while ((clSocket = serverListenerThread.removeNewUser()) != null)
          {
            try {
              Connection c = new Connection(clSocket);
              connections.add(c);
              io.println("Przyczono klienta " +c);
            } catch (IOException e) {
              io.println("Nie mona przyczy klienta bo nie mona ustawi "
                +clSocket +" na tryb nie-blokujcy.");
            }
          }
        }

        /* Przerzu connections do world.players(). */
        {
          Iterator i = connections.iterator();
          while (i.hasNext()) 
          {
            Connection c = (Connection) i.next();
            if (c.canCreatePlayer())
            {
              i.remove();
              String cToString = c.toString();
              try {            
                PlayerConnection pc = c.createPlayer(world, this);
              } catch (Connection.CannotActivatePlayerException e) {
                io.println("Prba przyczenia klienta " +cToString +  
                  "nie powioda si : " +e.getMessage() +
                  ". Poczenie z klientem zakoczone.");
              }
            }
          }
        }

        /* Odbieraj wiadomoci od userw. 
           TODO: podobnie jak przy sprawdzaniu dead() (patrz niej)
           tak i tutaj nie moemy uywa iteratora tylko dlatego e
           goInactive modyfikuje list world.players(). Podobnie jak
           przy dead() eby zrobi to elegancko naley rozszerzy
           funkcjonalnoc dead(). */
        {
          int i = 0;
          while (i < world.players().size())
          {
            PlayerConnection player = (PlayerConnection) 
              world.players().get(i);

            if (!player.sSocket.isConnected()) {
              inactivePlayers.add(new InactivePlayerConnection(player)); 
            } else
            {
              try {
                String s;
                if ((s = player.sSocket.read()) != null)
                  /* Przetwarzaj komend s od gracza player. */
                  UserCommand.execute(player, s);
                i++;
              } catch (UserCommand.EndGameException e) {
                inactivePlayers.add(new InactivePlayerConnection(player)); 
              }              
            }
          }
        }

        /* move creatures etc. */
        world.action();

        /* send pending messages. Prawie kada kawaek tej olbrzymiej ptli
           gry moe dopisywa rne rzeczy do getMessage() graczy.
           Wic dlaczego wysyamy je akurat tutaj ? eby do graczy ktrych
           za chwil rozczymy (w wyniku dead()) wysa im ich ostatnie
           wiadomoci. */
        sendPlayersMessages();

        /* check if some player died (must be done after world.action) */      
        {        
          /* TODO: moe zapisa adniej, jako wmontowa opcj do player.dead()
             eby usuwao poprzez podany iterator a nie bezporednio z 
             world.players(). */        
          boolean someoneDied;   
          do {
            someoneDied = false;
            Iterator i = world.players().iterator();
            while (i.hasNext()) 
              if (((Player) i.next()).dead()) {
                someoneDied = true;
                break;
              }
          } while (someoneDied);
        }

        /* check server console */
        if (ioCheckInput())
        {
          closeServerSession(); 
          return; /* wyjd z konstruktora Server */
        }
      }
    } finally { io.dispose(); }
  }
  
  /** Po pierwsze, zakada e io.lineReading. Sprawdza czy io.lineReady
     i jeli tak to robi io.lineEnd, interpretuje odczytan komend,
     i (tylko jeli ta komenda nie bya komend zakoczenia serwera, 
     patrz niej) wykonuje komend i robi io.lineBegin . 
     Jeli nie io.lineReady to nic nie robi.
     
     <p>Zwraca true tylko jeeli odczyta komend i t komend byo polecenie 
     zakoczenia serwera. W tym wyjtkowym przypadku ioCheckInput nic nie robi 
     w zwiazku z dan komend (nie prbuje jej wykona itd.), nie robi
     io.lineBegin przy wyjciu i zwraca true. W kadym innym przypadku
     (czyli gdy not io.lineReady lub odczytano i wykonano normaln komend)
     ioCheckInput zwraca false i po zakoczeniu ioCheckInput masz pewno
     e io.lineReading. */
  private boolean ioCheckInput()
  {  
    if (io.lineReady())
    {
      try {
        ServerCommand.execute(this, io.lineEnd());
      } catch (ServerCommand.CloseServerCommandException e) { return true; }

      io.lineBegin();
    }
    return false;      
  }
  
  /** Wysya zapisany getMessage kadego gracza. */
  public void sendPlayersMessages()
  {
    Iterator i = world.players().iterator();
    while (i.hasNext())
      ((PlayerConnection) i.next()).pushMessage();
  }

  /** Zamyka gniazda serwera (nasuchujce) i poczenia z klientami. */
  public void closeServerSession() 
  {
    serverListenerThread.interrupt(); 
    try {
      serverListenerThread.join(); 
    } catch (InterruptedException e) {
      throw new Utils.InternalError(e);
    }
    
    Iterator i = world.players().iterator();
    while (i.hasNext()) ((PlayerConnection) i.next()).sSocket.close();
  }

  public static void main(String[] argv)
  {    
    class OptionProcesser implements Utils.CommandLineOptionProcesser
    {      
      int paramServerPort = JamyUtils.DEFAULT_JAMY_PORT;
      final int DEFAULT_IO = JamyUtils.IO_CONSOLE;
      int paramIO = DEFAULT_IO;

      public void processCommandLineOption(int optionNum, String[] optionArgs)
      {
        switch (optionNum)
        {
          case -1:
            System.err.println("Excessive parameter \"" +optionArgs[0] +"\""); 
            System.exit(1); 

          case 0: 
            System.out.print(
              "Usage: jamy_server [OPTIONS...]" +Utils.nl +
              "  where available options are" +Utils.nl +
             /*01234567890123456789*/
              "  --help            Show this help" +Utils.nl +
              "  --port PORT       Listen on PORT (default port is " 
                +JamyUtils.DEFAULT_JAMY_PORT+ ")" +Utils.nl+
              JamyUtils.ioParamsHelp(DEFAULT_IO)
            );
            System.exit(0);

          case 1: 
            try {
              paramServerPort = Integer.parseInt(optionArgs[0]);
            } catch (NumberFormatException e) {
              System.err.println("Invalid number : " +e);
              System.exit(1);          
            }
            break;
            
          case 2: paramIO = JamyUtils.IO_CONSOLE; break;
          case 3: paramIO = JamyUtils.IO_SIMPLE_SWING; break;
          case 4: paramIO = JamyUtils.IO_STYLISH_SWING; break;
        }
      }
    };
    
    /* parse command-line */
    OptionProcesser processer = new OptionProcesser();
    Utils.parseCommandLine(argv, 
      new Utils.CommandLineOption[] {
        new Utils.CommandLineOption("help", 0),
        new Utils.CommandLineOption("port", 1),
        new Utils.CommandLineOption(JamyUtils.IO_PARAMS_NAMES[JamyUtils.IO_CONSOLE], 0),
        new Utils.CommandLineOption(JamyUtils.IO_PARAMS_NAMES[JamyUtils.IO_SIMPLE_SWING], 0),
        new Utils.CommandLineOption(JamyUtils.IO_PARAMS_NAMES[JamyUtils.IO_STYLISH_SWING], 0),
      }, processer);
  
    /* create Server instance */
    try {
      new Server(processer.paramServerPort, processer.paramIO);
    } catch (ServerSocketSetupException e) {
      System.err.println(e.getMessage());
      return;
    }
  }
}

/*
 * $Log: Server.java,v $
 * Revision 1.23  2004/02/19 21:54:25  michalis
 * + klasa ClientApplet ostatecznie zrobiona - dziaa !
 * * useStyles wycignite jako parametr konstruktorw IOSwing
 * + JamyUtils z pomocniczymi rzeczami gromadzcymi parametry do interfejsw,
 *   teraz i serwer i klient i single maj peen zestaw 3 parametrw --console, --simple-swing, --stylish-swing
 * * odpowiednie uaktualnienia w dokumentacji
 *
 * Revision 1.22  2004/02/18 14:37:37  michalis
 * * pl stringi
 * * --help output changed (to reflect that those classes will be run by bin/scripts)
 * preparations for ClientApplet:
 * * Client rozdzielony na Client i ClientStandalone
 * * znaczna cz pakietu common przeniesiona do nowego pakietu game
 * X* make jamy_i_nory_client_applet.jar
 *
 * Revision 1.21  2004/02/10 00:39:42  michalis
 * * po kilku bezproduktywnych commitach (jakie reorganizacje ...) co konkretnego:
 *   Zamroony na serwerze gracz jest chroniony hasem.
 *
 * Revision 1.20  2004/02/09 23:08:57  michalis
 * + GPL license clause i copyrighty (moje i gdzieniegdzie Michaa, sorry Rafa
 *   ale naprawd nic nie zostao) do wszystkich *.java
 *
 * Revision 1.19  2004/02/06 06:56:24  michalis
 * * Wyciem komentarze Rafaa "tu zmieniem" bo za bardzo zamiecay tekst.
 *   (Nie odwracaem dokonanych zmian w tekcie, usunem tylko te komentarze).
 * * Fragment o JamySingle poprawiony (co mi tam cigle zgrzytao).
 * * Poprawione cudzysowy na LaTeXowe
 * + Rozszerzona dokumentacja do uruchamiania serwera: parametr --swing
 * + Sekcja "obsuga serwera"
 *
 * Revision 1.18  2004/02/06 06:21:04  michalis
 * + PlayerConnection: dodatkowy konstruktor
 * + InactivePlayersList
 * * rozszerzona funkcjonalnoc Connection (musi umie wywoa oba konstruktory
 *   PlayerConnection)
 *   Dziaa ! Zamraanie i odmraanie graczy zrobione !
 *
 * Revision 1.17  2004/02/06 04:44:42  michalis
 * + InactivePlayer
 * + dodatkowy konstruktor do Being
 * + dodatkowy konstruktor Player(World, InactivePlayer)
 * * beforeDeath rozszerzone o parametr goingInactive
 * + Being.goInactive
 * * listToString moved from World to Utils
 * + ServerCommand : polecenie "mrozonki"
 *
 * Revision 1.16  2004/02/06 01:24:35  michalis
 * * implementacja UserCommandBlackTokens.matches przeniesiona do Utils.stringArraysNullEqual
 * + ServerCommand
 * * Server.ioCheckInput uproszczone dziki temu
 * * komendy serwera na polski, konsekwentnie
 * + ServerCommand "zabij ID"
 *
 * Revision 1.15  2004/02/05 23:27:44  michalis
 * * jeszcze mae poprawki w kwestii kiedy pisa TO-DO a kiedy to samo bez mylnika
 * * IOSwing konstruktor: parametr windowTitle
 * + Server obsuguje parametr --swing, ma pole IOSystem
 * + Utils.collectionToMultiLineString(Collection,String)
 * * Server mona ju kontrolowa interfejsem IOSystem, polecenia "x", "h", "p", "c"
 * + World.allCreatures/PlayersToString,
 * * UserCommand. "debug players/creatures" uywaj World.allCreatures/PlayersToString
 * * Poprawka w Server.ServerListener.run przy sleep
 *
 * Revision 1.14  2004/02/05 20:05:29  michalis
 * * Zrobiem eby w logach CVSa zapisanych w plikach byo wszdzie TO-DO
 *   z mylnikiem w rodku. Wprawdzie nieznacznie rozsynchronizowuje to
 *   zapisane w plikach logi z wynikiem 'cvs log' ale za to grep TO-DO
 *   (gdzie w wyrazie TO-DO naley usun mylnik) dziaa jak trzeba.
 *
 * Revision 1.13  2004/02/05 00:31:28  michalis
 * * docs improved
 *
 * Revision 1.12  2004/02/04 18:56:45  michalis
 * + some improvements in Client (waitForMessageAndPrint, sSocketRead, sScoketWrite)
 * + Player.followLink
 * * UserCommand.execute does not return String (it returns void).
 *   It should simply append something to player.getMessage().
 *   Wiele uproszcze zapisu w UserCommand - np. komenda "idz" jest teraz
 *   krciutka i wszystkim zajmuje si Player.followLink i Player.setNowLocation.
 * + PlayerConnection.pushMessage, * PlayerConnection.dead slightly improved
 * - Server.sendMessage, - Server.broadcastMessage (useless - we use only
 *   sendPlayersMessages and PlayerConnection.pushMessage)
 *
 * Revision 1.11  2004/02/04 15:36:44  michalis
 * + Dziki pozbyciu si PlayersMessages moemy teraz wypisywa stringi
 *   o tym kto wchodzi/wychodzi z pomieszczenia w setNowLocation.
 * + Being.beforeDead, Being.toStringBase eby setNowLocation
 *   mogo take wywietla kto si pojawia (jako nowy gracz/potwr w wiecie)
 *   uywane z konstruktora Being i z dead()
 * * Server zosta uproszczony w wyniku powyszej zmiany, Being.dead te
 *   (chocia pojawia si niewielka strata funkcjonalnoci - patrz TO-DO przy
 *   toStringBase())
 *
 * Revision 1.10  2004/02/04 14:00:28  michalis
 * * Being.tellEveryoneInNowLocation -> sayEveryoneInNowLocation
 *   (mniej mylca nazwa)
 * * Server powiadamia wszystkich graczy w komnacie e pojawi si nowy gracz
 * * PlayersMessages.tellEveryoneInLocation - docs updated,
 *   implemented in a cleaner way
 * + MessageGenerator
 * + MessageFormatter
 * * Being.attitude changed to realAttitude and observedAttitude
 * * Many fixes to use MessageFormatter when updating PlayersMessages.
 *   Many TO-DOs closed.
 * * Attitude.toStr translated to polish
 * * Poprawiony bug w Being.dead - Micha, nie mona tworzy PlayersMessages
 *   tu przed usuniciem gracza z gry - w czasie uywania PlayersMessages
 *   world.players() nie mog by modyfikowane. Z drugiej strony, to jest moja
 *   wina e jeszcze tego nie mona zrobi (bo to *powinno* by moliwe tylko
 *   musz zapisa inteligentniej PlayersMessages). OK, wanie mi przyszo do
 *   gowy jak to zrobi bardzo atwo i bardzo elegancko (klasa PlayersMessages
 *   w ogle nie bdzie potrzebna i unikniemy przerzucania wszdzie parametrw
 *   playersMessages) i w nastpnym commicie to bdzie zrobione (gupi, gupi,
 *   gupi jestem e od pocztku tego tak nie zrobiem).
 * * OK, serwer skunk.one.pl nie odpowiada wic nie bd commitowa, od razu
 *   zrobi to o czym mwiem powyej:
 *   + Player.getMessage, Player.putMessage
 *   + Location.tellEveryone (3 wersje)
 *   - PlayersMessages (funkcjonalno przerzucona w caoci do obiektw Player i
 *     Location)
 *   * masa uproszcze kodu (m.in. nie musimy ju przerzuca wszdzie
 *     parametrw PlayersMessages)
 *   - klasa Being.DeadAnswer (useless, teraz Being.dead() znowu zwraca prostego
 *     booleana)
 *   * w ogle Being.dead() jest teraz o niebo prostsze.
 *   Skoczyem! PlayersMessages usunite razem z wieloma niewygodami.
 *
 * Revision 1.9  2004/02/01 01:20:23  michalis
 * * small docs improvement
 *
 * Revision 1.8  2004/02/01 00:57:00  michalis
 * * Being.dead very small changes
 * * StringSocket small write improvement
 * + Server.Server processes messages from clients using UserCommand.
 *   Dziaa ! Merge JamySingle + Client zakoczony - mamy zrobione
 *   co co mona nazwa dziaajcym MUDem !
 *   (i udao mi si zachowa te JamySingle w wersji dziaajcej !)
 *
 * Revision 1.7  2004/02/01 00:08:56  michalis
 * + Being.DeadAnswer, dead() returns DeadAnswer
 * - Player/Creature.dead
 * * Player/Creature.setNowLocation: remove object from world.players/creatures list
 * * World.action implemented cleaner
 * * Creature.attack dostaje parametr PlayersMessages
 * + Creature.ElfKiller, dodany do World do lokacji "Jama"
 * + PlayersMessages.append
 * + PlayerConnection.server, + Connection.createPlayer "server" parameter
 * + gdy gracz umiera to jest komunikat dla System.out serwera, komunikat "Umierasz"
 *   dla umierajcego gracza i komunikat "kto-tam umiera" dla pozostaych graczy
 *   w tej lokacji
 * * another bugfix for checking dead state of players in server.
 *   Wszystko przez to e na razie mam doc niewygodne metody testowania tego kodu
 *   skoro gracze nie mog si rusza w wiecie. No, ale myl e obecnie
 *   wszystko ju jest jak trzeba - wszystkie elementy kodu zostay
 *   przetestowane dziki ElfKillerowi.
 *
 * Revision 1.6  2004/01/31 21:50:34  michalis
 * * bugfix (message "Gracz umiera ..." byo wypisywane nieco zbyt czsto...)
 *
 * Revision 1.5  2004/01/31 20:47:49  michalis
 * * JamySingle parameter console changed to --console
 * * UserCommand moved from tests/ to common/
 * * UserCommand small changes (very small doc updates, some things made public
 *   to be available from outside of package jamy_i_nory.common)
 * * tests/SimpleClient.java moved to client/Client.java
 * * tests/local_client.sh moved to client/ and modified
 * * Server: StringBuffer[] changed to PlayerMessages everywhere
 *
 * Revision 1.4  2004/01/31 06:35:03  michalis
 * + Server.sendMessages, Server now sends messages about "what is going on in
 *   the world" to players, dead() state of players is now checked.
 *
 * Revision 1.3  2004/01/31 06:02:11  michalis
 * + Server.world
 * + SimpleClient.newPlayer
 * * Idea of Connection changed : now it does not inherit from StringSocket.
 *   It just holds stringSocket inside, and can create PlayerConnection.
 * + PlayerConnection : inherits from Player, contains StringSocket.
 *   (This is what was eariler meant to be in Connection)
 * * Server.users split to connections and world.players()
 * + Server adds ready connections to world.players() using Connection.createPlayer
 * - Server.findById (it was just a TO-DO) removed,
 *   now it can be implemented using IdedList.byId
 * + important comments at world.action
 * * StringSocket.read bug (when receiving two messages *fast*) fixed
 * + Utils.InternalError(String, Throwable)
 * * some Server docs improved
 *
 * Revision 1.2  2004/01/30 00:59:44  michalis
 * * StringSocket.read small fix
 * + Server.broadcastMessage
 * - Removed some TO-DO in Server by Rafa; jeli dobrze je rozumiem to wykonaem je
 *   robic klas StringSocket
 * + Server wysya wiadomo do wszystkich klientw gdy podcza si nowy klient
 *   (to tylko testowe, w kocowej grze to nie ma by widoczne dla wszystkich)
 * + SimpleClient odbiera wiadomoci od serwera i wywietla; w ten sposb Server
 *   i SimpleClient robi pen komunikacj - obaj mog co wysya,
 *   obaj mog co odbiera
 *
 * Revision 1.1  2004/01/30 00:30:26  michalis
 * * SimpleServer renamed to Server
 *
 * Logs as SimpleServer.java:
 * Revision 1.11  2004/01/30 00:12:02  michalis
 * + SimpleServer uses Utils.parseCommandLine, params --help and --port implemented
 * + Utils.DEFAULT_JAMY_PORT
 * * SimpleClient: small improvement to message printed by --help
 * * small improvement to local_server.sh and server.sh
 *
 * Revision 1.10  2004/01/29 19:55:46  michalis
 * + code from server/Connection moved to common/StringSocket
 *   (with some minor improvements and clarification of interface)
 * - Connection class is empty for now - it only inherits everything from
 *   StringSocket, nothing more
 * * SimpleServer.sendMessage(Connection,String) implemented (using new
 *   Connection/StringSocket interface), so all sendMessage methods uncommented again;
 *   and they don't throw IOException anymore - this is already handled
 *   in StringSocket.write()
 * * SimpleServer.closeServerSession: some minor improvemnets
 * - SimpleServer comment : @author Rafal Wawrzkowicz removed - myl e
 *   zrobiem wystarczajco duo w tej klasie eby mie co do powiedzenia.
 *   Nie twrzmy szczegowych list autorw do kadej klasy - w logach kady
 *   widzi co kto zrobi; a oryginalny autor pliku z klas nie zawsze jest osob
 *   ktra doprowadzia klas do stanu wzgldnej uywalnoci...
 *
 * Revision 1.9  2004/01/29 16:14:45  michalis
 * + sleep(100)
 *
 * Revision 1.8  2004/01/29 16:01:25  michalis
 * + ServerListener.(add|remove)NewUser
 * * newUsers moved to ServerListener as private
 * * appropriate changes in SimpleServer, now it looks better (jest teraz
 *   trudniej omykowo obej synchronizacj newUsers)
 * * par stringw poprawionych (na uniwerku pod XEmacsem nie pisaem pl ogonkw)
 *
 * Revision 1.7  2004/01/29 14:13:18  michalis
 * * SimpleServer and Connection changed to use java.nio - ServerSocketChannel and
 *   SocketChannel instead ServerSocket and Socket. This allows to use non-blocking
 *   checking for end of stream, so Connection.isConnected works now.
 * * sendMessage temporarily broken
 *
 * Revision 1.6  2004/01/29 12:19:21  michalis
 * * using Connection.isConnected, getMessage
 * * closeServerSession does not throw IOException anymore
 *
 * Revision 1.5  2004/01/29 04:00:55  michalis
 * + Connection.toString
 * + SimpleClient wysya bajty (ze Stringa)
 * + SimpleServer odbiera bajty i wypisuje na konsol (jako String) - co i od kogo dosta
 * * Now SimpleServer can be stopped only by C-c (i.e., killing the process).
 *   Functionality to close server "gently" will be restored soon.
 *
 * Revision 1.4  2004/01/27 00:50:51  michalis
 * * creating serverSocket moved to SimpleServer constructor:
 *   SimpleServer should die, not it's thread, if creating serverSocket fails.
 *   One should always try to keep everything single-threaded as far as possible.
 * - removed service field (useless)
 * + setSoTimeout added so that accept() does not block forever (and can see
 *   that listeningEnabled is no longer true)
 * - removed listeningEnabled (useless, serverListenerThread.interrupt()
 *   already does the work and handles some specific cases too)
 * * fixed indentation (Rafa: nie zamierzam przerabia Twojego indentation
 *   na swoje bo tak mi si podoba; ale w tej klasie byy mieszane indentation
 *   - cz rzeczy na 3, cz na 4 spacje. Wic zmieniem wszystko na 2.
 *   Rb konsekwentnie to nie bd zmienia.)
 * + ServerSocketSetupException
 * + closing server with pressing enter
 * + closeServerSession now closes serverListenerThread
 * * ServerListener actually rewritten
 * * iterating over users (with users.indexOf(users.getLast()) - WTF was that ?!?!)
 *   in closeServerSession reimplemented using iterator.
 *
 * Revision 1.3  2004/01/26 20:56:45  michalis
 * + SimpleClient
 * * SimpleServer made public (so java jamy_i_nory.server.SimpleServer works)
 * * SimpleServer fixed (users were never created !)
 *
 * Revision 1.2  2004/01/26 20:09:55  michalis
 * * line endings CRLF -> LF
 * * main Makefile modified to clean server/ dir
 * + basic Makefile in server/
 * * class SimpleServerProtocol renamed to SimpleGameProtocol to match filename
 *   (Rafa, przecie to si w ogle nie kompilowao)
 * + required imports to Connection
 *   (Rafa, przecie to si w ogle nie kompilowao)
 * + package jamy_i_nory.server; na pocztku Message.java
 *   (Rafa, przecie to si w ogle nie kompilowao)
 * * SimpleServer.closeServerSession() fixed (downcast to Connection needed)
 *   (Rafa, przecie to si w ogle nie kompilowao)
 *
 * Revision 1.1  2004/01/26 19:41:03  michalis
 * Klasy serwera Rafaa:
 * * TABy zamienione na spacje (Rafa, do k.n., nie uywaj TABw)
 * * Windows-1250 zamieniony na ISO-8859-2 (Rafa, do k.n., nie uywaj
 *   Windows-1250; uywaj ogonkw w ISO albo wcale)
 * * "Server" -> "server" (all files removed from Server/ and added again to server/)
 * + Logi CVSa dodane
 *
 */
