/*
  Copyright 2004 Michalis Kamburelis.

  This file is part of "Jamy & Nory".

  "Jamy & Nory" is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  "Jamy & Nory" is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with "Jamy & Nory"; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package jamy_i_nory.single;

import jamy_i_nory.common.*;
import jamy_i_nory.common.iosystem.*;
import jamy_i_nory.game.*;

/** Ta klasa dostarcza metod {@link #main main} ktra uruchamia
   nasz gr w trybie nie-sieciowym. Ot, taka tekstowa przygodwka
   dla gracza ktry chce pogra z komputerem.

   <p>Some command-line options are allowed, run with --help to get
   more info. */
class JamySingle {

  /** IOSystem to be used by this class. Will be initialized in main. */
  static private IOSystem io;
  
  /** Interactive constructor. Uses {@link #io} to get required
     information about player, then calls normal {@linkplain Player#Player
     Player constructor}. */
  static private Player newPlayerInteractive(World aWorld)
  { 
    io.println("----------------------------------------");
    io.println("*** Tworzenie nowego gracza ***");

    String playerName = io.readString("Imi:", false);

    for (int i=0; i<Race.playerRaces.length; i++)
      io.println("  [" +i +"] " +Race.playerRaces[i].toDetailString());
    int raceNum = io.readInt("Wybierz ras:" , 0, Race.playerRaces.length-1);

    return new Player(aWorld, Race.playerRaces[raceNum], playerName);
  }  

  public static final void main(String[] argv)
  {
    class OptionProcesser implements Utils.CommandLineOptionProcesser
    {
      final int DEFAULT_IO = JamyUtils.IO_SIMPLE_SWING;
      int paramIO = DEFAULT_IO;

      public void processCommandLineOption(int optionNum, String[] optionArgs)
      {
        switch (optionNum)
        {
          case -1:
            System.err.println("Excessive parameter \"" +optionArgs[0] +"\""); 
            System.exit(1); 

          case 0: 
            System.out.print(
              "Usage: jamy_single [OPTIONS...]" +Utils.nl +
              "  where available options are" +Utils.nl +
             /*01234567890123456789*/
              "  --help            Show this help" +Utils.nl +
              JamyUtils.ioParamsHelp(DEFAULT_IO)
            );
            System.exit(0);

          case 1: paramIO = JamyUtils.IO_CONSOLE; break;
          case 2: paramIO = JamyUtils.IO_SIMPLE_SWING; break;
          case 3: paramIO = JamyUtils.IO_STYLISH_SWING; break;
        }
      }
    };
    
    /* parse command-line */
    OptionProcesser processer = new OptionProcesser();
    Utils.parseCommandLine(argv, 
      new Utils.CommandLineOption[] {
        new Utils.CommandLineOption("help", 0),
        new Utils.CommandLineOption(JamyUtils.IO_PARAMS_NAMES[JamyUtils.IO_CONSOLE], 0),
        new Utils.CommandLineOption(JamyUtils.IO_PARAMS_NAMES[JamyUtils.IO_SIMPLE_SWING], 0),
        new Utils.CommandLineOption(JamyUtils.IO_PARAMS_NAMES[JamyUtils.IO_STYLISH_SWING], 0),
      }, processer);  
  
    io = JamyUtils.newIO(processer.paramIO, "> ", "Jamy & Nory - wersja niesieciowa");
      
    try {
      try {  
        World world = new World();
        do {

          /* start new game with new player */
          Player player = newPlayerInteractive(world);
          /* wypisz tutaj player.getMessage() aby moliwie unika wypisywania
             czegokolwiek podczas io.lineReading bo to wyglda paskudnie na
             interfejsie konsolowym. */
          io.print(player.getMessage());
          
          io.lineBegin();
          try {
            while (!player.dead()) {
              world.action();
              io.print(player.getMessage());

              /* check user input */
              if (io.lineReady())
              {
                String command = io.lineEnd();
                /* exit cleanly when we ran out of input */
                if (command == null) {
                  io.println("Koniec (stdin zakoczone).");
                  throw new UserCommand.EndGameException();
                }
                
                UserCommand.execute(player, command);
                
                /* wypisz tutaj player.getMessage() z tego samego powodu co
                   opisaem w Client.waitForMessageAndPrint : eby moliwie 
                   unika wypisywania tej wiadomoci w czasie lineReading() co
                   wyglda paskudnie w interfejsie konsolowym. */
                io.print(player.getMessage());
                
                io.lineBegin();
              }
            }
          } finally { io.lineEndCancel(); }
          
          io.println("Umierasz.");
        } while (io.readBoolean("Start another game (in the same world) ?", true));
      } catch (UserCommand.EndGameException e) { io.println("Koniec."); }; 
    } finally { io.dispose(); }
  }
}

/*
 * $Log: JamySingle.java,v $
 * Revision 1.14  2004/02/19 21:54:27  michalis
 * + klasa ClientApplet ostatecznie zrobiona - dziaa !
 * * useStyles wycignite jako parametr konstruktorw IOSwing
 * + JamyUtils z pomocniczymi rzeczami gromadzcymi parametry do interfejsw,
 *   teraz i serwer i klient i single maj peen zestaw 3 parametrw --console, --simple-swing, --stylish-swing
 * * odpowiednie uaktualnienia w dokumentacji
 *
 * Revision 1.13  2004/02/18 14:37:42  michalis
 * * pl stringi
 * * --help output changed (to reflect that those classes will be run by bin/scripts)
 * preparations for ClientApplet:
 * * Client rozdzielony na Client i ClientStandalone
 * * znaczna cz pakietu common przeniesiona do nowego pakietu game
 * X* make jamy_i_nory_client_applet.jar
 *
 * Revision 1.12  2004/02/15 15:24:45  michalis
 * + pdf generating
 * + COPYING file
 * + Utils.check
 * + Location.add/removeLink
 * + LocationWithDynLink
 * * NoraOrBiblioteka.useItem simplified
 * * teraz i w bibliotece i w norze s drzwi
 * - Makefiles rozsiane po katalogach.
 * * Globalny Makefile rzdzi.
 *
 * Revision 1.11  2004/02/09 23:09:01  michalis
 * + GPL license clause i copyrighty (moje i gdzieniegdzie Michaa, sorry Rafa
 *   ale naprawd nic nie zostao) do wszystkich *.java
 *
 * Revision 1.10  2004/02/05 23:27:45  michalis
 * * jeszcze mae poprawki w kwestii kiedy pisa TO-DO a kiedy to samo bez mylnika
 * * IOSwing konstruktor: parametr windowTitle
 * + Server obsuguje parametr --swing, ma pole IOSystem
 * + Utils.collectionToMultiLineString(Collection,String)
 * * Server mona ju kontrolowa interfejsem IOSystem, polecenia "x", "h", "p", "c"
 * + World.allCreatures/PlayersToString,
 * * UserCommand. "debug players/creatures" uywaj World.allCreatures/PlayersToString
 * * Poprawka w Server.ServerListener.run przy sleep
 *
 * Revision 1.9  2004/02/05 20:28:24  michalis
 * * Klasy IOSystem/Std/Swing przeniesione do pakietu common.iosystem.
 *   Byy uywane take przez single/ wic ich dotychczasowa lokalizacja
 *   w client/ bya za. Ponadto zaraz bd uywane take przez server/.
 *
 * Revision 1.8  2004/02/05 20:05:30  michalis
 * * Zrobiem eby w logach CVSa zapisanych w plikach byo wszdzie TO-DO
 *   z mylnikiem w rodku. Wprawdzie nieznacznie rozsynchronizowuje to
 *   zapisane w plikach logi z wynikiem 'cvs log' ale za to grep TO-DO
 *   (gdzie w wyrazie TO-DO naley usun mylnik) dziaa jak trzeba.
 *
 * Revision 1.7  2004/02/04 23:55:03  michalis
 * + dodany przedmiocik "szkatuka" do lokacji "Salon" z 3 piercieniami
 * * Being.beforeDeath - maa poprawka eby upuszcza plecak przez dropOnFloor
 *   (zeby byy komunikaty Player.putMessage)
 * * some Being interface clarifications
 * + dodane wywoanie putMessage(nowLocationToSingle()) do konstruktora Player,
 *   to pozwala nieco uproci PlayerConnection i JamySingle
 *
 * Revision 1.6  2004/02/04 19:03:31  michalis
 * * JamySingle dostosowane do tego e UserCommand.execute nie zwraca ju Stringa
 *
 * Revision 1.5  2004/02/04 14:00:29  michalis
 * * Being.tellEveryoneInNowLocation -> sayEveryoneInNowLocation
 *   (mniej mylca nazwa)
 * * Server powiadamia wszystkich graczy w komnacie e pojawi si nowy gracz
 * * PlayersMessages.tellEveryoneInLocation - docs updated,
 *   implemented in a cleaner way
 * + MessageGenerator
 * + MessageFormatter
 * * Being.attitude changed to realAttitude and observedAttitude
 * * Many fixes to use MessageFormatter when updating PlayersMessages.
 *   Many TO-DOs closed.
 * * Attitude.toStr translated to polish
 * * Poprawiony bug w Being.dead - Micha, nie mona tworzy PlayersMessages
 *   tu przed usuniciem gracza z gry - w czasie uywania PlayersMessages
 *   world.players() nie mog by modyfikowane. Z drugiej strony, to jest moja
 *   wina e jeszcze tego nie mona zrobi (bo to *powinno* by moliwe tylko
 *   musz zapisa inteligentniej PlayersMessages). OK, wanie mi przyszo do
 *   gowy jak to zrobi bardzo atwo i bardzo elegancko (klasa PlayersMessages
 *   w ogle nie bdzie potrzebna i unikniemy przerzucania wszdzie parametrw
 *   playersMessages) i w nastpnym commicie to bdzie zrobione (gupi, gupi,
 *   gupi jestem e od pocztku tego tak nie zrobiem).
 * * OK, serwer skunk.one.pl nie odpowiada wic nie bd commitowa, od razu
 *   zrobi to o czym mwiem powyej:
 *   + Player.getMessage, Player.putMessage
 *   + Location.tellEveryone (3 wersje)
 *   - PlayersMessages (funkcjonalno przerzucona w caoci do obiektw Player i
 *     Location)
 *   * masa uproszcze kodu (m.in. nie musimy ju przerzuca wszdzie
 *     parametrw PlayersMessages)
 *   - klasa Being.DeadAnswer (useless, teraz Being.dead() znowu zwraca prostego
 *     booleana)
 *   * w ogle Being.dead() jest teraz o niebo prostsze.
 *   Skoczyem! PlayersMessages usunite razem z wieloma niewygodami.
 *
 * Revision 1.4  2004/02/01 01:16:58  michalis
 * * small docs improvements
 * - Client testing message ("Enter lines ...") removed
 *
 * Revision 1.3  2004/02/01 00:08:57  michalis
 * + Being.DeadAnswer, dead() returns DeadAnswer
 * - Player/Creature.dead
 * * Player/Creature.setNowLocation: remove object from world.players/creatures list
 * * World.action implemented cleaner
 * * Creature.attack dostaje parametr PlayersMessages
 * + Creature.ElfKiller, dodany do World do lokacji "Jama"
 * + PlayersMessages.append
 * + PlayerConnection.server, + Connection.createPlayer "server" parameter
 * + gdy gracz umiera to jest komunikat dla System.out serwera, komunikat "Umierasz"
 *   dla umierajcego gracza i komunikat "kto-tam umiera" dla pozostaych graczy
 *   w tej lokacji
 * * another bugfix for checking dead state of players in server.
 *   Wszystko przez to e na razie mam doc niewygodne metody testowania tego kodu
 *   skoro gracze nie mog si rusza w wiecie. No, ale myl e obecnie
 *   wszystko ju jest jak trzeba - wszystkie elementy kodu zostay
 *   przetestowane dziki ElfKillerowi.
 *
 * Revision 1.2  2004/01/31 21:40:44  michalis
 * + IOSystem and subclasses: method lineEndCancel
 * * JamySingle and Client use lineEndCancel to avoid some bug
 *
 * Revision 1.1  2004/01/31 21:19:13  michalis
 * * JamySingle use Utils.parseCommandLine, params: --help and --console
 * * JamySingle moved to single/ (this is not a testing class anymore)
 * + single/single.sh, single/Makefile
 *
 * Logs as tests/JamySingle.java:
 * Revision 1.13  2004/01/31 20:47:53  michalis
 * * JamySingle parameter console changed to --console
 * * UserCommand moved from tests/ to common/
 * * UserCommand small changes (very small doc updates, some things made public
 *   to be available from outside of package jamy_i_nory.common)
 * * tests/SimpleClient.java moved to client/Client.java
 * * tests/local_client.sh moved to client/ and modified
 * * Server: StringBuffer[] changed to PlayerMessages everywhere
 *
 * Revision 1.12  2004/01/31 20:10:31  michalis
 * + IOSystem: some TO-DO with comments moved from UserCommand to IOSystem
 * * UserCommand docs improved
 * * UserCommand.s renamed to commandPattern
 * + PlayersMessages
 * * World.messageToPlayersInLocation moved to PlayersMessages.tellEveryoneInLocation
 * * World.action changed to use PlayersMessages (much cleaner code now!)
 * * Creature.action and doAction use PlayersMessages
 * * UserCommand.execute interface changed:
 *   - world (excessive, one can always use player.world)
 *   + playersMessages (to tell other players about effects of this command)
 *   + returns String (tell command invoker about effects of this command)
 * * every command's execute changed to use new interface:
 *   nothing uses JamySingle.io, nothing may call Player.dead()
 *
 * Revision 1.11  2004/01/19 00:06:08  michalis
 * * world.doAction -> world.action
 *
 * Revision 1.10  2004/01/18 17:50:02  michalis
 * * DiceRandom.toString slightly improved (+ tests for it in BaseTests),
 * + Race.toDetailString (* used in JamySingle)
 *
 * Revision 1.9  2004/01/18 04:30:16  michalis
 * + Embedded main code in try {...} finally { io.lineEnd }
 *
 * Revision 1.8  2004/01/17 22:14:18  michalis
 * * some docs improved in JamySingle, IOSystem and IOSwing
 * * JOptionPane in IOSwing used only by swingThreadInvoke
 *
 * Revision 1.7  2004/01/17 17:02:27  michalis
 * * JamySingle updated to newest state of IOSystem and decendants:
 *   + calls io.dispose
 *   * default IOSystem implementation is IOSwing
 *   + you can use param "console" to switch back to IOStd
 *
 * Revision 1.6  2004/01/16 20:08:36  michalis
 * * JamySingle uywa world.doAction i nowego interfejsu IOSystem i IOStd
 *
 * Revision 1.5  2004/01/16 01:23:57  michalis
 * * JamySingle uses non-blocking io from IOStd, some TO-DOs added
 *
 * Revision 1.4  2004/01/15 20:35:19  michalis
 * * docs improved, doc/Makefile changed to generate docs for jamy_i_nory.client
 *
 * Revision 1.3  2004/01/15 20:04:38  michalis
 * + jamy_i_nory.client.IOSystem
 * + jamy_i_nory.client.IOStd
 * + JamySingle.io, JamySingle.newPlayerInteractive (from Player.newInteractive)
 * * JamySingle and UserCommand use JamySingle.io instead of System.in/out
 *
 * Revision 1.2  2004/01/14 22:29:23  michalis
 * + UserCommand.EndGameException, "debug players" i "samobojstwo" w UserCommand,
 *   JamySingle much cleaner now
 *
 * Revision 1.1  2004/01/13 04:39:00  michalis
 * + JamySingle (single-user version of game), LinkToLocationList, UserCommand
 *
 */
